@@ -165,9 +165,11 @@ APPKIT_EXPORT NSString *GSScreenNumber;
 - (void) hidecursor;
 - (void) showcursor;
 - (void) standardcursor: (int) style : (void**)cid;
-- (void) imagecursor: (NSPoint)hotp : (int)w : (int)h : (int)colors
-		    : (const unsigned char *) image : (void**)cid;
+- (void) imagecursor: (NSPoint)hotp : (NSImage *) image : (void**)cid;
 - (void) setcursorcolor: (NSColor *)fg : (NSColor *)bg : (void*)cid;
+- (void) recolorcursor: (NSColor *)fg : (NSColor *)bg : (void*) cid;
+- (void) setcursor: (void*) cid;
+- (void) freecursor: (void*) cid;
 - (void) setParentWindow: (int)parentWin 
           forChildWindow: (int)childWin;
 
