@@ -83,12 +83,14 @@ typedef struct
   is necessary, the mapping will have to be range to range. (Eg. if you
   have characters 'ab' mapped to glyphs 'AB', reordered to 'BA', then the
   range 'ab' will be mapped to the range 'BA'. */
-  unsigned int char_offset:21; /* This could be made a lot smaller, if necessary */
-
+  unsigned int char_offset:18; /* This could be made smaller, if necessary */
   unsigned int drawsOutsideLineFragment:1;
   unsigned int isNotShown:1;
+
   unsigned int inscription:3;
-  /* 3 unused */
+  unsigned int soft:1;
+  unsigned int elasitc:1;
+  unsigned int bidilevel:7; // Need to support 0..61
 } glyph_t;
 
 
