@@ -241,6 +241,7 @@
 @class NSMenuView;
 @class NSProgressIndicator;
 @class NSTableHeaderCell;
+@class NSTabViewItem;
 @class GSDrawTiles;
 
 /* First, declare names used for obtaining colors and/or tiles for specific
@@ -927,13 +928,68 @@ APPKIT_EXPORT	NSString	*GSThemeWillDeactivateNotification;
                     state: (int)inputState
                  andTitle: (NSString*)title;
 
-- (void) drawBrowserHeaderCell: (NSCell*)cell
+- (void) drawBrowserHeaderCell: (NSTableHeaderCell*)cell
 		     withFrame: (NSRect)rect
 			inView: (NSView*)view;
 
-- (NSRect) browserHeaderDrawingRectForCell: (NSCell*)cell
+- (NSRect) browserHeaderDrawingRectForCell: (NSTableHeaderCell*)cell
 				 withFrame: (NSRect)rect;
 
+- (void) drawTabViewRect: (NSRect)rect
+		  inView: (NSView *)view
+	       withItems: (NSArray *)items
+	    selectedItem: (NSTabViewItem *)item;
+
+- (void) drawScrollerRect: (NSRect)rect
+		   inView: (NSView *)view
+      		  hitPart: (NSScrollerPart)hitPart
+	     isHorizontal: (BOOL)isHorizontal;
+
+- (void) drawBrowserRect: (NSRect)rect
+		  inView: (NSView *)view
+	withScrollerRect: (NSRect)scrollerRect
+	      columnSize: (NSSize)columnSize;
+
+- (void) drawMenuRect: (NSRect)rect
+	       inView: (NSView *)view
+	 isHorizontal: (BOOL)horizontal
+	    itemCells: (NSArray *)itemCells;
+
+- (void) drawScrollViewRect: (NSRect)rect
+	             inView: (NSView *)view;
+
+- (void) drawBarInside: (NSRect)rect
+		inCell: (NSCell *)cell
+	       flipped: (BOOL)flipped;
+
+- (void) drawKnobInCell: (NSCell *)cell;
+
+- (NSRect) tableHeaderCellDrawingRectForBounds: (NSRect)theRect;
+							
+- (void) drawTableHeaderRect: (NSRect)aRect		
+		      inView: (NSView *)view;
+
+- (void) drawPopUpButtonCellInteriorWithFrame: (NSRect)cellFrame
+				     withCell: (NSCell *)cell
+				       inView: (NSView *)controlView;
+
+- (void) drawTableViewBackgroundInClipRect: (NSRect)clipRect
+				    inView: (NSView *)view
+		       withBackgroundColor: (NSColor *)backgroundColor;
+
+- (void) drawTableViewRect: (NSRect)aRect
+		    inView: (NSView *)view;
+
+- (void) drawTableViewGridInClipRect: (NSRect)aRect
+		      	      inView: (NSView *)view;
+
+- (void) highlightTableViewSelectionInClipRect: (NSRect)clipRect
+					inView: (NSView *)view
+			      selectingColumns: (BOOL)selectingColumns;
+
+- (void) drawTableViewRow: (int)rowIndex 
+		 clipRect: (NSRect)clipRect
+		   inView: (NSView *)view;
 @end
 
 /**
@@ -1055,6 +1111,31 @@ withRepeatedImage: (NSImage*)image
  */ 
 - (void)  setMenu: (NSMenu *)menu forWindow: (NSWindow *)window;
 
+/**
+ * Display the context menu when the right mouse button is pressed.
+ */
+- (void) rightMouseDisplay: (NSMenu *)menu
+                  forEvent: (NSEvent *)theEvent;
+
+/**
+ * Display popup menu item.
+ */
+- (void) displayPopUpMenu: (NSMenuView *)mr
+	    withCellFrame: (NSRect)cellFrame
+	controlViewWindow: (NSWindow *)cvWin
+	    preferredEdge: (NSRectEdge)edge
+	     selectedItem: (int)selectedItem;
+
+/**
+ * Process events for popups.
+ */
+- (BOOL) doesProcessEventsForPopUpMenu;
+
+/**
+ * Display the menu icon in the application.
+ */
+- (BOOL) menuShouldShowIcon;
+
 /**
  * Processes menu events for the theme.   The default implementation
  * does nothing.  
