@@ -39,7 +39,7 @@
 #define _GNUstep_H_NSApplication
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <AppKit/NSResponder.h>
+#import <AppKit/NSResponder.h>
 
 @class NSArray;
 @class NSAutoreleasePool;
@@ -145,6 +145,7 @@ APPKIT_EXPORT NSString	*NSEventTrackingRunLoopMode;
   NSMutableArray	*_hidden;
   NSMutableArray	*_inactive;
   NSWindow		*_hidden_key;
+  NSWindow              *_hidden_main;
   GSInfoPanel           *_infoPanel;
 
   /* This autorelease pool should only be created and used by -run, with
@@ -595,8 +596,8 @@ APPKIT_EXPORT NSString	*NSEventTrackingRunLoopMode;
 
 /**
  * Method used on OS X to allow an application to override the standard menu
- * obtained by right-clicking on the application's dock icon.  <em>Not sent
- * yet in GNUstep.</em>
+ * obtained by right-clicking on the application's dock icon.  <em>Called
+ * when the application uses Macintosh or Windows95 style menus.</em>
  */
 - (NSMenu *) applicationDockMenu: (NSApplication*)sender;
 
@@ -608,7 +609,7 @@ APPKIT_EXPORT NSString	*NSEventTrackingRunLoopMode;
  * method).  <em>Not sent yet under GNUstep.</em>
  */
 - (BOOL) applicationShouldHandleReopen: (NSApplication*)theApplication 
-		   hasVisibleWindows: (BOOL)flag;
+		     hasVisibleWindows: (BOOL)flag;
 
 /**
  * Called on OS X when the resolution or other characteristics of the display
