@@ -32,10 +32,10 @@
 #define _NSGraphicsContext_h_INCLUDE
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSObject.h>
-#include <Foundation/NSMapTable.h>
+#import <Foundation/NSObject.h>
+#import <Foundation/NSMapTable.h>
 
-#include <AppKit/AppKitDefines.h>
+#import <AppKit/AppKitDefines.h>
 
 @class NSDate;
 @class NSDictionary;
@@ -48,6 +48,7 @@
 @class NSFont;
 @class NSSet;
 @class NSBitmapImageRep;
+@class NSGradient;
 
 /*
  * Backing Store Types
@@ -60,11 +61,27 @@ typedef enum _NSBackingStoreType
 
 } NSBackingStoreType;
 
-/*
- * Compositing operators
+/** NSCompositingOperation defines how an image is drawn or composited onto another.
+ * <list>
+ *  <item>NSCompositeClear: Cleans out an area</item>
+ *  <item> NSCompositeCopy: <em>(Common)</em> Draws over the destination replacing it and carrying over transparency to the destination</item>
+ *  <item> NSCompositeSourceOver: <em>(Common)</em> Draws over the destination with transparency</item>
+ *  <item> NSCompositeSourceIn</item>
+ *  <item> NSCompositeSourceOut</item>
+ *  <item> NSCompositeSourceAtop</item>
+ *  <item> NSCompositeDestinationOver</item>
+ *  <item> NSCompositeDestinationIn</item>
+ *  <item> NSCompositeDestinationOut</item>
+ *  <item> NSCompositeDestinationAtop</item>
+ *  <item> NSCompositeXOR</item>
+ *  <item> NSCompositePlusDarker</item>
+ *  <item> NSCompositeHighlight</item>
+ *  <item> NSCompositePlusLighter</item>
+ * </list>
  */
 typedef enum _NSCompositingOperation
 {
+ 
   NSCompositeClear,
   NSCompositeCopy,
   NSCompositeSourceOver,
@@ -108,7 +125,7 @@ typedef enum _GSTextDrawingMode
 } GSTextDrawingMode;
 
 // We have to load this after the NSCompositingOperation are defined!!!
-#include <GNUstepGUI/GSMethodTable.h>
+#import <GNUstepGUI/GSMethodTable.h>
 
 /*
  * Window ordering
@@ -393,6 +410,12 @@ APPKIT_EXPORT NSGraphicsContext	*GSCurrentContext(void);
 	    fromRect: (NSRect)srcRect
 	   operation: (NSCompositingOperation)op
 	    fraction: (float)delta;
+- (BOOL) supportsDrawGState;
+- (void) GSdraw: (int)gstateNum
+        toPoint: (NSPoint)aPoint
+       fromRect: (NSRect)srcRect
+      operation: (NSCompositingOperation)op
+       fraction: (float)delta;
 - (void) GSDrawImage: (NSRect)rect : (void *)imageref;
 
 /* ----------------------------------------------------------------------- */
@@ -489,6 +512,21 @@ transform between current user space and image space for this image.</desc>
 
 @end
 
+@interface NSGraphicsContext (NSGradient)
+- (void) drawGradient: (NSGradient*)gradient
+           fromCenter: (NSPoint)startCenter
+               radius: (CGFloat)startRadius
+             toCenter: (NSPoint)endCenter 
+               radius: (CGFloat)endRadius
+              options: (NSUInteger)options;
+
+- (void) drawGradient: (NSGradient*)gradient
+            fromPoint: (NSPoint)startPoint
+              toPoint: (NSPoint)endPoint
+              options: (NSUInteger)options;
+
+@end
+
 /* NSGraphicContext constants */
 APPKIT_EXPORT NSString *NSGraphicsContextDestinationAttributeName;
 APPKIT_EXPORT NSString *NSGraphicsContextPDFFormat;
