@@ -30,8 +30,8 @@
 #define _GNUstep_H_NSKeyValueBinding
 
 #import <GNUstepBase/GSVersionMacros.h>
-#include <Foundation/NSObject.h>
-#include <AppKit/AppKitDefines.h>
+#import <Foundation/NSObject.h>
+#import <AppKit/AppKitDefines.h>
 
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
 
@@ -103,12 +103,17 @@ APPKIT_EXPORT id NSNotApplicableMarker;
 
 // Binding name constants
 APPKIT_EXPORT NSString *NSAlignmentBinding;
+APPKIT_EXPORT NSString *NSContentObjectBinding;
 APPKIT_EXPORT NSString *NSEditableBinding;
 APPKIT_EXPORT NSString *NSEnabledBinding;
 APPKIT_EXPORT NSString *NSFontBinding;
+APPKIT_EXPORT NSString *NSFontNameBinding;
+APPKIT_EXPORT NSString *NSFontSizeBinding;
 APPKIT_EXPORT NSString *NSHiddenBinding;
 APPKIT_EXPORT NSString *NSSelectedIndexBinding;
+APPKIT_EXPORT NSString *NSSelectedTagBinding;
 APPKIT_EXPORT NSString *NSTextColorBinding;
+APPKIT_EXPORT NSString *NSTitleBinding;
 APPKIT_EXPORT NSString *NSToolTipBinding;
 APPKIT_EXPORT NSString *NSValueBinding;
 
