@@ -31,13 +31,15 @@
 #define _GNUstep_H_NSPanel
 #import <GNUstepBase/GSVersionMacros.h>
 
-#include <AppKit/NSWindow.h>
+#import <AppKit/NSWindow.h>
 
 @class NSString;
 
 enum {
   NSUtilityWindowMask = 16,
-  NSDocModalWindowMask = 32
+  NSDocModalWindowMask = 32,
+  NSNonactivatingPanelMask = 128,
+  NSHUDWindowMask = 8192
 };
 
 enum {
