@@ -41,6 +41,7 @@
 @class NSTableHeaderView;
 @class NSText;
 @class NSImage;
+@class NSURL;
 
 typedef enum _NSTableViewDropOperation {
   NSTableViewDropOn,
@@ -137,8 +138,8 @@ typedef enum _NSTableViewColumnAutoresizingStyle
      tile at the end */
   BOOL _tilingDisabled;
 
-	unsigned int _draggingSourceOperationMaskForLocal;
-	unsigned int _draggingSourceOperationMaskForRemote;
+  NSDragOperation _draggingSourceOperationMaskForLocal;
+  NSDragOperation _draggingSourceOperationMaskForRemote;
 }
 
 /* Data Source */
@@ -314,7 +315,7 @@ typedef enum _NSTableViewColumnAutoresizingStyle
                              tableColumns: (NSArray*)cols
                                     event: (NSEvent*)event
                                    offset: (NSPoint*)offset;
-- (void) setDraggingSourceOperationMask: (unsigned int)mask
+- (void) setDraggingSourceOperationMask: (NSDragOperation)mask
                                forLocal: (BOOL)isLocal;
 #endif
 
@@ -338,6 +339,9 @@ typedef enum _NSTableViewColumnAutoresizingStyle
 
 @interface NSObject (NSTableDataSource)
 
+/**
+ * Returns the number of records that the data source manages for <em>aTableView</em>.
+ */
 - (int) numberOfRowsInTableView: (NSTableView *)aTableView;
 - (id) tableView: (NSTableView *)aTableView 
 objectValueForTableColumn: (NSTableColumn *)aTableColumn 
@@ -359,10 +363,19 @@ objectValueForTableColumn: (NSTableColumn *)aTableColumn
 - (BOOL) tableView: (NSTableView*)tableView
          writeRows: (NSArray*)rows
       toPasteboard: (NSPasteboard*)pboard;
+
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
+- (void) tableView: (NSTableView*)tableView
+  sortDescriptorsDidChange: (NSArray *)oldSortDescriptors;
+#endif
+
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
 - (BOOL) tableView: (NSTableView*)tableView
 writeRowsWithIndexes: (NSIndexSet*)rows
       toPasteboard: (NSPasteboard*)pboard;
+- (NSArray *) tableView: (NSTableView *)aTableView
+namesOfPromisedFilesDroppedAtDestination: (NSURL *)dropDestination
+forDraggedRowsWithIndexes: (NSIndexSet *)indexSet;
 #endif
 @end
 
@@ -379,8 +392,6 @@ APPKIT_EXPORT NSString *NSTableViewSelectionIsChangingNotification;
 
 - (BOOL) selectionShouldChangeInTableView: (NSTableView *)aTableView;
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
-- (void) tableView: (NSTableView*)tableView
-  sortDescriptorsDidChange: (NSArray *)oldSortDescriptors;
 - (void) tableView: (NSTableView*)tableView
 didClickTableColumn: (NSTableColumn *)tableColumn;
 - (void) tableView: (NSTableView*)tableView
