@@ -32,9 +32,10 @@
 
 #ifndef _GNUstep_H_NSView
 #define _GNUstep_H_NSView
+#import <GNUstepBase/GSVersionMacros.h>
 
-#include <AppKit/NSGraphicsContext.h>
-#include <AppKit/NSResponder.h>
+#import <AppKit/NSGraphicsContext.h>
+#import <AppKit/NSResponder.h>
 
 @class NSArray;
 @class NSAttributedString;
@@ -104,10 +105,14 @@ typedef enum _NSFocusRingType {
   id _matrixFromWindow;
 
   NSView* _super_view;
+PACKAGE_SCOPE
   NSMutableArray *_sub_views;
+@protected
   NSWindow *_window;
+PACKAGE_SCOPE
   NSMutableArray *_tracking_rects;
   NSMutableArray *_cursor_rects;
+@protected
   NSRect _invalidRect;
   NSRect _visibleRect;
   int _gstate;
@@ -119,14 +124,7 @@ typedef enum _NSFocusRingType {
    * Flags for internal use by NSView and it's subclasses.
    */
   struct _rFlagsType {
-    /*
-     * 'flipped_view' is set in NSViews designated initialiser (and other
-     * methods that create views) to be the value returned by [-isFlipped]
-     * This caching assumes that the value returned by [-isFlipped] will
-     * not change during the views lifetime - if it does, the view must
-     * be sure to change the flag accordingly.
-     */
-    unsigned	flipped_view:1;
+    unsigned	flipped_view:1;         /* Deprecated and unused.       */ 
     unsigned	has_subviews:1;		/* The view has subviews.	*/
     unsigned	has_currects:1;		/* The view has cursor rects.	*/
     unsigned	has_trkrects:1;		/* The view has tracking rects.	*/
@@ -194,8 +192,8 @@ typedef enum _NSFocusRingType {
 /*
  * Assigning a Tag
  */
-- (int) tag;
-- (id) viewWithTag: (int)aTag;
+- (NSInteger) tag;
+- (id) viewWithTag: (NSInteger)aTag;
 
 /*
  * Modifying the Frame Rectangle
@@ -241,6 +239,14 @@ typedef enum _NSFocusRingType {
 	      fromView: (NSView*)aView;
 - (NSSize) convertSize: (NSSize)aSize
 		toView: (NSView*)aView;
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+- (NSPoint) convertPointFromBase: (NSPoint)aPoint;
+- (NSPoint) convertPointToBase: (NSPoint)aPoint;
+- (NSRect) convertRectFromBase: (NSRect)aRect;
+- (NSRect) convertRectToBase: (NSRect)aRect;
+- (NSSize) convertSizeFromBase: (NSSize)aSize;
+- (NSSize) convertSizeToBase: (NSSize)aSize;
+#endif
 
 /*
  * Notifying Ancestor Views
