@@ -33,14 +33,12 @@
 #ifndef _GNUstep_H_GSLPRPrintPanel
 #define _GNUstep_H_GSLPRPrintPanel
 
-#include <AppKit/NSPrintPanel.h>
-
+#import "AppKit/NSPrintPanel.h"
 
 @interface GSLPRPrintPanel : NSPrintPanel
 {
 }
 
-
 @end
 
 #endif // _GNUstep_H_GSLPRPrintPanel
