@@ -27,19 +27,21 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include <Foundation/NSAutoreleasePool.h>
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSProcessInfo.h>
+#import <Foundation/NSAutoreleasePool.h>
+#import <Foundation/NSBundle.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSProcessInfo.h>
 
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSBitmapImageRep.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSGraphicsContext.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSView.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/DPSOperators.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSBitmapImageRep.h"
+#import "AppKit/NSNibLoading.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSGraphicsContext.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSView.h"
+#import "AppKit/NSWindow.h"
+#import "AppKit/DPSOperators.h"
 
 char **NSArgv = NULL;
 
@@ -50,6 +52,7 @@ int
 NSApplicationMain(int argc, const char **argv)
 {
   NSDictionary		*infoDict;
+  NSString              *mainModelFile;
   NSString		*className;
   Class			appClass;
   CREATE_AUTORELEASE_POOL(pool);
@@ -70,8 +73,20 @@ NSApplicationMain(int argc, const char **argv)
       NSLog(@"Bad application class '%@' specified", className);
       appClass = [NSApplication class];
     }
+  [appClass sharedApplication];
 
-  [[appClass sharedApplication] run];
+  mainModelFile = [infoDict objectForKey: @"NSMainNibFile"];
+  if (mainModelFile != nil && [mainModelFile isEqual: @""] == NO)
+    {
+      if ([NSBundle loadNibNamed: mainModelFile owner: NSApp] == NO)
+	{
+	  NSLog (_(@"Cannot load the main model file '%@'"), mainModelFile);
+	}
+    }
+
+  RECREATE_AUTORELEASE_POOL(pool);
+
+  [NSApp run];
 
   DESTROY(NSApp);
 
