@@ -213,7 +213,9 @@ GSCurrentServer(void)
 */
 - (id) initWithAttributes: (NSDictionary *)attributes
 {
-  [super init];
+  self = [super init];
+  if (nil == self)
+    return nil;
 
   server_info = [attributes mutableCopy];
   event_queue = [[NSMutableArray allocWithZone: [self zone]]
@@ -891,8 +893,7 @@ GSCurrentServer(void)
 /** Create a cursor from an image. Returns a pointer to the internal
     device representation that can be used later to make this cursor
     the current one */
-- (void) imagecursor: (NSPoint)hotp : (int)w : (int)h : (int) colors 
-		    : (const unsigned char *) image : (void**) cid
+- (void) imagecursor: (NSPoint)hotp : (NSImage *) image : (void**) cid
 {
   [self subclassResponsibility: _cmd];
 }
@@ -902,6 +903,28 @@ GSCurrentServer(void)
     background color bg. To keep the default color for the cursor, pass
     nil for fg and bg. */
 - (void) setcursorcolor: (NSColor *)fg : (NSColor *)bg : (void*) cid
+{
+  NSLog(@"Call to obsolete method -setcursorcolor:::");
+  [self recolorcursor: fg : bg : cid];
+  [self setcursor: cid];
+}
+
+/** Recolour the cursor given by the cid representation into having
+    a foreground color fg and a background color bg. */
+- (void) recolorcursor: (NSColor *)fg : (NSColor *)bg : (void*) cid
+{
+  [self subclassResponsibility: _cmd];
+}
+
+/** Set the cursor given by the cid representation as being the
+    current cursor. */
+- (void) setcursor: (void*) cid
+{
+  [self subclassResponsibility: _cmd];
+}
+
+/** Free the cursor given by the cid representation. */
+- (void) freecursor: (void*) cid
 {
   [self subclassResponsibility: _cmd];
 }
