@@ -141,15 +141,15 @@ static	GSDragView *sharedDragView = nil;
   if (self != nil)
     {
       NSRect winRect = {{0, 0}, {DWZ, DWZ}};
-      NSWindow *sharedDragWindow = [[isa windowClass] alloc];
+      NSWindow *sharedDragWindow = [[[isa windowClass] alloc]
+                                     initWithContentRect: winRect
+                                               styleMask: NSBorderlessWindowMask
+                                                 backing: NSBackingStoreNonretained
+                                                   defer: NO];
 
       dragCell = [[NSCell alloc] initImageCell: nil];
       [dragCell setBordered: NO];
       
-      [sharedDragWindow initWithContentRect: winRect
-				  styleMask: NSBorderlessWindowMask
-				    backing: NSBackingStoreNonretained
-				      defer: NO];
       [sharedDragWindow setContentView: self];
       // Kept alive by the window
       RELEASE(self);
@@ -180,7 +180,7 @@ static	GSDragView *sharedDragView = nil;
   return dragPasteboard;
 }
 
-- (int) draggingSequenceNumber
+- (NSInteger) draggingSequenceNumber
 {
   return dragSequence;
 }
@@ -190,7 +190,7 @@ static	GSDragView *sharedDragView = nil;
   return dragSource;
 }
 
-- (unsigned int) draggingSourceOperationMask
+- (NSDragOperation) draggingSourceOperationMask
 {
   // Mix in possible modifiers
   return dragMask & operationMask;
@@ -217,9 +217,18 @@ static	GSDragView *sharedDragView = nil;
   return loc;
 }
 
-- (NSArray *)namesOfPromisedFilesDroppedAtDestination:(NSURL *)dropDestination
+- (NSArray *) namesOfPromisedFilesDroppedAtDestination: (NSURL *)dropDestination
 {
-  return [[self draggingSource] namesOfPromisedFilesDroppedAtDestination: dropDestination];
+  if ([dragSource respondsToSelector:
+                    @selector(namesOfPromisedFilesDroppedAtDestination:)])
+    {
+      return [dragSource namesOfPromisedFilesDroppedAtDestination: 
+                           dropDestination];
+    }
+  else
+    {
+      return nil;
+    }
 }
 
 - (BOOL) isDragging
@@ -250,7 +259,7 @@ static	GSDragView *sharedDragView = nil;
 
   // Unset the target window  
   targetWindowRef = 0;
-  targetMask = NSDragOperationAll;
+  targetMask = NSDragOperationEvery;
   destExternal = NO;
 
   NSDebugLLog(@"NSDragging", @"Start drag with %@", [pboard types]);
@@ -423,7 +432,7 @@ static	GSDragView *sharedDragView = nil;
 - (BOOL) _updateOperationMask: (NSEvent*) theEvent
 {
   unsigned int mod = [theEvent modifierFlags];
-  unsigned int oldOperationMask = operationMask;
+  NSDragOperation oldOperationMask = operationMask;
 
   if (operationMask == NSDragOperationIgnoresModifiers)
     {
@@ -444,7 +453,7 @@ static	GSDragView *sharedDragView = nil;
     }
   else
     {
-      operationMask = NSDragOperationAll;
+      operationMask = NSDragOperationEvery;
     }
 
   return (operationMask != oldOperationMask);
@@ -477,7 +486,7 @@ static	GSDragView *sharedDragView = nil;
   NSCursor *newCursor;
   NSString *name;
   NSString *iname;
-  int       mask;
+  NSDragOperation mask;
 
   mask = dragMask & operationMask;
 
@@ -544,20 +553,20 @@ static	GSDragView *sharedDragView = nil;
 	}
     }
   
-  [newCursor set];
-
   if ((targetWindowRef != 0) && mask != NSDragOperationNone)
     {
-      [GSCurrentServer() setcursorcolor: [NSColor greenColor] 
+      [GSCurrentServer() recolorcursor: [NSColor greenColor] 
 		      : [NSColor blackColor] 
 		      : [newCursor _cid]];
     }
   else
     {
-      [GSCurrentServer() setcursorcolor: [NSColor blackColor] 
+      [GSCurrentServer() recolorcursor: [NSColor blackColor] 
 		      : [NSColor whiteColor] 
 		      : [newCursor _cid]];
     }
+
+  [newCursor set];
 }
 
 - (void) _sendLocalEvent: (GSAppKitSubtype)subtype
@@ -646,7 +655,17 @@ static	GSDragView *sharedDragView = nil;
       [self _updateOperationMask: theEvent];
     }
 
-  dragMask = [dragSource draggingSourceOperationMaskForLocal: !destExternal];
+
+  if ([dragSource respondsToSelector:
+                    @selector(draggingSourceOperationMaskForLocal:)])
+    {
+      dragMask = [dragSource draggingSourceOperationMaskForLocal: !destExternal];
+    }
+  else
+    {
+      dragMask = NSDragOperationCopy | NSDragOperationLink |
+        NSDragOperationGeneric | NSDragOperationPrivate;
+    }
   
   // --- Setup the event loop ------------------------------------------
   [self _updateAndMoveImageToCorrectPosition];
@@ -775,7 +794,7 @@ static	GSDragView *sharedDragView = nil;
           NSDebugLLog(@"NSDragging", @"got GSAppKitDraggingStatus\n");
           if ((int)[theEvent data1] == targetWindowRef)
             {
-              unsigned int newTargetMask = [theEvent data2];
+              NSDragOperation newTargetMask = (NSDragOperation)[theEvent data2];
 
               if (newTargetMask != targetMask)
                 {
@@ -874,7 +893,13 @@ static	GSDragView *sharedDragView = nil;
  
   //--- Move drag image to the new position -----------------------------------
   [self _moveDraggedImageToNewPosition];
-  
+
+  if ([dragSource respondsToSelector:
+              @selector(draggedImage:movedTo:)])
+    {
+      [dragSource draggedImage: [self draggedImage] movedTo: dragPosition];
+    }
+
   //--- Determine target window ---------------------------------------------
   destWindow = [self windowAcceptingDnDunder: dragPosition
                                    windowRef: &mouseWindowRef];
@@ -919,9 +944,19 @@ static	GSDragView *sharedDragView = nil;
   //  Reset drag mask when we switch from external to internal or back
   if (oldDestExternal != destExternal)
     {
-      unsigned int newMask;
+      NSDragOperation newMask;
+
+      if ([dragSource respondsToSelector:
+                        @selector(draggingSourceOperationMaskForLocal:)])
+        {
+          newMask = [dragSource draggingSourceOperationMaskForLocal: !destExternal];
+        }
+      else
+        {
+          newMask = NSDragOperationCopy | NSDragOperationLink |
+            NSDragOperationGeneric | NSDragOperationPrivate;
+        }
 
-      newMask = [dragSource draggingSourceOperationMaskForLocal: !destExternal];
       if (newMask != dragMask)
         {
           dragMask = newMask;
