@@ -46,6 +46,7 @@
 #import "AppKit/NSView.h"
 #import "AppKit/NSWindow.h"
 #import "GNUstepGUI/GSGormLoading.h"
+#import "NSDocumentFrameworkPrivate.h"
 
 static const int currentVersion = 1; // GSNibItem version number...
 
@@ -53,22 +54,6 @@ static const int currentVersion = 1; // GSNibItem version number...
 - (BOOL) isInInterfaceBuilder;
 @end
 
-@interface NSApplication (GSNibContainer)
-- (void)_deactivateVisibleWindow: (NSWindow *)win;
-@end
-
-@implementation NSApplication (GSNibContainer)
-/* Since awakeWithContext often gets called before the the app becomes
-   active, [win -orderFront:] requests get ignored, so we add the window
-   to the inactive list, so it gets sent an -orderFront when the app
-   becomes active. */
-- (void) _deactivateVisibleWindow: (NSWindow *)win
-{
-  if (_inactive)
-    [_inactive addObject: win];
-}
-@end
-
 /*
  * This private class is used to collect the nib items while the 
  * .gorm file is being unarchived.  This is done to allow only
@@ -157,8 +142,6 @@ static NSString *GSInternalNibItemAddedNotification = @"_GSInternalNibItemAddedN
 
       // Add these objects with there old names as the code expects them
       context = AUTORELEASE([context mutableCopyWithZone: [context zone]]);
-      obj = [context objectForKey: NSNibTopLevelObjects];
-      obj = [context objectForKey: NSNibOwner];
 
       isAwake = YES;
       /*
@@ -217,6 +200,16 @@ static NSString *GSInternalNibItemAddedNotification = @"_GSInternalNibItemAddedN
 	  [NSApp setWindowsMenu: menu];
 	}
 
+      /*
+       * Set the Recent Documents menu.
+       */
+      menu = [nameTable objectForKey: @"NSRecentDocumentsMenu"];
+      if (menu != nil && [menu isKindOfClass: [NSMenu class]] == YES)
+	{
+	  [[NSDocumentController sharedDocumentController]
+	      _setRecentDocumentsMenu: menu];
+	}
+
 
       /* 
        * See if the user has passed in the NSNibTopLevelObjects key.
@@ -298,10 +291,7 @@ static NSString *GSInternalNibItemAddedNotification = @"_GSInternalNibItemAddedN
 	  while (pos-- > 0)
 	    {
 	      NSWindow *win = [visibleWindows objectAtIndex: pos];
-	      if ([NSApp isActive])
-		[win orderFront: self];
-	      else
-		[NSApp _deactivateVisibleWindow: win];
+	      [win orderFront: self];
 	    }
 	}
 
@@ -554,7 +544,7 @@ static NSString *GSInternalNibItemAddedNotification = @"_GSInternalNibItemAddedN
 	{
 	  if(GSObjCIsKindOf(cls, [NSApplication class]))
 	    {
-	      obj = [cls sharedApplication];
+	      obj = RETAIN([cls sharedApplication]);
 	    }
 	  else
 	    {
