@@ -25,22 +25,23 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "GNUstepGUI/GSHorizontalTypesetter.h"
 
 #include <math.h>
 
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSGeometry.h>
-#include <Foundation/NSLock.h>
-#include <Foundation/NSValue.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSGeometry.h>
+#import <Foundation/NSLock.h>
+#import <Foundation/NSThread.h>
+#import <Foundation/NSValue.h>
 
-#include "AppKit/NSAttributedString.h"
-#include "AppKit/NSParagraphStyle.h"
-#include "AppKit/NSTextAttachment.h"
-#include "AppKit/NSTextContainer.h"
-#include "AppKit/NSTextStorage.h"
-#include "GNUstepGUI/GSLayoutManager.h"
+#import "AppKit/NSAttributedString.h"
+#import "AppKit/NSParagraphStyle.h"
+#import "AppKit/NSTextAttachment.h"
+#import "AppKit/NSTextContainer.h"
+#import "AppKit/NSTextStorage.h"
+#import "GNUstepGUI/GSLayoutManager.h"
+#import "GNUstepGUI/GSHorizontalTypesetter.h"
 
 
 
@@ -188,6 +189,10 @@ the last time or not, we wouldn't need to clear the cache every time */
 					atIndex: i
 					longestEffectiveRange: &paragraphRange
 					inRange: paragraphRange];
+      if (curParagraphStyle == nil)
+        {
+          curParagraphStyle = [NSParagraphStyle defaultParagraphStyle];
+        }
 
       curFont = [curLayoutManager effectiveFontForGlyphAtIndex: glyph
 				range: &fontRange];
@@ -643,7 +648,7 @@ restart: ;
 			     sweepDirection: NSLineSweepRight
 			     movementDirection: line_frags_num?NSLineDoesntMove:NSLineMoveDown
 			     remainingRect: &remain];
-      if (NSEqualRects(rect, NSZeroRect))
+      if (NSIsEmptyRect(rect))
         break;
 
       line_frags_num++;
@@ -1283,8 +1288,9 @@ NS_DURING
 
   *nextGlyphIndex = curGlyph;
 NS_HANDLER
-  [lock unlock];
-  [localException raise];
+  NSLog(@"GSHorizontalTypesetter - %@", [localException reason]);
+//  [lock unlock];
+//[localException raise];
   ret=0; /* This is never reached, but it shuts up the compiler. */
 NS_ENDHANDLER
   [lock unlock];
