@@ -26,24 +26,24 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSEnumerator.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSProcessInfo.h>
+#import <Foundation/NSBundle.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSProcessInfo.h>
 
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSButton.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSImageView.h"
-#include "AppKit/NSPasteboard.h"
-#include "AppKit/NSTextField.h"
-#include "GNUstepGUI/GSInfoPanel.h"
-#include "GNUstepGUI/GSTheme.h"
-#include "GSGuiPrivate.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSButton.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSImageView.h"
+#import "AppKit/NSPasteboard.h"
+#import "AppKit/NSTextField.h"
+#import "GNUstepGUI/GSInfoPanel.h"
+#import "GNUstepGUI/GSTheme.h"
+#import "GSGuiPrivate.h"
 
 static id
 value_from_info_plist_for_key (NSString *key)
@@ -398,7 +398,11 @@ new_label (NSString *value)
    * Create GUI Objects 
    */
   f = NSZeroRect;
-  f.size = [icon size];
+  if (icon != nil)
+    {
+      f.size = [icon size];
+    }
+
   iconButton = AUTORELEASE([[NSButton alloc] initWithFrame: f]); 
   [iconButton setImage: icon];
   [iconButton setBordered: NO];
@@ -477,7 +481,7 @@ new_label (NSString *value)
    */
 
   /** width **/
-  tmp_A = [icon size].width;
+  tmp_A = f.size.width;
   /* distance between icon and title */
   tmp_A += 10;
   /* compute the maximum of the following three sizes */
@@ -533,7 +537,7 @@ new_label (NSString *value)
      standard height of 48.  The code tries to be nice so that 50 or
      47 should more or less work -- but beware that 200 or 20 will
      *not* work. */
-  tmp_A = [icon size].height;
+  tmp_A = f.size.height;
 
   if (description)
     tmp_A += 10; 
@@ -691,7 +695,6 @@ new_label (NSString *value)
   f = [themeLabel frame];
   f.origin.x = (width - f.size.width) / 2;
   f.origin.y = tmp_b - 5 - f.size.height;
-  tmp_b = f.origin.y;
   [cv addSubview: themeLabel];
   [themeLabel setFrame: f];
   [themeLabel setTarget: [GSTheme class]];
