@@ -1,7 +1,7 @@
 /*
    GSLayoutManager.m
 
-   Copyright (C) 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 2002-2011 Free Software Foundation, Inc.
 
    Author: Alexander Malmberg <alexander@malmberg.org>
    Date: November 2002 - February 2003
@@ -25,24 +25,22 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSCharacterSet.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSEnumerator.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSValue.h>
-#include <GNUstepBase/Unicode.h>
+#import <Foundation/NSCharacterSet.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSValue.h>
 
-
-#include "AppKit/NSAttributedString.h"
-#include "AppKit/NSTextStorage.h"
-#include "AppKit/NSTextContainer.h"
+#import "AppKit/NSAttributedString.h"
+#import "AppKit/NSTextStorage.h"
+#import "AppKit/NSTextContainer.h"
 
 /* just for NSAttachmentCharacter */
-#include "AppKit/NSTextAttachment.h"
+#import "AppKit/NSTextAttachment.h"
 
-#include "GNUstepGUI/GSFontInfo.h"
-#include "GNUstepGUI/GSTypesetter.h"
-#include "GNUstepGUI/GSLayoutManager_internal.h"
+#import "GNUstepGUI/GSFontInfo.h"
+#import "GNUstepGUI/GSTypesetter.h"
+#import "GNUstepGUI/GSLayoutManager_internal.h"
 
 /* TODO: is using rand() here ok? */
 static int random_level(void)
@@ -65,7 +63,7 @@ static glyph_run_t *run_insert(glyph_run_head_t **context, int level)
   int i, size;
 
   size = sizeof(glyph_run_head_t) * level + sizeof(glyph_run_t);
-  h = objc_malloc(size);
+  h = malloc(size);
   memset(h, 0, size);
 
   for (i = level; i >= 0; i--, h++)
@@ -96,7 +94,7 @@ static void run_remove(glyph_run_head_t **context, glyph_run_t *r)
 
   // Free the glyphs
   if (r->glyphs)
-    objc_free(r->glyphs);
+    free(r->glyphs);
 
   h = &r->head;
   if (h->next)
@@ -106,7 +104,7 @@ static void run_remove(glyph_run_head_t **context, glyph_run_t *r)
     context[i]->next = context[i]->next->next;
 
   h -= r->level;
-  objc_free(h);
+  free(h);
 }
 
 /*
@@ -118,7 +116,7 @@ static inline void run_free_glyphs(glyph_run_t *r)
     {
       r->head.complete = 0;
       r->head.glyph_length = 0;
-      objc_free(r->glyphs);
+      free(r->glyphs);
       r->glyphs = NULL;
     }
  }
@@ -232,16 +230,16 @@ Private method used internally by GSLayoutManager for sanity checking.
     {
       next = (glyph_run_t *)cur->head.next;
       if (cur->glyphs)
-        objc_free(cur->glyphs);
+        free(cur->glyphs);
       [self _run_free_attributes: cur];
       // Find the start of the allocated memory
       h = &cur->head;
       h -= cur->level;
-      objc_free(h);
+      free(h);
     }
 
   // Free the head element
-  objc_free(glyphs);
+  free(glyphs);
   glyphs = NULL;
 }
 
@@ -255,7 +253,7 @@ Private method used internally by GSLayoutManager for sanity checking.
   glyph_run_head_t *h;
 
   size = sizeof(glyph_run_head_t) * (SKIP_LIST_DEPTH - 1) + sizeof(glyph_run_t);
-  glyphs = objc_malloc(size);
+  glyphs = malloc(size);
   memset(glyphs, 0, size);
 
   for (h = glyphs, i = SKIP_LIST_DEPTH; i; i--, h++)
@@ -748,10 +746,18 @@ Fills in all glyph holes up to last. only looking at levels below level
     return;
   length = [_textStorage length];
 
-  /* OPT: this can be done __much__ more efficiently */
   while (glyphs->glyph_length <= last && (glyphs->char_length < length || !glyphs->complete))
     {
-      [self _generateGlyphsUpToCharacter: glyphs->char_length];
+      // Make an estimate for the character position
+      unsigned int char_last;
+
+      if (glyphs->glyph_length == 0)
+        char_last = last;
+      else
+        char_last = glyphs->char_length + 1 +
+          (last - glyphs->glyph_length) * (glyphs->char_length / (glyphs->glyph_length + 1));
+
+      [self _generateGlyphsUpToCharacter: char_last];
     }
 }
 
@@ -843,7 +849,8 @@ Fills in all glyph holes up to last. only looking at levels below level
   glyph_run_t *r;
   unsigned int pos;
 
-  *isValidIndex = NO;
+  if (isValidIndex != NULL)
+    *isValidIndex = NO;
 
   /* glyph '-1' is returned in other places as an "invalid" marker; this
   way, we can say that it isn't valid without building all glyphs */
@@ -864,7 +871,9 @@ Fills in all glyph holes up to last. only looking at levels below level
   if (!r || !r->glyphs) /* shouldn't happen */
     return NSNullGlyph;
 
-  *isValidIndex = YES;
+  if (isValidIndex != NULL)
+    *isValidIndex = YES;
+
   return r->glyphs[glyphIndex - pos].g;
 }
 
@@ -1740,7 +1749,25 @@ places where we switch.
 - (int) intAttribute: (int)attributeTag
      forGlyphAtIndex: (unsigned int)glyphIndex
 {
-  [self subclassResponsibility: _cmd];
+  glyph_run_t *run;
+  glyph_t *g;
+  unsigned int pos;
+
+  run = run_for_glyph_index(glyphIndex, glyphs, &pos, NULL);
+  if (run && run->glyphs && (run->head.glyph_length < glyphIndex - pos))
+    {
+      g = &run->glyphs[glyphIndex - pos];
+
+      if (attributeTag == NSGlyphAttributeInscribe)
+        return g->inscription;
+      else if (attributeTag == NSGlyphAttributeSoft)
+        return g->soft;
+      else if (attributeTag == NSGlyphAttributeElastic)
+        return g->elasitc;
+      else if (attributeTag == NSGlyphAttributeBidiLevel)
+        return g->bidilevel;
+    }
+ 
   return 0;
 }
 
@@ -1768,12 +1795,12 @@ places where we switch.
 	  for (j = 0, lf = tc->linefrags; j < tc->num_linefrags + tc->num_soft; j++, lf++)
 	    {
 	      if (lf->points)
-		objc_free(lf->points);
+		free(lf->points);
 	      if (lf->attachments)
-		objc_free(lf->attachments);
+		free(lf->attachments);
 	    }
 
-	  objc_free(tc->linefrags);
+	  free(tc->linefrags);
 	}
       tc->linefrags = NULL;
       tc->num_linefrags = tc->num_soft = 0;
@@ -1860,12 +1887,12 @@ places where we switch.
             {
               if (lf->points)
                 {
-                  objc_free(lf->points);
+                  free(lf->points);
                   lf->points = NULL;
                 }
               if (lf->attachments)
                 {
-                  objc_free(lf->attachments);
+                  free(lf->attachments);
                   lf->attachments = NULL;
                 }
             }
@@ -1940,12 +1967,12 @@ places where we switch.
             {
               if (lf->points)
                 {
-                  objc_free(lf->points);
+                  free(lf->points);
                   lf->points = NULL;
                 }
               if (lf->attachments)
                 {
-                  objc_free(lf->attachments);
+                  free(lf->attachments);
                   lf->attachments = NULL;
                 }
             }
@@ -2158,7 +2185,7 @@ by calling this incorrectly.
       if (!tc->size_linefrags)
 	{
 	  tc->size_linefrags = 16;
-	  tc->linefrags = objc_malloc(sizeof(linefrag_t) * tc->size_linefrags);
+	  tc->linefrags = malloc(sizeof(linefrag_t) * tc->size_linefrags);
 	}
       tc->num_linefrags = 1;
       lf = tc->linefrags;
@@ -2168,7 +2195,7 @@ by calling this incorrectly.
       if (tc->size_linefrags <= tc->num_linefrags)
 	{
 	  tc->size_linefrags += tc->size_linefrags / 2;
-	  tc->linefrags = objc_realloc(tc->linefrags, sizeof(linefrag_t) * tc->size_linefrags);
+	  tc->linefrags = realloc(tc->linefrags, sizeof(linefrag_t) * tc->size_linefrags);
 	}
       tc->num_linefrags++;
       lf = &tc->linefrags[tc->num_linefrags - 1];
@@ -2182,12 +2209,12 @@ by calling this incorrectly.
 	    break;
 	  if (lf->points)
 	    {
-	      objc_free(lf->points);
+	      free(lf->points);
 	      lf->points = NULL;
 	    }
 	  if (lf->attachments)
 	    {
-	      objc_free(lf->attachments);
+	      free(lf->attachments);
 	      lf->attachments = NULL;
 	    }
 	}
@@ -2201,7 +2228,7 @@ by calling this incorrectly.
 	  if (tc->size_linefrags <= tc->num_linefrags + tc->num_soft)
 	    {
 	      tc->size_linefrags += tc->size_linefrags / 2;
-	      tc->linefrags = objc_realloc(tc->linefrags, sizeof(linefrag_t) * tc->size_linefrags);
+	      tc->linefrags = realloc(tc->linefrags, sizeof(linefrag_t) * tc->size_linefrags);
 	    }
 	  memmove(&tc->linefrags[tc->num_linefrags + 1], &tc->linefrags[tc->num_linefrags], tc->num_soft * sizeof(linefrag_t));
 	}
@@ -2277,7 +2304,7 @@ forStartOfGlyphRange: (NSRange)glyphRange
 			      __PRETTY_FUNCTION__];
 	  return;
 	}
-      lp = lf->points = objc_malloc(sizeof(linefrag_point_t));
+      lp = lf->points = malloc(sizeof(linefrag_point_t));
       lf->num_points++;
     }
   else
@@ -2291,7 +2318,7 @@ forStartOfGlyphRange: (NSRange)glyphRange
 	  return;
 	}
       lf->num_points++;
-      lf->points = objc_realloc(lf->points, sizeof(linefrag_point_t) * lf->num_points);
+      lf->points = realloc(lf->points, sizeof(linefrag_point_t) * lf->num_points);
       lp = &lf->points[lf->num_points - 1];
     }
   lp->pos = glyphRange.location;
@@ -2340,7 +2367,7 @@ forStartOfGlyphRange: (NSRange)glyphRange
 
   /* TODO: we do no sanity checking of attachment size ranges. might want
   to consider doing it */
-  lf->attachments = objc_realloc(lf->attachments,
+  lf->attachments = realloc(lf->attachments,
 		      sizeof(linefrag_attachment_t) * (lf->num_attachments + 1));
   la = &lf->attachments[lf->num_attachments++];
 
@@ -2624,7 +2651,7 @@ forStartOfGlyphRange: (NSRange)glyphRange
     [self _invalidateLayoutFromContainer: index];
 
   num_textcontainers++;
-  textcontainers = objc_realloc(textcontainers,
+  textcontainers = realloc(textcontainers,
 			 sizeof(textcontainer_t) * num_textcontainers);
 
   for (i = num_textcontainers - 1; i > index; i--)
@@ -2652,11 +2679,11 @@ forStartOfGlyphRange: (NSRange)glyphRange
     textcontainers[i] = textcontainers[i + 1];
 
   if (num_textcontainers)
-    textcontainers = objc_realloc(textcontainers,
+    textcontainers = realloc(textcontainers,
 			   sizeof(textcontainer_t) * num_textcontainers);
   else
     {
-      objc_free(textcontainers);
+      free(textcontainers);
       textcontainers = NULL;
     }
 
@@ -2849,7 +2876,7 @@ forStartOfGlyphRange: (NSRange)glyphRange
   int i;
   textcontainer_t *tc;
 
-  objc_free(rect_array);
+  free(rect_array);
   rect_array_size = 0;
   rect_array = NULL;
 
@@ -2858,7 +2885,7 @@ forStartOfGlyphRange: (NSRange)glyphRange
     {
       [tc->textContainer release];
     }
-  objc_free(textcontainers);
+  free(textcontainers);
   textcontainers = NULL;
 
   [self _freeGlyphs];
@@ -3128,12 +3155,12 @@ forStartingGlyphAtIndex: (NSUInteger)glyph
   len = glyph - gpos + length;
   if (!run->glyphs)
     {
-      run->glyphs = objc_malloc(sizeof(glyph_t) * len);
+      run->glyphs = malloc(sizeof(glyph_t) * len);
       memset(run->glyphs, 0, sizeof(glyph_t) * len);
     }
   else if (run->head.glyph_length < len)
     {
-      run->glyphs = objc_realloc(run->glyphs, sizeof(glyph_t) * len);
+      run->glyphs = realloc(run->glyphs, sizeof(glyph_t) * len);
       memset(&run->glyphs[glyph - gpos], 0, sizeof(glyph_t) * length);
     }
   run->head.glyph_length = len;
@@ -3165,9 +3192,25 @@ forStartingGlyphAtIndex: (NSUInteger)glyph
                    value: (NSInteger)anInt
          forGlyphAtIndex: (NSUInteger)glyphIndex
 {
-  // FIXME
-  [self subclassResponsibility: _cmd];
-}
+  glyph_run_t *run;
+  glyph_t *g;
+  unsigned int pos;
+
+  run = run_for_glyph_index(glyphIndex, glyphs, &pos, NULL);
+  if (run && run->glyphs && (run->head.glyph_length < glyphIndex - pos))
+    {
+      g = &run->glyphs[glyphIndex - pos];
+
+      if (attributeTag == NSGlyphAttributeInscribe)
+        g->inscription = anInt;
+      else if (attributeTag == NSGlyphAttributeSoft)
+        g->soft = anInt;
+      else if (attributeTag == NSGlyphAttributeElastic)
+        g->elasitc = anInt;
+      else if (attributeTag == NSGlyphAttributeBidiLevel)
+        g->bidilevel = anInt;
+    }
+ }
 
 /*
  * NSCoding protocol
