@@ -56,7 +56,7 @@
 #define GSI_ARRAY_RETAIN(A, X)	RETAIN((X).obj)
 #define GSI_ARRAY_RELEASE(A, X)	RELEASE((X).obj)
 #define GSI_ARRAY_TYPES GSUNION_OBJ
-#import <GNUstepBase/GSIArray.h>
+#include <GNUstepBase/GSIArray.h>
 
 #import "GNUstepGUI/GSNibLoading.h"
 #import "AppKit/NSApplication.h"
@@ -458,8 +458,7 @@ static BOOL _isInInterfaceBuilder = NO;
                       initWithContentRect: _windowRect
                       styleMask: _windowStyle
                       backing: _backingStoreType
-                      defer: _flags.isDeferred
-                      screen: nil];
+                      defer: _flags.isDeferred];
       
       // set flags...
       [_realObject setHidesOnDeactivate: _flags.isHiddenOnDeactivate];
@@ -472,7 +471,7 @@ static BOOL _isInInterfaceBuilder = NO;
       // [_realObject setAutoPositionMask: _flags.autoPositionMask]; // currently not implemented for nibs
       // [_realObject setAutoPosition: _flags.autoPosition];
       [_realObject setDynamicDepthLimit: _flags.dynamicDepthLimit];
-      [_realObject setFrameAutosaveName: _autosaveName];
+      // [_realObject setFrameAutosaveName: _autosaveName]; // done after setting the min/max sizes
 
       // reset attributes...
       [_realObject setContentView: _view];
@@ -482,10 +481,13 @@ static BOOL _isInInterfaceBuilder = NO;
 
       [_view _fixSubviews];
 
+      // FIXME What is the point of calling -setFrame:display: here? It looks
+      // like an effective no op to me.
       // resize the window...
       [_realObject setFrame: [NSWindow frameRectForContentRect: [self windowRect] 
                                        styleMask: [self windowStyle]]
                    display: NO];
+      [_realObject setFrameAutosaveName: _autosaveName];
     } 
   return _realObject;
 }
@@ -1036,42 +1038,40 @@ static BOOL _isInInterfaceBuilder = NO;
 
 - (id) nibInstantiate
 {
-  Class aClass;
-  
   if ([NSClassSwapper isInInterfaceBuilder])
     {
       _view = self;
       return self;
     }
-  else
+
+  if (_view == nil)
     {
-      aClass = NSClassFromString(_className);
-    }
+      Class aClass;
   
-  // If the class name is nil, assume NSView.
-  if (_className == nil)
-    {
-      aClass = [NSView class];
-    }
+      // If the class name is nil, assume NSView.
+      if (_className == nil)
+        {
+          aClass = [NSView class];
+        }
+      else
+        {
+          aClass = NSClassFromString(_className);
+        }
   
-  if (aClass == nil)
-    {
-      [NSException raise: NSInternalInconsistencyException
-		   format: @"Unable to find class '%@'", _className];
-    }
-  else
-    {
-      _view = [[aClass allocWithZone: NSDefaultMallocZone()] initWithFrame: [self frame]];
+      if (aClass == nil)
+        {
+          [NSException raise: NSInternalInconsistencyException
+                      format: @"Unable to find class '%@'", _className];
+        }
+      else
+        {
+          _view = [[aClass allocWithZone: NSDefaultMallocZone()] initWithFrame: [self frame]];
+        }
     }
 
   return _view;
 }
 
-- (id) awakeAfterUsingCoder:  (NSCoder *)coder
-{
-  return _view;
-}
-
 - (id) nibInstantiateWithCoder: (NSCoder *)coder
 {
   if ([NSClassSwapper isInInterfaceBuilder])
@@ -1135,46 +1135,46 @@ static BOOL _isInInterfaceBuilder = NO;
   if ([NSClassSwapper isInInterfaceBuilder])
     {
       self = [super initWithCoder: coder];
+      if (self == nil)
+        {
+          return nil;
+        }
     }
 
-  if (self != nil)
+  if ([coder allowsKeyedCoding])
     {
-      if ([coder allowsKeyedCoding])
+      // get the super stuff without calling super...
+      if ([coder containsValueForKey: @"NSFrame"])
         {
-	  // get the super stuff without calling super...
-	  if ([coder containsValueForKey: @"NSFrame"])
-	    {
-	      _frame = [coder decodeRectForKey: @"NSFrame"];
-	    }
-	  else
-	    {
-	      _frame = NSZeroRect;
-	      if ([coder containsValueForKey: @"NSFrameSize"])
-		{
-		  _frame.size = [coder decodeSizeForKey: @"NSFrameSize"];
-		}
-	    }
-	  
-          ASSIGN(_className, [coder decodeObjectForKey: @"NSClassName"]);
-          ASSIGN(_extension, [coder decodeObjectForKey: @"NSExtension"]);
-
-	  if ([self nibInstantiate] != nil)
-	    {
-	      [self nibInstantiateWithCoder: coder];
-	    }
-
-	  if (self != _view)
-	    {
-	      AUTORELEASE(self);
-	      [(NSKeyedUnarchiver *)coder replaceObject: self withObject: _view];
-	    }
+          _frame = [coder decodeRectForKey: @"NSFrame"];
         }
       else
         {
-          [NSException raise: NSInvalidArgumentException 
-                       format: @"Can't decode %@ with %@.",NSStringFromClass([self class]),
-                       NSStringFromClass([coder class])];
+          _frame = NSZeroRect;
+          if ([coder containsValueForKey: @"NSFrameSize"])
+            {
+              _frame.size = [coder decodeSizeForKey: @"NSFrameSize"];
+            }
         }
+      
+      ASSIGN(_className, [coder decodeObjectForKey: @"NSClassName"]);
+      ASSIGN(_extension, [coder decodeObjectForKey: @"NSExtension"]);
+      
+      if ([self nibInstantiate] != nil)
+        {
+          [self nibInstantiateWithCoder: coder];
+        }
+      
+      if (self != _view)
+        {
+          AUTORELEASE(self);
+        }
+    }
+  else
+    {
+      [NSException raise: NSInvalidArgumentException 
+                  format: @"Can't decode %@ with %@.",NSStringFromClass([self class]),
+                   NSStringFromClass([coder class])];
     }
 
   return _view;
@@ -1474,6 +1474,7 @@ static BOOL _isInInterfaceBuilder = NO;
       Class newCellClass = [newClass cellClass];
       if (newCellClass != [NSCell class])
         {
+          RELEASE(obj);
           result = [[newCellClass alloc] initWithCoder: coder];      
         }
     }
@@ -1732,6 +1733,8 @@ static BOOL _isInInterfaceBuilder = NO;
 				NSObjectMapValueCallBacks, 2);
       _objects = NSCreateMapTable(NSObjectMapKeyCallBacks,
 				  NSObjectMapValueCallBacks, 2);
+      _oids = NSCreateMapTable(NSObjectMapKeyCallBacks,
+			       NSObjectMapValueCallBacks, 2);
       
       // 
       // Get the maps.  There is no need to retain these, 
@@ -1750,6 +1753,10 @@ static BOOL _isInInterfaceBuilder = NO;
 	[coder decodeObjectForKey: @"NSObjectsKeys"];
       objectsValues = (NSArray *)
 	[coder decodeObjectForKey: @"NSObjectsValues"];
+      oidsKeys = (NSArray *)
+	[coder decodeObjectForKey: @"NSOidsKeys"];
+      oidsValues = (NSArray *)
+	[coder decodeObjectForKey: @"NSOidsValues"];
 
       // Fill in the maps...
       [self _buildMap: _classes 
@@ -1761,6 +1768,9 @@ static BOOL _isInInterfaceBuilder = NO;
       [self _buildMap: _objects 
 	    withKeys: objectsKeys 
 	    andValues: objectsValues];
+      [self _buildMap: _oids 
+	    withKeys: oidsKeys 
+	    andValues: oidsValues];
       
       //
       // Only get these maps when in the editor.  They
@@ -1770,10 +1780,6 @@ static BOOL _isInInterfaceBuilder = NO;
       if ([NSClassSwapper isInInterfaceBuilder])
 	{
 	  // Only get these when in the editor...
-	  oidsKeys = (NSArray *)
-	    [coder decodeObjectForKey: @"NSOidsKeys"];
-	  oidsValues = (NSArray *)
-	    [coder decodeObjectForKey: @"NSOidsValues"];
 	  accessibilityOidsKeys = (NSArray *)
 	    [coder decodeObjectForKey: @"NSAccessibilityOidsKeys"];
 	  accessibilityOidsValues = (NSArray *)
@@ -1781,14 +1787,9 @@ static BOOL _isInInterfaceBuilder = NO;
 
 	  _accessibilityOids = NSCreateMapTable(NSObjectMapKeyCallBacks,
 						NSObjectMapValueCallBacks, 2);	  
-	  _oids = NSCreateMapTable(NSObjectMapKeyCallBacks,
-				   NSObjectMapValueCallBacks, 2);
 	  [self _buildMap: _accessibilityOids 
 		withKeys: accessibilityOidsKeys 
 		andValues: accessibilityOidsValues];
-	  [self _buildMap: _oids 
-		withKeys: oidsKeys 
-		andValues: oidsValues];
 	}
 
       // instantiate...
@@ -1844,10 +1845,10 @@ static BOOL _isInInterfaceBuilder = NO;
   NSFreeMapTable(_objects);
   NSFreeMapTable(_names);
   NSFreeMapTable(_classes);
+  NSFreeMapTable(_oids);
   // these are not allocated when not in interface builder.
   if ([NSClassSwapper isInInterfaceBuilder])
     {
-      NSFreeMapTable(_oids);
       NSFreeMapTable(_accessibilityOids);
     }
 
@@ -1888,6 +1889,39 @@ static BOOL _isInInterfaceBuilder = NO;
   // set the new root object.
   [_root setRealObject: owner];
 
+  // iterate over all objects, instantiate them and fill in top level array.
+  /* Note: We instantiate all objects before establishing any connections
+     between them, so that any shared instances defined in the nib are
+     initialized before being used. This sequence is important when, e.g.,
+     the nib defines a shared document controller that is an instance of a
+     subclass of NSDocumentController. */
+  objs = NSAllMapTableKeys(_objects);
+  en = [objs objectEnumerator];
+  while ((obj = [en nextObject]) != nil)
+    {
+      id v = NSMapGet(_objects, obj);
+      NSInteger oid = [(id)NSMapGet(_oids, obj) intValue];
+
+      obj = [self instantiateObject: obj];
+      // Object is top level if it isn't the owner but points to it.
+      /* Don't record proxy objects in the top level array. The only
+	 reliable way to identify proxy objects seems to look at their
+	 object ID. Apparently, Apple is using fixed negative IDs for
+	 proxy objects (-1 = File's Owner, -2 = First Responder,
+	 -3 = NSApplication). */
+      if (oid >= 0)
+	{
+	  if ((v == owner || v == _root) && (obj != owner) && (obj != _root))
+	    {
+	      [topLevelObjects addObject: obj];
+	      // All top level objects must be released by the caller to avoid
+	      // leaking, unless they are going to be released by other nib
+	      // objects on behalf of the owner.
+	      RETAIN(obj);
+	    }
+        }
+    }
+
   // iterate over connections, instantiate and then establish them.
   en = [_connections objectEnumerator];
   while ((obj = [en nextObject]) != nil)
@@ -1899,30 +1933,29 @@ static BOOL _isInInterfaceBuilder = NO;
         }
     }
 
-  // iterate over all objects, instantiate, awaken objects and fill
-  // in top level array.
-  // We use the _names map not the _objects map here. This seems to give better results.
-  objs = NSAllMapTableKeys(_names);
+  // awaken all objects except proxy objects.
+  objs = NSAllMapTableKeys(_objects);
   en = [objs objectEnumerator];
   while ((obj = [en nextObject]) != nil)
     {
-      id v = NSMapGet(_objects, obj);
-      obj = [self instantiateObject: obj];
-      // Object is top level if it isn't the owner but points to it. 
-      if ((v == owner || v == _root) && (obj != owner) && (obj != _root))
+      NSInteger oid = [(id)NSMapGet(_oids, obj) intValue];
+      if (oid >= 0)
         {
-          [topLevelObjects addObject: obj];
-          // All top level objects must be released by the caller to avoid
-          // leaking, unless they are going to be released by other nib
-          // objects on behalf of the owner.
-          RETAIN(obj);
+          if ([obj respondsToSelector: @selector(realObject)])
+            {
+              obj = [obj realObject];
+            }
+          if ([obj respondsToSelector: @selector(awakeFromNib)])
+            {
+              [obj awakeFromNib];
+            }
         }
+    }
 
-      // awaken the object.
-      if ([obj respondsToSelector: @selector(awakeFromNib)])
-        {
-          [obj awakeFromNib];
-        }
+  // awaken the owner
+  if ([owner respondsToSelector: @selector(awakeFromNib)])
+    {
+      [owner awakeFromNib];
     }
 
   // bring visible windows to front...
