@@ -63,9 +63,8 @@
 #define	GSI_MAP_RELEASE_VAL(M, X)	
 #define	GSI_MAP_EQUAL(M, X,Y)	[(X).obj isEqualToDictionary: (Y).obj]
 #define GSI_MAP_KTYPES	GSUNION_OBJ
-#define GSI_MAP_VTYPES	GSUNION_INT
+#define GSI_MAP_VTYPES	GSUNION_NSINT
 #define	GSI_MAP_NOCLEAN	1
-
 #include <GNUstepBase/GSIMap.h>
 
 static NSDictionary	*blank;
@@ -195,11 +194,11 @@ cacheAttributes(NSDictionary *attrs)
        * in an immutable dictionary that can safely be cached.
        */
       attrs = [[NSDictionary alloc] initWithDictionary: attrs copyItems: NO];
-      GSIMapAddPair(&attrMap, (GSIMapKey)((id)attrs), (GSIMapVal)(unsigned)1);
+      GSIMapAddPair(&attrMap, (GSIMapKey)((id)attrs), (GSIMapVal)(NSUInteger)1);
     }
   else
     {
-      node->value.uint++;
+      node->value.nsu++;
       attrs = RETAIN(node->key.obj);
     }
   AUNLOCK();
@@ -220,7 +219,7 @@ unCacheAttributes(NSDictionary *attrs)
       node = GSIMapNodeForKeyInBucket(&attrMap, bucket, (GSIMapKey)((id)attrs));
       if (node != 0)
 	{
-	  if (--node->value.uint == 0)
+	  if (--node->value.nsu == 0)
 	    {
 	      GSIMapRemoveNodeFromMap(&attrMap, bucket, node);
 	      GSIMapFreeNode(&attrMap, node);
@@ -623,7 +622,7 @@ _attributesAtIndexEffectiveRange(
   return _textProxy;
 }
 
-- (NSDictionary*) attributesAtIndex: (unsigned)index
+- (NSDictionary*) attributesAtIndex: (NSUInteger)index
 		     effectiveRange: (NSRange*)aRange
 {
   unsigned dummy;
@@ -646,8 +645,10 @@ _attributesAtIndexEffectiveRange(
 - (void) setAttributes: (NSDictionary*)attributes
 		 range: (NSRange)range
 {
-  unsigned	tmpLength, arrayIndex, arraySize;
-  NSRange	effectiveRange;
+  unsigned	tmpLength;
+  unsigned      arrayIndex = 0;
+  unsigned      arraySize;
+  NSRange	effectiveRange = NSMakeRange(0, NSNotFound);
   NSRange	originalRange = range;
   unsigned	afterRangeLoc, beginRangeLoc;
   NSDictionary	*attrs;
@@ -692,7 +693,6 @@ SANITY();
 	  if (NSMaxRange(effectiveRange) > afterRangeLoc)
 	    {
 	      range.length = NSMaxRange(effectiveRange) - range.location;
-	      afterRangeLoc = NSMaxRange(range);
 	    }
 	}
       else if (effectiveRange.location > beginRangeLoc)
@@ -776,9 +776,10 @@ changeInLength: 0];
 - (void) replaceCharactersInRange: (NSRange)range
 		       withString: (NSString*)aString
 {
-  unsigned	tmpLength, arrayIndex, arraySize;
-  NSRange	effectiveRange;
-  NSDictionary	*attrs;
+  unsigned	tmpLength;
+  unsigned      arrayIndex = 0;
+  unsigned      arraySize;
+  NSRange	effectiveRange = NSMakeRange(0, NSNotFound);
   GSTextInfo	*info;
   int		moveLocations;
   unsigned	start;
@@ -821,8 +822,8 @@ SANITY();
     start = range.location - 1;
   else
     start = range.location;
-  attrs = _attributesAtIndexEffectiveRange(start, &effectiveRange,
-    tmpLength, _infoArray, &arrayIndex);
+  _attributesAtIndexEffectiveRange(start, &effectiveRange,
+                                   tmpLength, _infoArray, &arrayIndex);
 
   moveLocations = [aString length] - range.length;
 
@@ -872,8 +873,8 @@ SANITY();
    */
   if ((moveLocations + range.length) == 0)
     {
-      attrs = _attributesAtIndexEffectiveRange(start, &effectiveRange,
-        tmpLength, _infoArray, &arrayIndex);
+      _attributesAtIndexEffectiveRange(start, &effectiveRange,
+                                       tmpLength, _infoArray, &arrayIndex);
       arrayIndex++;
 
       if (effectiveRange.location == range.location
@@ -926,7 +927,7 @@ changeInLength: [aString length] - range.length];
 }
 
 // The superclass implementation is correct but too slow
-- (unsigned int) length
+- (NSUInteger) length
 {
   return [_textChars length];
 }
