@@ -637,7 +637,7 @@ typedef	struct {
   int	pos;
   int	end;
 
-  if (elementState < 0 || elementState > GSThemeSelectedState)
+  if (elementState > GSThemeSelectedState)
     {
       pos = 0;
       end = GSThemeSelectedState;
@@ -1120,7 +1120,7 @@ typedef	struct {
   int	pos;
   int	end;
 
-  if (elementState < 0 || elementState > GSThemeSelectedState)
+  if (elementState > GSThemeSelectedState)
     {
       pos = 0;
       end = GSThemeSelectedState;
@@ -1345,7 +1345,7 @@ typedef	struct {
        * signature of methodSignatureForSelector:, so we hack in
        * the signature required manually :-(
        */
-      if (sel_eq(aSelector, _cmd))
+      if (sel_isEqual(aSelector, _cmd))
 	{
 	  static	NSMethodSignature	*sig = nil;
 
