@@ -26,14 +26,14 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "AppKit/NSImageView.h"
-#include "AppKit/NSScrollView.h"
-#include "AppKit/NSTextField.h"
-#include "AppKit/NSTextContainer.h"
-#include "AppKit/NSTextView.h"
-#include "AppKit/NSWindow.h"
-#include "GSThemePrivate.h"
-#include "GSGuiPrivate.h"
+#import "AppKit/NSImageView.h"
+#import "AppKit/NSScrollView.h"
+#import "AppKit/NSTextField.h"
+#import "AppKit/NSTextContainer.h"
+#import "AppKit/NSTextView.h"
+#import "AppKit/NSWindow.h"
+#import "GSThemePrivate.h"
+#import "GSGuiPrivate.h"
 
 static NSTextField *
 new_label (NSString *value)
@@ -79,7 +79,6 @@ static GSThemeInspector	*sharedInspector = nil;
 - (id) init
 {
   NSRect	frame;
-  NSView	*content;
 
   frame.size = NSMakeSize(272,388);
   frame.origin = NSZeroPoint;
@@ -90,7 +89,7 @@ static GSThemeInspector	*sharedInspector = nil;
     defer: NO];
   
   [self setReleasedWhenClosed: NO];
-  content = [self contentView];
+
   return self;
 }
 
@@ -118,6 +117,7 @@ static GSThemeInspector	*sharedInspector = nil;
   iv = [[NSImageView alloc] initWithFrame: frame];
   [iv setImage: [[GSTheme theme] icon]];
   [content addSubview: iv];
+  RELEASE(iv);
 
   width = cFrame.size.width - 58;
   tf = new_label([theme name]);
@@ -159,6 +159,7 @@ static GSThemeInspector	*sharedInspector = nil;
       frame.origin.y = nameFrame.origin.y - frame.size.height - 25;
       [view setFrame: frame];
       [content addSubview: view];
+      RELEASE(view);
     }
 
   details = [[theme infoDictionary] objectForKey: @"GSThemeDetails"];
