@@ -44,6 +44,7 @@
 /* Customization View */
 @interface GSToolbarCustomizationView : NSView
 {
+  NSArray *_paletteItems;
 }
 
 - (void) setToolbarItems: (NSArray *)items;
@@ -54,10 +55,16 @@
 
 @implementation GSToolbarCustomizationView
 
+- (void)dealloc
+{
+  [_paletteItems release];
+  [super dealloc];
+}
+
 /* If the toolbar item has a custom view and the item is in use in the 
    toolbar, this view has already a superview. We need to make a copy of it 
    in order to be able to put it in the customization view. 
-   As a safety measure, we makes a copy of all toolbar items, thereby of all
+   As a safety measure, we make a copy of all toolbar items, thereby of all
    views. This ensures the toolbar items displayed in the palette don't 
    reference a toolbar. */
 - (NSArray *) paletteItemsWithToolbarItems: (NSArray *)items
@@ -66,7 +73,6 @@
   NSEnumerator *e = [items objectEnumerator];
   NSToolbarItem *item = nil;
 
-  e = [items objectEnumerator];
   while ((item = [e nextObject]) != nil)
     {
       NSToolbarItem *newItem = [item copy];
@@ -76,6 +82,7 @@
       [newItem setEnabled: YES];
       [newItem _layout];
       [paletteItems addObject: newItem];
+      RELEASE(newItem);
     }
 
   NSDebugLLog(DEBUG_LEVEL, @"Generated palette items %@ from toolbar items %@", 
@@ -93,7 +100,6 @@
   float vAccumulator = 0.0;
   NSEnumerator *e = [[self subviews] objectEnumerator];
   NSView *layoutedView = nil;
-  int lastRow = 0;
   int index = 0;
 
   // Loop over all subviews
@@ -125,12 +131,11 @@
           maxHeight -= vAccumulator;
           hAccumulator = 0.0;
           vAccumulator = height;
-          lastRow = index;
         }
 
       [layoutedView setFrameOrigin: NSMakePoint(hAccumulator, 
                                                 maxHeight - vAccumulator)];
-	  [layoutedView setAutoresizingMask:NSViewMinYMargin];
+      [layoutedView setAutoresizingMask:NSViewMinYMargin];
       hAccumulator += width;
       index++;
     }
@@ -146,10 +151,11 @@
 
 - (void) setToolbarItems: (NSArray *)items
 {
-  NSArray *paletteItems = [self paletteItemsWithToolbarItems: items];
   NSEnumerator *e;
   NSView *itemView;
   NSToolbarItem *item;
+  [_paletteItems release];
+  _paletteItems = [[self paletteItemsWithToolbarItems: items] retain];
 
   // Remove all old subviews
   e = [[self subviews] objectEnumerator];
@@ -159,8 +165,8 @@
     }
 
   NSDebugLLog(DEBUG_LEVEL, @"Will insert the views of toolbar items %@ in \
-    customization view", paletteItems);
-  e = [paletteItems objectEnumerator];
+    customization view", _paletteItems);
+  e = [_paletteItems objectEnumerator];
   while ((item = [e nextObject]) != nil)
     {
       itemView = [item _backView];
