@@ -27,32 +27,30 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include <Foundation/NSObject.h>
-#include <Foundation/NSArray.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSEnumerator.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSUserDefaults.h>
-#include <Foundation/NSString.h>
-#include "AppKit/NSButton.h"
-#include "AppKit/NSClipView.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSColorList.h"
-#include "AppKit/NSDragging.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSMenu.h"
-#include "AppKit/NSPasteboard.h"
+#import <Foundation/NSObject.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSUserDefaults.h>
+#import <Foundation/NSString.h>
+#import "AppKit/NSButton.h"
+#import "AppKit/NSClipView.h"
+#import "AppKit/NSDragging.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSMenu.h"
+#import "AppKit/NSPasteboard.h"
 // It contains GSMovableToolbarItemPboardType declaration
-#include "AppKit/NSToolbarItem.h"
-#include "AppKit/NSView.h"
-#include "AppKit/NSWindow.h"
+#import "AppKit/NSToolbarItem.h"
+#import "AppKit/NSView.h"
+#import "AppKit/NSWindow.h"
 
-#include "GNUstepGUI/GSTheme.h"
-#include "GNUstepGUI/GSToolbarView.h"
+#import "GNUstepGUI/GSTheme.h"
+#import "GNUstepGUI/GSToolbarView.h"
 
-#include "NSToolbarFrameworkPrivate.h"
+#import "NSToolbarFrameworkPrivate.h"
 
 typedef enum {
   ToolbarViewDefaultHeight = 62,
@@ -60,80 +58,16 @@ typedef enum {
   ToolbarViewSmallHeight = 52
 } ToolbarViewHeight;
 
+// Borrow this from  NSToolbarItem.m
+static const int InsetItemViewX = 10;
+
 static const int ClippedItemsViewWidth = 28;
 
 // Internal
 static const int current_version = 1;
-static NSColorList *SystemExtensionsColors;
 
 static int draggedItemIndex = NSNotFound;
 
-// Toolbar color extensions
-
-static void initSystemExtensionsColors(void)
-{
-  NSColor *toolbarBackgroundColor;
-  NSColor *toolbarBorderColor;
-  NSDictionary *colors;
-  
-  /* Set up a dictionary containing the names of all the system extensions 
-     colours as keys and with colours as values. */
-  toolbarBorderColor = [NSColor colorWithCalibratedRed: 0.5 
-                                                 green: 0.5 
-                                                  blue: 0.5 
-                                                 alpha: 1.0];
-  
-  // Window background color by tranparency
-  toolbarBackgroundColor = [NSColor clearColor]; 
-  
-  colors = [[NSDictionary alloc] initWithObjectsAndKeys: 
-    toolbarBackgroundColor, @"toolbarBackgroundColor",
-    toolbarBorderColor, @"toolbarBorderColor", nil];
-                                                             
-  SystemExtensionsColors = [NSColorList colorListNamed: @"System extensions"];
-  if (SystemExtensionsColors == nil)
-    {
-      SystemExtensionsColors = [[NSColorList alloc] initWithName: @"System extensions"];
-    }
-
-    {
-      NSEnumerator *e;
-      NSString *colorKey;
-      NSColor *color;
-      BOOL changed = NO;
-
-      // Set up default system extensions colors
-
-      e = [colors keyEnumerator];
-  
-      while ((colorKey = (NSString *)[e nextObject])) 
-        {
-          if ([SystemExtensionsColors colorWithKey: colorKey])
-            continue;
-
-          color = [colors objectForKey: colorKey];
-          [SystemExtensionsColors setColor: color forKey: colorKey];
-
-          changed = YES;
-        }
-
-      if (changed)
-        [SystemExtensionsColors writeToFile: nil];
-    }
-}
-
-@implementation NSColor (GSToolbarViewAdditions)
-+ (NSColor *) toolbarBackgroundColor
-{
-  return [SystemExtensionsColors colorWithKey: @"toolbarBackgroundColor"];
-}
-
-+ (NSColor *) toolbarBorderColor
-{
-  return [SystemExtensionsColors colorWithKey: @"toolbarBorderColor"]; 
-}
-@end
-
 /*
  * Toolbar related code
  */
@@ -271,7 +205,8 @@ static void initSystemExtensionsColors(void)
 + (void) initialize
 {
   if (self == [GSToolbarView class])
-    initSystemExtensionsColors();
+    {
+    }
 }
 
 - (id) initWithFrame: (NSRect)frame
@@ -457,12 +392,12 @@ static void initSystemExtensionsColors(void)
 
 - (NSDragOperation) draggingEntered: (id <NSDraggingInfo>)info
 {
-  return [self updateItemWhileDragging:info exited:NO];
+  return [self updateItemWhileDragging: info exited: NO];
 }
 
 - (NSDragOperation) draggingUpdated: (id <NSDraggingInfo>)info
 {
-  return [self updateItemWhileDragging:info exited:NO];
+  return [self updateItemWhileDragging: info exited: NO];
 }
 
 - (void) draggingEnded: (id <NSDraggingInfo>)info
@@ -472,7 +407,7 @@ static void initSystemExtensionsColors(void)
 
 - (void) draggingExited: (id <NSDraggingInfo>)info
 {
-  [self updateItemWhileDragging:info exited:YES];
+  [self updateItemWhileDragging: info exited: YES];
 }
 
 - (BOOL) prepareForDragOperation: (id <NSDraggingInfo>)info
@@ -484,7 +419,7 @@ static void initSystemExtensionsColors(void)
 {
   NSToolbar *toolbar = [self toolbar];
 
-  [self updateItemWhileDragging:info exited:NO];
+  [self updateItemWhileDragging: info exited: NO];
   
   draggedItemIndex = NSNotFound;
   
@@ -510,7 +445,7 @@ static void initSystemExtensionsColors(void)
 
 - (BOOL) isOpaque
 {
-  if ([[NSColor toolbarBackgroundColor] alphaComponent] < 1.0)
+  if ([[[GSTheme theme] toolbarBackgroundColor] alphaComponent] < 1.0)
     {
       return NO;
     }
@@ -614,19 +549,17 @@ static void initSystemExtensionsColors(void)
 
 - (void) _handleBackViewsFrame
 {
-  NSEnumerator *e = [[_toolbar items] objectEnumerator];
-  NSToolbarItem *item;
-  NSView *itemBackView;
-  NSRect itemBackViewFrame;
   float x = 0;
   float newHeight = 0;
-  // ---
   NSArray *subviews = [_clipView subviews];
+  NSEnumerator *e = [[_toolbar items] objectEnumerator];
+  NSToolbarItem *item;
   
-  //_heightFromLayout = 0;
-
   while ((item = [e nextObject]) != nil) 
     {
+      NSView *itemBackView;
+      NSRect itemBackViewFrame;
+
       itemBackView = [item _backView];
       if ([subviews containsObject: itemBackView] == NO
         || [item _isModified] 
@@ -645,7 +578,7 @@ static void initSystemExtensionsColors(void)
       x += [itemBackView frame].size.width;
       
       if (itemBackViewFrame.size.height > newHeight)
-       newHeight = itemBackViewFrame.size.height;
+        newHeight = itemBackViewFrame.size.height;
     }
     
   if (newHeight > 0)
@@ -655,91 +588,157 @@ static void initSystemExtensionsColors(void)
 - (void) _takeInAccountFlexibleSpaces
 {
   NSArray *items = [_toolbar items];
-  NSEnumerator *e = [items objectEnumerator];
+  NSEnumerator *e;
   NSToolbarItem *item;
-  NSView *backView; 
-  NSRect lastBackViewFrame;
-  float lengthAvailable;
-  unsigned int flexibleSpaceItemsNumber = 0;
+  NSView *backView, *view;
+  CGFloat lengthAvailable;
   BOOL mustAdjustNext = NO;
-  float x = 0;
+  CGFloat x = 0, visibleItemsMinWidth = 0, backViewsWidth = 0;
+  NSMutableArray *variableWidthItems = [NSMutableArray array];
+  int flexibleItemsCount = 0, maxWidthItemsCount = 0;
+  CGFloat spacePerFlexItem, extraSpace = 0;
+  CGFloat toolbarWidth = [self frame].size.width;
+  int i, n = [items count];
+  NSMutableArray *visibleItems = [NSMutableArray array];
+  static const int FlexItemWeight = 4; // non-space flexible item counts as much as 4 flexible spaces
   
-  if ([items count] == 0)
+  if (n == 0)
     return; 
   
-  lastBackViewFrame = [[[items lastObject] _backView] frame];
-  lengthAvailable = [self frame].size.width - NSMaxX(lastBackViewFrame);
+  // First determine which items can fit in toolbar if all are at their minimum width.
+  // We'd like to show as many items as possible. These are our visibleItems.
+  for (i=0; i < n; i++) 
+    {
+      item = [items objectAtIndex:i];
+      backView = [item _backView];
+      view = [item view];
+      if (view != nil)
+        backViewsWidth += [item minSize].width + 2*InsetItemViewX;
+      else
+        backViewsWidth += [backView frame].size.width;
+
+      if ((backViewsWidth + ClippedItemsViewWidth <= toolbarWidth)
+        || (i == n - 1 && backViewsWidth <= toolbarWidth))
+        {
+          visibleItemsMinWidth = backViewsWidth;
+          [visibleItems addObject:item];
+        }
+      else
+        {
+          break;
+        }
+    }
+  // next, figure out how much additional space there is for expanding flexible items
+  lengthAvailable = toolbarWidth - visibleItemsMinWidth;
+  if ([visibleItems count] < n)
+    lengthAvailable -= ClippedItemsViewWidth;
 
   if (lengthAvailable < 1)
     return;
-  
+ 
+  // We want to divide available space evenly among all flexible items, but some items may
+  // reach their maximum width, making more space available for the other items.
+  // To do this, first we count the flexible items, gathering a list of those that may
+  // have a maximum width.
+  // To match observed behavior on Cocoa (which is NOT as documented!) we allocate only 1/4
+  // as much space to flexible spaces as we do to other flexible items.
+  e = [visibleItems objectEnumerator];
   while ((item = [e nextObject]) != nil) 
-  {
-    if ([item _isFlexibleSpace])
     {
-      flexibleSpaceItemsNumber++;
+      if ([item _isFlexibleSpace])
+        {
+          flexibleItemsCount++;
+        }
+      else
+        {
+          CGFloat minWidth = [item minSize].width;
+          CGFloat maxWidth = [item maxSize].width;
+          if (minWidth < maxWidth)
+            {
+              [variableWidthItems addObject:item];
+              flexibleItemsCount += FlexItemWeight; // gets FlexItemWeight times the weight of a flexible space
+            }
+        }
     }
-  }
-  
-  if (lengthAvailable < flexibleSpaceItemsNumber)
-    return; 
+  if (flexibleItemsCount == 0)
+    return;
+    
+  // Now go through any variableWidthItems to see if the available space per item would
+  // cause any of them to exceed their maximum width, and calculate the extra space available
+  spacePerFlexItem = MAX(lengthAvailable / flexibleItemsCount, 0);
+  e = [variableWidthItems objectEnumerator];
+  while ((item = [e nextObject]) != nil)
+    {
+      CGFloat minWidth = [item minSize].width;
+      CGFloat maxWidth = [item maxSize].width;
+      if (maxWidth-minWidth < spacePerFlexItem * FlexItemWeight)
+        {
+          extraSpace += spacePerFlexItem * FlexItemWeight - (maxWidth-minWidth); // give back unneeded space
+          maxWidthItemsCount += FlexItemWeight;
+        }
+    }
+  // Recalculate spacePerFlexItem (unless all flexible items are going to their max width)
+  if (flexibleItemsCount > maxWidthItemsCount)
+    spacePerFlexItem += extraSpace / (flexibleItemsCount-maxWidthItemsCount);
   
+  // Finally, go through all items, adjusting their width and positioning them as needed
   e = [items objectEnumerator];
   while ((item = [e nextObject]) != nil)
   {
     backView = [item _backView];
     if ([item _isFlexibleSpace])
-    {
-      NSRect backViewFrame = [backView frame];
+      {
+        NSRect backViewFrame = [backView frame];
       
-      [backView setFrame: NSMakeRect(x, backViewFrame.origin.y,
-        lengthAvailable / flexibleSpaceItemsNumber, 
-        backViewFrame.size.height)];
-      mustAdjustNext = YES;
-    }
+        [backView setFrame: NSMakeRect(x, backViewFrame.origin.y,
+          spacePerFlexItem, 
+          backViewFrame.size.height)];        
+        mustAdjustNext = YES;
+      }
+    else if ([variableWidthItems indexOfObjectIdenticalTo:item] != NSNotFound)
+      {
+        NSRect backViewFrame = [backView frame];
+        CGFloat maxFlex = [item maxSize].width - [item minSize].width;
+        CGFloat flexAmount = MIN(maxFlex, spacePerFlexItem * FlexItemWeight);
+        CGFloat newWidth = [item minSize].width + flexAmount + 2 * InsetItemViewX;
+        [backView setFrame: NSMakeRect(x, backViewFrame.origin.y,
+          newWidth, 
+          backViewFrame.size.height)];
+        mustAdjustNext = YES;
+      }
     else if (mustAdjustNext)
-    {
-      NSRect backViewFrame = [backView frame];
+      {
+        NSRect backViewFrame = [backView frame];
       
-      [backView setFrame: NSMakeRect(x, backViewFrame.origin.y,
-        backViewFrame.size.width, backViewFrame.size.height)];
-    }
+        [backView setFrame: NSMakeRect(x, backViewFrame.origin.y,
+          backViewFrame.size.width, backViewFrame.size.height)];
+      }
+    view = [item view];
+    if (view != nil)
+      {
+        NSRect viewFrame = [view frame];
+        // Subtract InsetItemViewX
+        viewFrame.size.width = [backView frame].size.width - 2 * InsetItemViewX;
+        viewFrame.origin.x = InsetItemViewX;
+        [view setFrame: viewFrame];
+      }
     x += [backView frame].size.width;
   }
-  
 }
 
 - (void) _handleViewsVisibility
 {
-  NSArray *items = [_toolbar items];
-  
-  /* The back views which are associated with each toolbar item (the toolbar
-     items doesn't reflect the toolbar view content) */
-  NSArray *backViews = [items valueForKey: @"_backView"];
-  
-  /* The back views which will be visible in the toolbar view (when
-     _handleViewsVisibility will be terminated). */
-  NSArray *visibleBackViews;
-  
+  NSArray *backViews;
   NSArray *subviews;
   NSEnumerator *e;
   NSView *backView;
-  NSRect clipViewFrame;
   
-  // First, we resize
-  [self _handleBackViewsFrame];
-  [self _takeInAccountFlexibleSpaces];
-  
-  // Then we retrieve the back views which should be visible now that the resize
-  // process has been taken in account
-  visibleBackViews = [self _visibleBackViews];
-  
-  // ---
-     
+  /* The back views which are associated with each toolbar item (the toolbar
+     items doesn't reflect the toolbar view content) */
+  backViews = [[_toolbar items] valueForKey: @"_backView"];
+
   // We remove each back view associated with a removed toolbar item
- 
   e = [[_clipView subviews] objectEnumerator];
-  
   while ((backView = [e nextObject]) != nil) 
     {
       if ([backViews containsObject: backView] == NO)
@@ -750,10 +749,8 @@ static void initSystemExtensionsColors(void)
     }
       
   // We add each backView associated with an added toolbar item
-  
-  e = [backViews objectEnumerator];
   subviews = [_clipView subviews];
- 
+  e = [backViews objectEnumerator];
   while ((backView = [e nextObject]) != nil) 
   {
     if ([subviews containsObject: backView] == NO)
@@ -761,21 +758,22 @@ static void initSystemExtensionsColors(void)
         [_clipView addSubview: backView];
       }
   }
-  
-  // ---
-     
-  /* We manage the clipped items view in the case it should become visible or
-     invisible */
-  
-  clipViewFrame = [_clipView frame];  
-  
-  if ([visibleBackViews count] < [backViews count])
+}
+
+- (void) _manageClipView
+{
+  NSRect clipViewFrame = [_clipView frame];
+  int count = [[_toolbar items] count];
+  // Retrieve the back views which should be visible now that the resize
+  // process has been taken in account
+  NSArray *visibleBackViews = [self _visibleBackViews];
+
+  if ([visibleBackViews count] < count)
     {
       NSView *lastVisibleBackView = [visibleBackViews lastObject];
       float width = 0;
       
       // Resize the clip view
-      
       if (lastVisibleBackView != nil)
         width = NSMaxX([lastVisibleBackView frame]);  
       [_clipView setFrame: NSMakeRect(clipViewFrame.origin.x,
@@ -784,21 +782,19 @@ static void initSystemExtensionsColors(void)
                                       clipViewFrame.size.height)]; 
         
       // Adjust the clipped items mark frame handling   
-      
       [_clippedItemsMark layout];
-      
-      clipViewFrame = [_clipView frame]; // We get the new _clipView frame
+
+      // We get the new _clipView frame      
+      clipViewFrame = [_clipView frame];
       [_clippedItemsMark setFrameOrigin: NSMakePoint(
         [self frame].size.width - ClippedItemsViewWidth, clipViewFrame.origin.y)];
         
-      // ---
-        
       if ([_clippedItemsMark superview] == nil)       
         [self addSubview: _clippedItemsMark];  
       
     }
   else if (([_clippedItemsMark superview] != nil) 
-    && ([visibleBackViews count] == [backViews count])) 
+    && ([visibleBackViews count] == count)) 
     {      
       [_clippedItemsMark removeFromSuperview];
       
@@ -811,7 +807,15 @@ static void initSystemExtensionsColors(void)
 
 - (void) _reload 
 {  
+  // First, we resize
+  [self _handleBackViewsFrame];
+  [self _takeInAccountFlexibleSpaces];
+  
   [self _handleViewsVisibility]; 
+  /* We manage the clipped items view in the case it should become visible or
+     invisible */
+  [self _manageClipView];
+
   [self setNeedsDisplay: YES];
 }
 
@@ -842,28 +846,30 @@ static void initSystemExtensionsColors(void)
 - (NSArray *) _visibleBackViews 
 {
   NSArray *items = [_toolbar items];
-  NSView *backView;
+  NSView *backView, *view;
   int i, n = [items count];
   float backViewsWidth = 0, toolbarWidth = [self frame].size.width;
 
-  //[_visibleBackViews release];
-  NSMutableArray *_visibleBackViews = [NSMutableArray array];
+  NSMutableArray *visibleBackViews = [NSMutableArray array];
   
   for (i = 0; i < n; i++)
     {
-      backView = [[items objectAtIndex:i] _backView];
-  
-      backViewsWidth += [backView frame].size.width;
+      NSToolbarItem *item = [items objectAtIndex:i];
+      backView = [item _backView];
+      view = [item view];
+      if (view != nil)
+        backViewsWidth += [item minSize].width + 2*InsetItemViewX;
+      else
+        backViewsWidth += [backView frame].size.width;
 
       if ((backViewsWidth + ClippedItemsViewWidth <= toolbarWidth)
         || (i == n - 1 && backViewsWidth <= toolbarWidth))
         {
-          [_visibleBackViews addObject: backView];
+          [visibleBackViews addObject: backView];
         }     
     }
   
-  return _visibleBackViews;
-  
+  return visibleBackViews;
 }
 
 - (NSColor *) standardBackgroundColor
