@@ -190,12 +190,25 @@
               styleMask: aStyle];
 }
 
-#if 0
+/* If the contentView is removed from the window we must make sure the
+ * window no longer tries to access it.  This situation may occur, for
+ * example, when people create inspectors where they want to swap in
+ * and out views.  In the example I saw, a bunch of non-visible
+ * windows were created to create the inspector views.  When an
+ * inspector view was needed, it was added as a subview in the visible
+ * inspector window.  We need to make sure that when 'addSubview:' is
+ * called to add the view to another window, all references to it in
+ * the old window will automatically disappear (this is how it works
+ * on Apple too).
+ */
 - (void) removeSubview: (NSView*)aView
 {
   RETAIN(aView);
   /*
-   * If the content view is removed, we must let the window know.
+   * If the content view is removed (for example, because it was added
+   * to another view in another window), we must let the window know.
+   * Otherwise, it would keep trying to resize/manage it as if it was
+   * its content view, while it actually is now in another window!
    */
   [super removeSubview: aView];
   if (aView == [_window contentView])
@@ -204,7 +217,6 @@
     }
   RELEASE(aView);
 }
-#endif
 
 - (void) setBackgroundColor: (NSColor *)color
 {
@@ -242,13 +254,14 @@
       NSMenuView *menuView;
       GSTheme *theme = [GSTheme theme];
       float menuBarHeight = [theme menuHeightForWindow: _window];
-      
+      NSRect menuRect = 
+	NSMakeRect(contentViewFrame.origin.x,
+		   (NSMaxY(contentRect) + 1) - menuBarHeight,  
+		   contentViewFrame.size.width, 
+		   menuBarHeight);
+
       menuView = [[_window menu] menuRepresentation];
-      [menuView setFrame: NSMakeRect(
-              contentViewFrame.origin.x,
-              NSMaxY(contentViewFrame) - menuBarHeight, 
-              contentViewFrame.size.width, 
-              menuBarHeight)];
+      [menuView setFrame: menuRect];
       contentViewFrame.size.height -= menuBarHeight;
     }
 
@@ -256,7 +269,7 @@
     {
       GSToolbarView *tv = [tb _toolbarView];
       float newToolbarViewHeight;
-	  
+      NSRect toolbarRect;
       // If the width changed we may need to recalculate the height
       if (contentViewFrame.size.width != [tv frame].size.width)
 	{
@@ -265,10 +278,11 @@
 	  [tv _reload];
 	}
       newToolbarViewHeight = [tv _heightFromLayout];
-      [tv setFrame: NSMakeRect(contentViewFrame.origin.x,
+      toolbarRect = NSMakeRect(contentViewFrame.origin.x,
 			       NSMaxY(contentViewFrame) - newToolbarViewHeight,
 			       contentViewFrame.size.width, 
-			       newToolbarViewHeight)];
+			       newToolbarViewHeight);
+      [tv setFrame: toolbarRect];
       contentViewFrame.size.height -= newToolbarViewHeight;
     }
 
@@ -451,15 +465,15 @@
   float	menubarHeight = [[GSTheme theme] 
 			  menuHeightForWindow: 
 			    _window];
+  NSRect menuRect = NSMakeRect(contentRect.origin.x,
+			       (NSMaxY(contentRect) + 1) - menubarHeight,  
+			       contentRect.size.width, 
+			       menubarHeight);
   hasMenu = YES;
   // Plug the menu view
-  [menuView setFrame: NSMakeRect(
-          contentRect.origin.x,
-          NSMaxY(contentRect), 
-          contentRect.size.width, 
-          menubarHeight)];
+  [menuView setFrame: menuRect];
   [self addSubview: menuView];
-  
+
   [self changeWindowHeight: menubarHeight];  
 }
 
