@@ -42,18 +42,12 @@
 #import "AppKit/NSNibLoading.h"
 #import "GNUstepGUI/GSModelLoaderFactory.h"
 #import "GNUstepGUI/GSNibLoading.h"
+#import "GNUstepGUI/GSXibLoading.h"
 
 @interface NSApplication (NibCompatibility)
 - (void) _setMainMenu: (NSMenu*)aMenu;
 @end
 
-// Hack: This allows the class name FirstResponder in NSCustomObject and
-// correctly returns nil as the corresponding object.
-@interface FirstResponder: NSObject
-{
-}
-@end
-
 @implementation FirstResponder
 
 + (id) allocWithZone: (NSZone*)zone
@@ -63,13 +57,6 @@
 
 @end
 
-@interface IBClassDescriptionSource: NSObject
-{
-  NSString *majorKey;
-  NSString *minorKey;
-}
-@end
-
 @implementation IBClassDescriptionSource
 
 - (id) initWithCoder: (NSCoder*)coder
@@ -103,16 +90,6 @@
 
 @end
 
-@interface IBPartialClassDescription: NSObject
-{
-  NSString *className;
-  NSString *superclassName;
-  NSMutableDictionary *actions;
-  NSMutableDictionary *outlets;
-  IBClassDescriptionSource *sourceIdentifier;
-}
-@end
-
 @implementation IBPartialClassDescription
 
 - (id) initWithCoder: (NSCoder*)coder
@@ -161,12 +138,6 @@
 
 @end
 
-@interface IBClassDescriber: NSObject
-{
-  NSMutableArray *referencedPartialClassDescriptions;
-}
-@end
-
 @implementation IBClassDescriber
 
 - (id) initWithCoder: (NSCoder*)coder
@@ -195,16 +166,6 @@
 
 @end
 
-@interface IBConnection: NSObject <NSCoding>
-{
-  NSString *label;
-  id source;
-  id destination;
-}
-- (id) nibInstantiate;
-- (void) establishConnection;
-@end
-
 @implementation IBConnection
 
 - (id) initWithCoder: (NSCoder*)coder
@@ -266,11 +227,6 @@
 
 @end
 
-@interface IBActionConnection: IBConnection
-{
-}
-@end
-
 @implementation IBActionConnection
 
 - (void) establishConnection
@@ -283,11 +239,6 @@
 
 @end
 
-@interface IBOutletConnection: IBConnection
-{
-}
-@end
-
 @implementation IBOutletConnection
 
 - (void) establishConnection
@@ -337,13 +288,6 @@
 
 @end
 
-@interface IBConnectionRecord: NSObject
-{
-  IBConnection *connection;
-  int connectionID;
-}
-@end
-
 @implementation IBConnectionRecord
 
 - (id) initWithCoder: (NSCoder*)coder
@@ -387,15 +331,6 @@
 
 @end
 
-@interface IBObjectRecord: NSObject
-{
-  int objectID;
-  id object;
-  id children;
-  id parent;
-}
-@end
-
 @implementation IBObjectRecord
 
 - (id) initWithCoder: (NSCoder*)coder
@@ -436,12 +371,11 @@
   [super dealloc];
 }
 
-@end
-
-@interface IBMutableOrderedSet: NSObject
+- (id) object
 {
-  NSArray *orderedObjects;
+  return object;
 }
+
 @end
 
 @implementation IBMutableOrderedSet
@@ -469,20 +403,10 @@
   [super dealloc];
 }
 
-@end
-
-@interface IBObjectContainer: NSObject <NSCoding>
+- (NSArray *)orderedObjects
 {
-  NSMutableArray *connectionRecords;
-  IBMutableOrderedSet *objectRecords;
-  NSMutableDictionary *flattenedProperties;
-  NSMutableDictionary *unlocalizedProperties;
-  id activeLocalization;
-  NSMutableDictionary *localizations;
-  id sourceID;
-  int maxID;
+  return orderedObjects;
 }
-- (id) nibInstantiate;
 @end
 
 @implementation IBObjectContainer
@@ -543,35 +467,9 @@
   return self;
 }
 
-@end
-
-
-
-@interface GSXibElement: NSObject
+- (NSEnumerator *) objectRecordEnumerator
 {
-  NSString *type;
-  NSDictionary *attributes;
-  NSString *value;
-  NSMutableDictionary *elements;
-  NSMutableArray *values;
-}
-- (NSString*) type;
-- (NSString*) value;
-- (NSDictionary*) elements;
-- (NSArray*) values;
-- (void) addElement: (GSXibElement*)element;
-- (void) setElement: (GSXibElement*)element forKey: (NSString*)key;
-- (void) setValue: (NSString*)text;
-- (NSString*) attributeForKey: (NSString*)key;
-- (GSXibElement*) elementForKey: (NSString*)key;
-@end
-
-@interface GSXibKeyedUnarchiver: NSKeyedUnarchiver
-{
-  NSMutableDictionary *objects;
-  NSMutableArray *stack;
-  GSXibElement *currentElement;
-  NSMutableDictionary *decoded;
+  return [[objectRecords orderedObjects] objectEnumerator];
 }
 
 @end
@@ -632,8 +530,13 @@
           // add the menu...
           [NSApp _setMainMenu: obj];
         }
+    }
       
-      // awaken the object.
+  // awaken all objects.
+  en = [objects objectRecordEnumerator];
+  while ((obj = [en nextObject]) != nil)
+    {
+      obj = [obj object];
       if ([obj respondsToSelector: @selector(awakeFromNib)])
         {
           [obj awakeFromNib];
@@ -946,6 +849,8 @@ didStartElement: (NSString *)elementName
 
       // Create instance.
       o = [c allocWithZone: [self zone]];
+      // Make sure the object stays around, even when replaced.
+      RETAIN(o);
       if (key != nil)
         [decoded setObject: o forKey: key];
       r = [o initWithCoder: self];
@@ -954,7 +859,7 @@ didStartElement: (NSString *)elementName
 	  [delegate unarchiver: self
 	      willReplaceObject: o
 		     withObject: r];
-	  o = r;
+	  ASSIGN(o, r);
           if (key != nil)
             [decoded setObject: o forKey: key];
 	}
@@ -964,7 +869,7 @@ didStartElement: (NSString *)elementName
 	  [delegate unarchiver: self
 	      willReplaceObject: o
 		     withObject: r];
-	  o = r;
+	  ASSIGN(o, r);
           if (key != nil)
             [decoded setObject: o forKey: key];
 	}
@@ -976,23 +881,18 @@ didStartElement: (NSString *)elementName
 	      [delegate unarchiver: self
 		  willReplaceObject: o
 			 withObject: r];
-	      o = r;
+              ASSIGN(o, r);
               if (key != nil)
                 [decoded setObject: o forKey: key];
 	    }
 	}
-      if (key != nil)
-        {
-          // Retained in decoded
-          RELEASE(o);
-        }
-      else
-        AUTORELEASE(o);
+      // Balance the retain above
+      RELEASE(o);
 
       // pop
       currentElement = last;
 
-      return o;
+      return AUTORELEASE(o);
     }
   else if ([@"string" isEqualToString: elementName])
     {
