@@ -28,34 +28,35 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include <Foundation/NSArray.h>
-#include <Foundation/NSAutoreleasePool.h>
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSCharacterSet.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSError.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSFileManager.h>
-#include <Foundation/NSPathUtilities.h>
-#include <Foundation/NSRange.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSValue.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSAutoreleasePool.h>
+#import <Foundation/NSBundle.h>
+#import <Foundation/NSCharacterSet.h>
+#import <Foundation/NSData.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSError.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSFileManager.h>
+#import <Foundation/NSPathUtilities.h>
+#import <Foundation/NSRange.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSValue.h>
 
-#include "AppKit/NSAttributedString.h"
-#include "AppKit/NSDocumentController.h"
-#include "AppKit/NSParagraphStyle.h"
-#include "AppKit/NSPasteboard.h"
-#include "AppKit/NSTextAttachment.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSFileWrapper.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSFontManager.h"
+#import "AppKit/NSAttributedString.h"
+#import "AppKit/NSDocumentController.h"
+#import "AppKit/NSParagraphStyle.h"
+#import "AppKit/NSPasteboard.h"
+#import "AppKit/NSTextAttachment.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSFileWrapper.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSFontManager.h"
 // For the colour name spaces
-#include "AppKit/NSGraphics.h"
+#import "AppKit/NSGraphics.h"
 #import "AppKit/NSTextTable.h"
 
-#include "GNUstepGUI/GSTextConverter.h"
-#include "GSGuiPrivate.h"
+#import "GNUstepGUI/GSTextConverter.h"
+#import "GSGuiPrivate.h"
 
 /* Cache class pointers to avoid the expensive lookup by string. */ 
 static Class dictionaryClass = nil;
@@ -765,7 +766,37 @@ create_error(int code, NSString* desc)
 
   if (type == nil)
     {
-      // FIXME: try to determine type
+      // Make sure this buffer is long enough
+      char prefix[14];
+      NSUInteger l = [data length];
+      if (l < sizeof(prefix))
+	{
+	  [data getBytes: prefix length: l];
+	  prefix[l] = 0;
+	}
+      else
+	{
+	  [data getBytes: prefix length: sizeof(prefix)];
+	}
+
+      // The list of file types below was derived from Apache's conf/magic file
+      // FIXME extend the list
+      if (strncmp(prefix, "{\\rtf", 5) == 0)
+	{
+	  type = NSRTFTextDocumentType;
+	}
+      else if (strncasecmp(prefix, "<!doctype html", 14) == 0 ||
+	       strncasecmp(prefix, "<head", 5) == 0 ||
+	       strncasecmp(prefix, "<title", 6) == 0 ||
+	       strncasecmp(prefix, "<html", 5) == 0 ||
+	       strncmp(prefix, "<!--", 4) == 0 ||
+	       strncasecmp(prefix, "<h1", 3) == 0)
+	{
+	  type = NSHTMLTextDocumentType;
+	}
+    }
+  if (type == nil)
+    {
       if (error)
 	*error = create_error(0, NSLocalizedString(@"No type specified for data.",
 						   @"Error description"));
@@ -845,8 +876,14 @@ create_error(int code, NSString* desc)
 - (id) initWithURL: (NSURL *)url 
 documentAttributes: (NSDictionary **)dict
 {
-  NSDictionary *options = [NSDictionary dictionaryWithObject: [url baseURL]
-                                        forKey: NSBaseURLDocumentOption];
+  NSURL *baseURL = [url baseURL];
+  NSDictionary *options = nil;
+
+  if (baseURL != nil)
+    {
+      [NSDictionary dictionaryWithObject: baseURL
+				  forKey: NSBaseURLDocumentOption];
+    }
 
   return [self initWithURL: url
                options: options
@@ -859,6 +896,7 @@ documentAttributes: (NSDictionary **)dict
 documentAttributes: (NSDictionary **)dict
              error: (NSError **)error
 {
+  NSURL *baseURL;
   NSData *data = [url resourceDataUsingCache: YES];
 
   if (data == nil)
@@ -871,14 +909,18 @@ documentAttributes: (NSDictionary **)dict
     }
 
   // Pass on baseURL
-  if (options == nil)
-    options = [NSDictionary dictionaryWithObject: [url baseURL]
-					  forKey: NSBaseURLDocumentOption];
-  else if ([options objectForKey: NSBaseURLDocumentOption] == nil)
+  baseURL = [url baseURL];
+  if (baseURL != nil)
     {
-      options = AUTORELEASE([options mutableCopy]);
-      [(NSMutableDictionary*)options setObject: [url baseURL]
-                             forKey: NSBaseURLDocumentOption];
+      if (options == nil)
+	options = [NSDictionary dictionaryWithObject: baseURL
+					      forKey: NSBaseURLDocumentOption];
+      else if ([options objectForKey: NSBaseURLDocumentOption] == nil)
+	{
+	  options = AUTORELEASE([options mutableCopy]);
+	  [(NSMutableDictionary*)options setObject: baseURL
+					    forKey: NSBaseURLDocumentOption];
+	}
     }
 
   return [self initWithData: data
