@@ -3,7 +3,7 @@
 
    Methods for loading .png images.
 
-   Copyright (C) 2003 Free Software Foundation, Inc.
+   Copyright (C) 2003-2011 Free Software Foundation, Inc.
    
    Written by: Alexander Malmberg <alexander@malmberg.org>
    Date: 2003-12-07
@@ -27,21 +27,13 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "config.h"
-#include "NSBitmapImageRep+PNG.h"
+#import "config.h"
 
-#if HAVE_LIBPNG
+/* we include PNG stuff only if required and before the resto to avoid header and
+   redeclaration problems (setjmp, etc) */
+#ifdef HAVE_LIBPNG
 
-#ifdef HAVE_LIBPNG_PNG_H
-#include <libpng/png.h>
-#else
 #include <png.h>
-#endif
-
-#include <Foundation/NSData.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSValue.h>
-#include "AppKit/NSGraphics.h"
 
 #if defined(PNG_FLOATING_POINT_SUPPORT)
 #  define PNG_FLOATING_POINT 1
@@ -54,6 +46,20 @@
 #  define PNG_gAMA 0
 #endif
 
+#endif /* HAVE_LIBPNG */
+
+/* we import all the standard headers to allow compilation without PNG */
+#import <Foundation/NSData.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSValue.h>
+#import "AppKit/NSGraphics.h"
+#import "NSBitmapImageRep+PNG.h"
+
+
+#ifdef HAVE_LIBPNG
+
 @implementation NSBitmapImageRep (PNG)
 
 + (BOOL) _bitmapIsPNG: (NSData *)imageData
@@ -229,7 +235,7 @@ static void reader_func(png_structp png_struct, png_bytep data,
     initWithBytesNoCopy: buf
 		 length: bytes_per_row * height];
 
-  if (PNG_INFO_gAMA & png_info->valid)
+  if (png_get_valid(png_struct, png_info, PNG_INFO_gAMA))
   {
     double file_gamma = 2.2;
     if (PNG_FLOATING_POINT)
