@@ -95,7 +95,9 @@ static NSTextFieldCell *titleCell;
 
 - (id) init
 {
-  [super init];
+  self = [super init];
+  if (nil == self)
+    return nil;
 
   _isLoaded = NO;
 
@@ -174,7 +176,6 @@ static NSTextFieldCell *titleCell;
 {
   if ([aDecoder allowsKeyedCoding])
     {
-      
     }
   else
     {
@@ -939,7 +940,7 @@ static NSTextFieldCell *titleCell;
 */
 - (void) setLastColumn: (int)column
 {
-  int i, count, num;
+  int i, count;
   NSBrowserColumn *bc;
   NSScrollView *sc;
 
@@ -958,7 +959,6 @@ static NSTextFieldCell *titleCell;
 
   // Unloads columns.
   count = [_browserColumns count];
-  num = [self numberOfVisibleColumns];
 
   for (i = column + 1; i < count; ++i)
     {
@@ -1677,6 +1677,13 @@ static NSTextFieldCell *titleCell;
     }
 }
 
+- (void) scrollRowToVisible: (NSInteger)row inColumn: (NSInteger)column
+{
+  NSMatrix *matrix = [self matrixInColumn: column];
+
+  [matrix scrollCellToVisibleAtRow: row
+                            column: 1];
+}
 
 /*
  * Showing a horizontal scroller
@@ -2324,73 +2331,10 @@ static NSTextFieldCell *titleCell;
 
 - (void) drawRect: (NSRect)rect
 {
-  // Load the first column if not already done
-  if (!_isLoaded)
-    {
-      [self loadColumnZero];
-    }
-
-  // Draws titles
-  if (_isTitled)
-    {
-      int i;
-
-      for (i = _firstVisibleColumn; i <= _lastVisibleColumn; ++i)
-        {
-          NSRect titleRect = [self titleFrameOfColumn: i];
-          if (NSIntersectsRect (titleRect, rect) == YES)
-            {
-              [self drawTitleOfColumn: i
-                    inRect: titleRect];
-            }
-        }
-    }
-
-  // Draws scroller border
-  if (_hasHorizontalScroller && _separatesColumns)
-    {
-      NSRect scrollerBorderRect = _scrollerRect;
-      NSSize bs = [[GSTheme theme] sizeForBorderType: NSBezelBorder];
-
-      scrollerBorderRect.origin.x = 0;
-      scrollerBorderRect.origin.y = 0;
-      scrollerBorderRect.size.width += 2 * bs.width;
-      scrollerBorderRect.size.height += (2 * bs.height) - 1;
-
-      if ((NSIntersectsRect (scrollerBorderRect, rect) == YES) && _window)
-        {
-          [[GSTheme theme] drawGrayBezel: scrollerBorderRect withClip: rect];
-        }
-    }
-
-  if (!_separatesColumns)
-    {
-      NSPoint p1,p2;
-      int     i, visibleColumns;
-      float   hScrollerWidth = _hasHorizontalScroller ? scrollerWidth : 0;
-      
-      // Columns borders
-      [[GSTheme theme] drawGrayBezel: _bounds withClip: rect];
-      
-      [[NSColor blackColor] set];
-      visibleColumns = [self numberOfVisibleColumns]; 
-      for (i = 1; i < visibleColumns; i++)
-        {
-          p1 = NSMakePoint((_columnSize.width * i) + 2 + (i-1), 
-                           _columnSize.height + hScrollerWidth + 2);
-          p2 = NSMakePoint((_columnSize.width * i) + 2 + (i-1),
-                           hScrollerWidth + 2);
-          [NSBezierPath strokeLineFromPoint: p1 toPoint: p2];
-        }
-
-      // Horizontal scroller border
-      if (_hasHorizontalScroller)
-        {
-          p1 = NSMakePoint(2, hScrollerWidth + 2);
-          p2 = NSMakePoint(rect.size.width - 2, hScrollerWidth + 2);
-          [NSBezierPath strokeLineFromPoint: p1 toPoint: p2];
-        }
-    }
+  [[GSTheme theme] drawBrowserRect: rect
+		   inView: self
+		   withScrollerRect: _scrollerRect
+		   columnSize: _columnSize];
 }
 
 /* Informs the receivers's subviews that the receiver's bounds
@@ -2455,7 +2399,6 @@ static NSTextFieldCell *titleCell;
         }
       if (selectedColumn == -1)
         {
-          selectedColumn = 0;
           matrix = [self matrixInColumn: 0];
 
           if ([[matrix cells] count])
@@ -2517,6 +2460,9 @@ static NSTextFieldCell *titleCell;
         case NSRightArrowFunctionKey:
           [self moveRight:self];
           return;
+        case NSBackTabCharacter:
+          [_window selectKeyViewPrecedingView: self];
+          return;
         case NSTabCharacter:
           {
             if ([theEvent modifierFlags] & NSShiftKeyMask)
@@ -2529,7 +2475,6 @@ static NSTextFieldCell *titleCell;
               }
           }
           return;
-          break;
         }
     }
 
@@ -2826,10 +2771,9 @@ static NSTextFieldCell *titleCell;
   else
     {
       int colCount;
-      id dummy;
       
       // Here to keep compatibility with old version
-      dummy = [aDecoder decodeObject];
+      [aDecoder decodeObject];
       _browserCellPrototype = RETAIN([aDecoder decodeObject]);
       _browserMatrixClass   = NSClassFromString ((NSString *)[aDecoder decodeObject]);
       
