@@ -28,18 +28,18 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "config.h"
+#import "config.h"
 
-#include <Foundation/NSUserDefaults.h>
+#import <Foundation/NSUserDefaults.h>
 
-#include "AppKit/NSBrowserCell.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSWindow.h"
-#include "GSGuiPrivate.h"
+#import "AppKit/NSBrowserCell.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSWindow.h"
+#import "GSGuiPrivate.h"
 
 /*
  * Class variables
@@ -106,7 +106,10 @@ static NSFont *_leafFont;
  */
 - (id) initTextCell: (NSString *)aString
 {
-  [super initTextCell: aString];
+  self = [super initTextCell: aString];
+  if (nil == self)
+    return self;
+
   //_alternateImage = nil;
   //_browsercell_is_leaf = NO; 
   //_browsercell_is_loaded = NO;
@@ -120,7 +123,10 @@ static NSFont *_leafFont;
 
 - (id) initImageCell: (NSImage *)anImage
 {
-  [super initImageCell: anImage];
+  self = [super initImageCell: anImage];
+  if (nil == self)
+    return self;
+  
   //_alternateImage = nil;
   //_browsercell_is_leaf = NO; 
   //_browsercell_is_loaded = NO;
@@ -357,7 +363,9 @@ static NSFont *_leafFont;
 
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
-  [super initWithCoder: aDecoder];
+  self = [super initWithCoder: aDecoder];
+  if (nil == self)
+    return self;
 
   if ([aDecoder allowsKeyedCoding])
     {
