@@ -61,7 +61,7 @@
 
 typedef struct _GSButtonCellFlags 
 {
-#ifdef WORDS_BIGENDIAN
+#if GS_WORDS_BIGENDIAN == 1
   unsigned int isPushin:1;
   unsigned int changeContents:1;
   unsigned int changeBackground:1;
@@ -1574,7 +1574,8 @@ typedef struct _GSButtonCellFlags
       // encode button image source, if it exists...
       if (bi != nil)
         {
-          [aCoder encodeObject: bi forKey: @"NSAlternateImage"];      
+          [aCoder encodeObject: bi forKey: @"NSAlternateImage"];
+          RELEASE(bi);
         }
       else if (_altImage != nil)
         {
