@@ -46,6 +46,7 @@
 #import "AppKit/NSImage.h"
 #import "AppKit/NSMatrix.h"
 #import "AppKit/NSPasteboard.h"
+#import "AppKit/NSPanel.h"
 #import "AppKit/NSSlider.h"
 #import "AppKit/NSSplitView.h"
 #import "AppKit/NSWindow.h"
@@ -206,6 +207,7 @@ static int _gs_gui_color_picker_mode = NSRGBModeColorPanel;
 - (id) _initWithoutGModel
 {
   NSRect contentRect = {{352, 519}, {200, 270}};
+  NSSize maxContentSize = {500, 675};
   NSRect topViewRect = {{0, 0}, {200, 270}};
   NSRect magnifyRect = {{4, 230}, {50, 36}};
   NSRect wellRect = {{58, 230}, {138, 36}};
@@ -222,16 +224,20 @@ static int _gs_gui_color_picker_mode = NSRGBModeColorPanel;
   NSColorWell *well;
   int i;
   unsigned int style = NSTitledWindowMask | NSClosableWindowMask
-                      | NSResizableWindowMask;
+                      | NSResizableWindowMask | NSUtilityWindowMask;
 
   self = [super initWithContentRect: contentRect 
 			  styleMask: style
 			    backing: NSBackingStoreRetained
 			      defer: NO
 			     screen: nil];
+  if (nil == self)
+    return nil;
+
   [self setTitle: _(@"Colors")];
   [self setBecomesKeyOnlyIfNeeded: YES];
-
+  [self setContentMinSize: contentRect.size];
+  [self setContentMaxSize: maxContentSize];
   v = [self contentView];
 
   _topView = [[NSView alloc] initWithFrame: topViewRect];
@@ -673,7 +679,7 @@ static int _gs_gui_color_picker_mode = NSRGBModeColorPanel;
 }
 
 /** <p>Sets the NSColor displayed to aColor. This method post a
-    NSColorPanelColorChangedNotification notification if needed.</p>
+    NSColorPanelColorDidChangeNotification notification if needed.</p>
     <p>See Also: -color [NSColorWell-setColor:]
     </p>
 */
@@ -688,7 +694,7 @@ static int _gs_gui_color_picker_mode = NSRGBModeColorPanel;
     [NSApp sendAction: _action to: _target from: self];  
 
   [[NSNotificationCenter defaultCenter]
-      postNotificationName: NSColorPanelColorChangedNotification
+      postNotificationName: NSColorPanelColorDidChangeNotification
                     object: (id)self];
 }
 
@@ -707,7 +713,9 @@ static int _gs_gui_color_picker_mode = NSRGBModeColorPanel;
 
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
-  [super initWithCoder: aDecoder];
+  self = [super initWithCoder: aDecoder];
+  if (nil == self)
+    return nil;
 
   return self;
 }
