@@ -26,41 +26,37 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSArray.h>
-#include <Foundation/NSRunLoop.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSAutoreleasePool.h>
-#include <Foundation/NSValue.h>
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSBezierPath.h"
-#include "AppKit/NSBox.h"
-#include "AppKit/NSBrowser.h"
-#include "AppKit/NSBrowserCell.h"
-#include "AppKit/NSButtonCell.h"
-#include "AppKit/NSComboBox.h"
-#include "AppKit/NSComboBoxCell.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSGraphicsContext.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSMatrix.h"
-#include "AppKit/NSPanel.h"
-#include "AppKit/NSScreen.h"
-#include "AppKit/NSScroller.h"
-#include "AppKit/NSScrollView.h"
-#include "AppKit/NSTableColumn.h"
-#include "AppKit/NSTableView.h"
-#include "AppKit/NSTextView.h"
-#include "GNUstepGUI/GSTheme.h"
-#include "GSGuiPrivate.h"
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSRunLoop.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSAutoreleasePool.h>
+#import <Foundation/NSValue.h>
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSBox.h"
+#import "AppKit/NSBrowser.h"
+#import "AppKit/NSBrowserCell.h"
+#import "AppKit/NSButtonCell.h"
+#import "AppKit/NSComboBox.h"
+#import "AppKit/NSComboBoxCell.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSGraphicsContext.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSPanel.h"
+#import "AppKit/NSScreen.h"
+#import "AppKit/NSScroller.h"
+#import "AppKit/NSScrollView.h"
+#import "AppKit/NSTableColumn.h"
+#import "AppKit/NSTableView.h"
+#import "AppKit/NSTextView.h"
+#import "GNUstepGUI/GSTheme.h"
+#import "GSGuiPrivate.h"
 
 static NSNotificationCenter *nc;
-static const BOOL ForceBrowser = NO;
 
 @interface GSComboBoxTableView : NSTableView
 {
-
 }
 
 @end
@@ -135,13 +131,18 @@ static GSComboWindow *gsWindow = nil;
 		     defer: (BOOL)flag
 {
   NSBox *box;
-  NSScrollView *scrollView;
   NSRect borderRect;
+  NSScrollView *scrollView;
+  NSTableColumn *column;
+  NSCell *cell;
    
   self = [super initWithContentRect: contentRect
 		          styleMask: aStyle
 		            backing: bufferingType
 		              defer: flag];	  
+  if (nil == self)
+    return self;
+
   [self setLevel: NSPopUpMenuWindowLevel];
   [self setBecomesKeyOnlyIfNeeded: YES];
   
@@ -154,10 +155,8 @@ static GSComboWindow *gsWindow = nil;
   borderRect = contentRect;
   RELEASE(box);
   
-  if (!ForceBrowser)
-    {
   _tableView = [[GSComboBoxTableView alloc] 
-    initWithFrame: NSMakeRect(0, 0, 100, 100)];
+                     initWithFrame: NSMakeRect(0, 0, 100, 100)];
   [_tableView setAutoresizingMask: NSViewWidthSizable | NSViewHeightSizable];
   //[_tableView setBackgroundColor: [NSColor whiteColor]];
   [_tableView setDrawsGrid: NO];
@@ -166,8 +165,14 @@ static GSComboWindow *gsWindow = nil;
   [_tableView setAutoresizesAllColumnsToFit: YES];
   [_tableView setHeaderView: nil];
   [_tableView setCornerView: nil];
-  [_tableView addTableColumn: [[NSTableColumn alloc] initWithIdentifier: @"content"]];
-  [[_tableView tableColumnWithIdentifier:@"content"]  setDataCell: [[NSCell alloc] initTextCell: @""]];
+  
+  column = [[NSTableColumn alloc] initWithIdentifier: @"content"];
+  cell = [[NSCell alloc] initTextCell: @""];
+  [column setDataCell: cell];
+  RELEASE(cell);
+  [_tableView addTableColumn: column];
+  RELEASE(column);
+  
   [_tableView setDataSource: self];
   [_tableView setDelegate: self];
   [_tableView setAction: @selector(clickItem:)];
@@ -175,34 +180,14 @@ static GSComboWindow *gsWindow = nil;
   
   scrollView = [[NSScrollView alloc] initWithFrame: NSMakeRect(borderRect.origin.x, 
                                                                borderRect.origin.y,
-				                             borderRect.size.width, 
-						          borderRect.size.height)];
+                                                               borderRect.size.width, 
+                                                               borderRect.size.height)];
   [scrollView setHasVerticalScroller: YES];
   [scrollView setDocumentView: _tableView];
   [box setContentView: scrollView];
+  RELEASE(scrollView);
   
   [_tableView reloadData];
-    }
-  else
-    {
-  _browser = [[NSBrowser alloc] initWithFrame: NSMakeRect(borderRect.origin.x, 
-                                                          borderRect.origin.y,
-				                        borderRect.size.width, 
-						     borderRect.size.height)];
-  [_browser setMaxVisibleColumns: 1];
-  [_browser setTitled: NO];
-  [_browser setHasHorizontalScroller: NO];
-  [_browser setTarget: self];
-  [_browser setAction: @selector(clickItem:)];
-  [_browser setDelegate: self];
-  [_browser setAutoresizingMask: NSViewWidthSizable | NSViewHeightSizable];
-  [_browser setAllowsEmptySelection: YES];
-  [_browser setAllowsMultipleSelection: NO];
-  [_browser setReusesColumns: YES];
-  // Create an empty matrix
-  [_browser loadColumnZero];
-  [box setContentView: _browser];
-    }
 
   return self;
 }
@@ -220,7 +205,6 @@ static GSComboWindow *gsWindow = nil;
 
 - (void) layoutWithComboBoxCell: (NSComboBoxCell *)comboBoxCell
 {
-  NSMatrix *matrix = [_browser matrixInColumn: 0];
   NSSize bsize = [[GSTheme theme] sizeForBorderType: NSLineBorder];
   NSSize size;
   float itemHeight;
@@ -236,14 +220,7 @@ static GSComboWindow *gsWindow = nil;
   if (itemHeight <= 0)
     {
       // FIX ME : raise NSException
-      if (!ForceBrowser)
-        {
       itemHeight = [_tableView rowHeight];
-        }
-      else
-        {
-      itemHeight = [matrix cellSize].height;
-        }
     }
   size.height = itemHeight;
     
@@ -265,14 +242,7 @@ static GSComboWindow *gsWindow = nil;
       size.width = 0;
     }
    
-  if (!ForceBrowser)
-    {
   [_tableView setRowHeight: size.height];
-    }
-  else
-    {
-  [matrix setCellSize: size];
-    }
     
   // Just check intercell spacing
 
@@ -280,25 +250,11 @@ static GSComboWindow *gsWindow = nil;
   if (intercellSpacing.height <= 0)
     {
       // FIX ME : raise NSException
-      if (!ForceBrowser)
-        {
       intercellSpacing.height = [_tableView intercellSpacing].height;
-        }
-      else
-        {
-      intercellSpacing.height = [matrix intercellSpacing].height;
-	}
     }
   else 
     {
-      if (!ForceBrowser)
-        {
       [_tableView setIntercellSpacing: intercellSpacing];
-        }
-      else
-        {
-      [matrix setIntercellSpacing: intercellSpacing];
-        }
     }
     
     
@@ -465,6 +421,10 @@ static GSComboWindow *gsWindow = nil;
             {
               [self validateSelection];
             }
+	  else if (key == 0x001b)
+	    {
+	      break;
+	    }
           else
             {
 	      [NSApp sendEvent: event];
@@ -493,14 +453,7 @@ static GSComboWindow *gsWindow = nil;
 
 - (void) reloadData
 {
-  if (!ForceBrowser)
-    {
   [_tableView reloadData];
-    }
-  else
-    {
-  [_browser loadColumnZero];
-    }
   [self selectItemAtIndex: [_cell indexOfSelectedItem]];
 }
 
@@ -514,32 +467,13 @@ static GSComboWindow *gsWindow = nil;
 {
   NSRect rect;
   
-  if (!ForceBrowser)
-    {
   rect = [_tableView frameOfCellAtColumn: 0 row: index];
   [_tableView scrollPoint: rect.origin]; 
-    }
-  else
-    {  
-  NSMatrix *matrix = [_browser matrixInColumn: 0];
-
-  rect = [matrix cellFrameAtRow: index column: 0];
-  [matrix scrollPoint: rect.origin];
-    }
 }
 
 - (void) scrollItemAtIndexToVisible: (int)index
 {
-  if (!ForceBrowser)
-    {
   [_tableView scrollRowToVisible: index];
-    }
-  else
-    {
-  NSMatrix *matrix = [_browser matrixInColumn: 0];
-
-  [matrix scrollCellToVisibleAtRow: index column: 0];
-    }
 }
 
 - (void) selectItemAtIndex: (int)index
@@ -547,36 +481,15 @@ static GSComboWindow *gsWindow = nil;
   if (index < 0)
     return;
   
-  if (!ForceBrowser)
-    {
   if ([_tableView selectedRow] == index || [_tableView numberOfRows] <= index)
     return;
   
   [_tableView selectRow: index byExtendingSelection: NO];     
-    }
-  else
-    {
-  NSMatrix *matrix = [_browser matrixInColumn: 0];
-  
-  if ([matrix selectedRow] == index || [matrix numberOfRows] <= index)
-    return;
-    
-  [_browser selectRow: index inColumn: 0];
-    }   
 }
 
 - (void) deselectItemAtIndex: (int)index
 {
-  if (!ForceBrowser)
-    {
   [_tableView deselectAll: self];
-    }
-  else
-    {
-  NSMatrix *matrix = [_browser matrixInColumn: 0];
-
-  [matrix deselectSelectedCell];
-    }
 }
 
 // Target/Action method
@@ -585,19 +498,7 @@ static GSComboWindow *gsWindow = nil;
   if (_cell == nil)
     return;
   
-  if (!ForceBrowser)  
-    {
-      [_cell _setSelectedItem: [sender selectedRow]];
-    }
-  else
-    {
-      [_cell _setSelectedItem: [sender selectedRowInColumn: 0]];
-      // NSBrowser has nothing like browserSelectionDidChange: delegate method
-      // which means selection changes not resulting from a click should be 
-      // handled here by calling previously -setSendsActionOnArrowKeys: on the
-      // browser.
-    }
-  
+  [_cell _setSelectedItem: [sender selectedRow]];
   [self validateSelection];
   
   [nc postNotificationName: NSComboBoxSelectionDidChangeNotification
@@ -650,8 +551,6 @@ static GSComboWindow *gsWindow = nil;
 // Key actions methods
 - (void) moveUpSelection
 {
-  if (!ForceBrowser)
-    {
   int index = [_tableView selectedRow] - 1;
 
   if (index > -1 && index < [_tableView numberOfRows])
@@ -659,22 +558,10 @@ static GSComboWindow *gsWindow = nil;
       [_tableView selectRow: index byExtendingSelection: NO];
       [_tableView scrollRowToVisible: index];
     }
-    }
-  else
-    {
-  int index = [_browser selectedRowInColumn: 0] - 1;
-
-  if (index > -1 && index < [[_browser matrixInColumn: 0] numberOfRows])
-    {
-      [_browser selectRow: index inColumn: 0];
-    }
-    }
 }
 
 - (void) moveDownSelection
 {
-  if (!ForceBrowser)
-    { 
   int index = [_tableView selectedRow] + 1;
   
   if (index > -1 && index < [_tableView numberOfRows])
@@ -682,16 +569,6 @@ static GSComboWindow *gsWindow = nil;
       [_tableView selectRow: index byExtendingSelection: NO];
       [_tableView scrollRowToVisible: index];
     }
-    }
-  else
-    {
-  int index = [_browser selectedRowInColumn: 0] + 1;
-
-  if (index > -1 && index < [[_browser matrixInColumn: 0] numberOfRows])
-    {
-      [_browser selectRow: index inColumn: 0];
-    }
-    }
 }
 
 - (void) validateSelection
@@ -1696,7 +1573,9 @@ static inline NSRect buttonCellFrameFromRect(NSRect cellRect)
 - (id) initWithCoder: (NSCoder *)aDecoder
 {
   self = [super initWithCoder: aDecoder];
-  
+  if (nil == self)
+    return nil;
+
   if ([aDecoder allowsKeyedCoding])
     {
       //id delegate = [aDecoder decodeObjectForKey: @"NSDelegate"];
@@ -1840,6 +1719,7 @@ static inline NSRect buttonCellFrameFromRect(NSRect cellRect)
 	      [textObject scrollRangeToVisible: NSMakeRange(location, length)];
 	    }
         }
+      RELEASE(myString);
     }
 }
 
@@ -1883,6 +1763,7 @@ static inline NSRect buttonCellFrameFromRect(NSRect cellRect)
   NSWindow *cvWindow = [controlView window];
   NSRect buttonRect = buttonCellFrameFromRect(cellFrame);
 
+  _control_view = controlView;
   [controlView lockFocus];
   [_buttonCell highlight: YES 
 	       withFrame: buttonRect
