@@ -30,20 +30,20 @@
 
 #include "config.h"
 
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSNotification.h>
-#include "AppKit/NSActionCell.h"
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSCell.h"
-#include "AppKit/NSControl.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSKeyValueBinding.h"
-#include "AppKit/NSTextStorage.h"
-#include "AppKit/NSTextView.h"
-#include "AppKit/NSWindow.h"
-#include "GSBindingHelpers.h"
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSNotification.h>
+#import "AppKit/NSActionCell.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSCell.h"
+#import "AppKit/NSControl.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSKeyValueBinding.h"
+#import "AppKit/NSTextStorage.h"
+#import "AppKit/NSTextView.h"
+#import "AppKit/NSWindow.h"
+#import "GSBindingHelpers.h"
 
 /*
  * Class variables
@@ -962,7 +962,7 @@ static NSNotificationCenter *nc;
 
   if ([aDecoder allowsKeyedCoding])
     {
-      NSCell *cell = RETAIN([aDecoder decodeObjectForKey: @"NSCell"]);
+      NSCell *cell = [aDecoder decodeObjectForKey: @"NSCell"];
       
       if (cell != nil)
         {
@@ -972,7 +972,7 @@ static NSNotificationCenter *nc;
         {
           // This is needed for subclasses without cells, like NSColorWeel
           // as we store some properties only on the cell.
-          NSCell *cell = [[[self class] cellClass] new];
+          cell = [[[self class] cellClass] new];
 
           [self setCell: cell];
           RELEASE(cell);
@@ -1004,25 +1004,33 @@ static NSNotificationCenter *nc;
 {
   if ([binding isEqual: NSValueBinding])
     {
+      GSKeyValueBinding *kvb;
+
       [self unbind: binding];
       // FIXME: We could also do the mapping via 
       // setKeys:triggerChangeNotificationsForDependentKey:
-      [[GSKeyValueBinding alloc] initWithBinding: @"objectValue"
-                                 withName: NSValueBinding
-                                 toObject: anObject
-                                 withKeyPath: keyPath
-                                 options: options
-                                 fromObject: self];
+      kvb = [[GSKeyValueBinding alloc] initWithBinding: @"objectValue"
+                                              withName: NSValueBinding
+                                              toObject: anObject
+                                           withKeyPath: keyPath
+                                               options: options
+                                            fromObject: self];
+      // The binding will be retained in the binding table
+      RELEASE(kvb);
     }
   else if ([binding hasPrefix: NSEnabledBinding])
     {
+      GSKeyValueBinding *kvb;
+
       [self unbind: binding];
-      [[GSKeyValueAndBinding alloc] initWithBinding: NSEnabledBinding 
-                                    withName: binding 
-                                    toObject: anObject
-                                    withKeyPath: keyPath
-                                    options: options
-                                    fromObject: self];
+      kvb = [[GSKeyValueAndBinding alloc] initWithBinding: NSEnabledBinding 
+                                                 withName: binding 
+                                                 toObject: anObject
+                                              withKeyPath: keyPath
+                                                  options: options
+                                               fromObject: self];
+      // The binding will be retained in the binding table
+      RELEASE(kvb);
     }
   else
     {
