@@ -1052,6 +1052,8 @@ originalContentsURL: (NSURL *)orig
 
   ASSIGN(_save_type, [[sender selectedItem] representedObject]);
   extensions = [controller fileExtensionsFromType: _save_type];
+  if ([extensions containsObject: @"*"])
+    extensions = nil;
   [(NSSavePanel *)[sender window] setAllowedFileTypes: extensions];
 }
 
@@ -1069,6 +1071,17 @@ originalContentsURL: (NSURL *)orig
     {
       directory = [[self fileName] stringByDeletingLastPathComponent];
       file = [[self fileName] lastPathComponent];
+      if (![savePanel allowsOtherFileTypes])
+	{
+	  NSArray *exts = [savePanel allowedFileTypes];
+	  if ([exts count] && ![exts containsObject: [file pathExtension]] &&
+	      ![exts containsObject: @"*"])
+	    {
+	      file = [file stringByDeletingPathExtension];
+	      file = [file stringByAppendingPathExtension:
+			     [exts objectAtIndex: 0]];
+	    }
+	}
       return [savePanel runModalForDirectory: directory file: file];
     }
 
@@ -1157,6 +1170,8 @@ originalContentsURL: (NSURL *)orig
   if ([types count] > 0)
     {
       NSArray *extensions = [controller fileExtensionsFromType: [self fileType]];
+      if ([extensions containsObject: @"*"])
+	extensions = nil;
       [savePanel setAllowedFileTypes: extensions];
     }
 
@@ -1272,7 +1287,7 @@ originalContentsURL: (NSURL *)orig
   NSArray *exts = [[NSDocumentController sharedDocumentController]
                       fileExtensionsFromType: typeName];
   
-  if ([exts count])
+  if ([exts count] && ![exts containsObject: @"*"])
     return (NSString *)[exts objectAtIndex: 0];
 
   return @"";
@@ -1290,8 +1305,19 @@ originalContentsURL: (NSURL *)orig
 
 - (void) setPrintInfo: (NSPrintInfo *)printInfo
 {
+  NSUndoManager *undoManager = [self undoManager];
+
+  if (undoManager != nil)
+    {
+      [[undoManager prepareWithInvocationTarget: self]
+	setPrintInfo: _print_info];
+      // FIXME undoManager -setActionName:
+    }
   ASSIGN(_print_info, printInfo);
-  [self updateChangeCount: NSChangeDone];
+  if (undoManager == nil)
+    {
+      [self updateChangeCount: NSChangeDone];
+    }
 }
 
 
@@ -1453,9 +1479,9 @@ originalContentsURL: (NSURL *)orig
 
 - (BOOL) validateUserInterfaceItem: (id <NSValidatedUserInterfaceItem>)anItem
 {
-  if (sel_eq([anItem action], @selector(revertDocumentToSaved:)))
+  if (sel_isEqual([anItem action], @selector(revertDocumentToSaved:)))
     return ([self fileName] != nil && [self isDocumentEdited]);
-  if (sel_eq([anItem action], @selector(saveDocument:)))
+  if (sel_isEqual([anItem action], @selector(saveDocument:)))
     return [self isDocumentEdited];
 
   // FIXME should validate spa popup items; return YES if it's a native type.
@@ -1550,6 +1576,13 @@ originalContentsURL: (NSURL *)orig
       saved = [self writeWithBackupToFile: fileName 
                     ofType: [self fileTypeFromLastRunSavePanel]
                     saveOperation: saveOperation];
+      if (saved &&
+	  (saveOperation == NSSaveOperation ||
+	   saveOperation == NSSaveAsOperation))
+	{
+	  [[NSDocumentController sharedDocumentController]
+	    noteNewRecentDocument: self];
+	}
     }
 
   if (delegate != nil && didSaveSelector != NULL)
@@ -1567,13 +1600,20 @@ originalContentsURL: (NSURL *)orig
   forSaveOperation: (NSSaveOperationType)op
              error: (NSError **)error
 {
-  return [self writeSafelyToURL: url
-               ofType: type
-               forSaveOperation: op
-               error: error];
+  BOOL saved =
+    [self writeSafelyToURL: url
+	  ofType: type
+          forSaveOperation: op
+          error: error];
+  if (saved && (op == NSSaveOperation || op == NSSaveAsOperation))
+    {
+      [[NSDocumentController sharedDocumentController]
+	noteNewRecentDocument: self];
+    }
+  return saved;
 }
 
-- (BOOL) saveToURL: (NSURL *)url
+- (void) saveToURL: (NSURL *)url
             ofType: (NSString *)type
   forSaveOperation: (NSSaveOperationType)op
           delegate: (id)delegate
@@ -1591,6 +1631,11 @@ originalContentsURL: (NSURL *)orig
     {
       [self presentError: error]; 
     }
+  else if (op == NSSaveOperation || op == NSSaveAsOperation)
+    {
+      [[NSDocumentController sharedDocumentController]
+	noteNewRecentDocument: self];
+    }
 
   if (delegate != nil && didSaveSelector != NULL)
     {
@@ -1600,8 +1645,6 @@ originalContentsURL: (NSURL *)orig
       if (meth)
         meth(delegate, didSaveSelector, self, saved, contextInfo);
     }
-
-  return saved;
 }
 
 - (IBAction) revertDocumentToSaved: (id)sender
