@@ -28,24 +28,24 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSArray.h>
-#include <Foundation/NSFileManager.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSPathUtilities.h>
-#include <Foundation/NSProcessInfo.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSURL.h>
-#include <Foundation/NSUserDefaults.h>
-#include <Foundation/NSTimer.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSFileManager.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSPathUtilities.h>
+#import <Foundation/NSProcessInfo.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSURL.h>
+#import <Foundation/NSUserDefaults.h>
+#import <Foundation/NSTimer.h>
 
-#include "AppKit/NSDocumentController.h"
-#include "AppKit/NSOpenPanel.h"
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSMenu.h"
-#include "AppKit/NSMenuItem.h"
-#include "AppKit/NSWorkspace.h"
-#include "NSDocumentFrameworkPrivate.h"
-#include "GSGuiPrivate.h"
+#import "AppKit/NSDocumentController.h"
+#import "AppKit/NSOpenPanel.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSMenu.h"
+#import "AppKit/NSMenuItem.h"
+#import "AppKit/NSWorkspace.h"
+#import "NSDocumentFrameworkPrivate.h"
+#import "GSGuiPrivate.h"
 
 static NSString *NSTypesKey             = @"NSTypes";
 static NSString *NSNameKey              = @"NSName";
@@ -65,6 +65,7 @@ static NSString *CFBundleTypeRole       = @"CFBundleTypeRole";
 
 // FIXME: Looks like this was changed to @"NSRecentDocumentRecords"
 static NSString *NSRecentDocuments      = @"NSRecentDocuments";
+static NSString *NSMaximumRecentDocumentCount = @"NSMaximumRecentDocumentCount";
 
 static NSString *NSEditorRole = @"Editor";
 static NSString *NSViewerRole = @"Viewer";
@@ -101,6 +102,7 @@ static NSDictionary *TypeInfoForName (NSArray *types, NSString *typeName)
   return nil;
 }
 
+/*
 static NSDictionary *
 TypeInfoForHumanReadableName (NSArray *types, NSString *typeName)
 {
@@ -124,14 +126,9 @@ TypeInfoForHumanReadableName (NSArray *types, NSString *typeName)
 
   return nil;
 }
+*/
 
 
-@interface NSDocumentController (RecentsMenu)
-- (NSMenu *) _recentMenu;
-- (void) _updateOpenRecentMenu;
-- (IBAction) _openRecentDocument: (id)sender;
-@end
-
 /** <p>
     NSDocumentController is a class that controls a set of NSDocuments
     for an application. As an application delegate, it responds to the
@@ -280,7 +277,7 @@ TypeInfoForHumanReadableName (NSArray *types, NSString *typeName)
                        objectForKey: NSRecentDocuments];
   if (_recent_documents)
     {
-      int i, count;
+      int i, count, max;
 
       _recent_documents = [_recent_documents mutableCopy];
       count = [_recent_documents count];
@@ -303,6 +300,12 @@ TypeInfoForHumanReadableName (NSArray *types, NSString *typeName)
               [_recent_documents replaceObjectAtIndex: i withObject: url];
             }
         }
+
+      max = [self maximumRecentDocumentCount];
+      if (count > max)
+	{
+	  [_recent_documents removeObjectsInRange: NSMakeRange(0, count - max)];
+	}
     } 
   else
     _recent_documents = RETAIN([NSMutableArray array]);
@@ -325,6 +328,7 @@ TypeInfoForHumanReadableName (NSArray *types, NSString *typeName)
   RELEASE (_documents);
   RELEASE (_recent_documents);
   RELEASE (_types);
+  RELEASE (_recent_documents_menu);
   [super dealloc];
 }
 
@@ -741,6 +745,8 @@ TypeInfoForHumanReadableName (NSArray *types, NSString *typeName)
 
   [openPanel setDirectory: [self currentDirectory]];
   [openPanel setAllowsMultipleSelection: YES];
+  [openPanel setCanChooseFiles: YES];
+  [openPanel setCanChooseDirectories: NO];
   return openPanel;
 }
 
@@ -767,7 +773,7 @@ TypeInfoForHumanReadableName (NSArray *types, NSString *typeName)
       [array addObjectsFromArray: [typeInfo objectForKey: CFBundleTypeExtensions]];
     }
   
-  return array;
+  return [array containsObject: @"*"] == NO ? (NSArray *)array : (NSArray *)nil;
 }
 
 /** Uses -runModalOpenPanel:forTypes: to allow the user to select
@@ -1184,11 +1190,11 @@ static BOOL _shouldClose = YES;
 
 - (BOOL) validateUserInterfaceItem: (id <NSValidatedUserInterfaceItem>)anItem
 {
-  if (sel_eq([anItem action], @selector(saveAllDocuments:)))
+  if (sel_isEqual([anItem action], @selector(saveAllDocuments:)))
     {
       return [self hasEditedDocuments];
     }
-  if (sel_eq([anItem action], @selector(newDocument:)))
+  if (sel_isEqual([anItem action], @selector(newDocument:)))
     {
       return ([self defaultType] != nil); 
     }
@@ -1213,7 +1219,8 @@ static BOOL _shouldClose = YES;
 - (NSString *) typeFromFileExtension: (NSString *)fileExtension
 {
   int i, count = [_types count];
-        
+
+  // Check for a document type with the supplied extension
   for (i = 0; i < count; i++)
     {
       NSDictionary *typeInfo = [_types objectAtIndex: i];
@@ -1234,7 +1241,28 @@ static BOOL _shouldClose = YES;
           return type;
         }
     }
-        
+
+  // No exact match; check for a document type that supports any extension
+  for (i = 0; i < count; i++)
+    {
+      NSDictionary *typeInfo = [_types objectAtIndex: i];
+      
+      if ([[typeInfo objectForKey: NSUnixExtensionsKey] containsObject: @"*"]
+	|| [[typeInfo objectForKey: NSDOSExtensionsKey] containsObject: @"*"]
+	|| [[typeInfo objectForKey: CFBundleTypeExtensions]
+          containsObject: @"*"])
+        {
+          NSString *type = [typeInfo objectForKey: NSNameKey];
+
+          if (type == nil)
+            {
+              type = [typeInfo objectForKey: CFBundleTypeName];
+            }
+          return type;
+        }
+    }
+
+  // No luck
   return nil;
 }
 
@@ -1280,14 +1308,25 @@ static BOOL _shouldClose = YES;
   [_recent_documents removeAllObjects];
   [[NSUserDefaults standardUserDefaults] 
     setObject: _recent_documents forKey: NSRecentDocuments];
-	[self _updateOpenRecentMenu];
+  [self _updateRecentDocumentsMenu];
 }
 
 // The number of remembered recent documents
 - (NSUInteger) maximumRecentDocumentCount
 {
-  // FIXME: Should come from user defaults
-  return 5;
+  NSUserDefaults *sud = [NSUserDefaults standardUserDefaults];
+  NSInteger count = 5;
+  if ([sud objectForKey: NSMaximumRecentDocumentCount])
+    {
+      count = [sud integerForKey: NSMaximumRecentDocumentCount];
+      if (count < 0)
+	count = 5;
+    }
+  else
+    {
+      count = 5;
+    }
+  return count;
 }
 
 - (void) noteNewRecentDocument: (NSDocument *)aDocument
@@ -1308,7 +1347,7 @@ static BOOL _shouldClose = YES;
       // Always keep the current object at the end of the list
       [_recent_documents removeObjectAtIndex: index];
     }
-  else if ([_recent_documents count] > [self maximumRecentDocumentCount])
+  else if ([_recent_documents count] >= [self maximumRecentDocumentCount])
     {
       [_recent_documents removeObjectAtIndex: 0];
     }
@@ -1326,7 +1365,7 @@ static BOOL _shouldClose = YES;
   [[NSUserDefaults standardUserDefaults] 
     setObject: a forKey: NSRecentDocuments];
   RELEASE(a);
-	[self _updateOpenRecentMenu];
+  [self _updateRecentDocumentsMenu];
 }
 
 - (NSArray *) recentDocumentURLs
@@ -1585,21 +1624,26 @@ static NSString *processName = nil;
 @end
 
 
-@implementation NSDocumentController (RecentsMenu)
+@implementation NSDocumentController (RecentDocumentsMenu)
 
-- (NSMenu *) _recentMenu
+- (NSMenu *) _recentDocumentsMenu
 {
-  // FIXME
-  return nil;
+  return _recent_documents_menu;
+}
+
+- (void) _setRecentDocumentsMenu: (NSMenu *)aMenu
+{
+  ASSIGN(_recent_documents_menu, aMenu);
+  [self _updateRecentDocumentsMenu];
 }
 
 // should be handled by making us the delegate of the recent's menu
-- (void) _updateOpenRecentMenu
+- (void) _updateRecentDocumentsMenu
 {
   NSMenu *recentMenu;
   int i;
 
-  recentMenu = [self _recentMenu];
+  recentMenu = [self _recentDocumentsMenu];
   if (!recentMenu)
     {
       return;
@@ -1666,10 +1710,23 @@ static NSString *processName = nil;
   if (idx < 0 || idx >= [_recent_documents count])
     {
       // something went wrong, ignore
-      [self _updateOpenRecentMenu];
+      [self _updateRecentDocumentsMenu];
       return;
     }
   url = (NSURL *)[_recent_documents objectAtIndex: idx];
+  if ([url isFileURL])
+    {
+      /* From Apple's documentation of the -noteNewRecentDocumentURL: method:
+	 "Applications not based on NSDocument must also implement the
+	 application:openFile: method in the application delegate to
+	 handle requests from the Open Recent menu command." */
+      id appDelegate = [NSApp delegate];
+      if ([appDelegate respondsToSelector: @selector(application:openFile:)])
+        {
+          if ([appDelegate application: NSApp openFile: [url path]])
+            return;
+        }
+    }
   [self openDocumentWithContentsOfURL: url display: YES];
 }
 
