@@ -30,18 +30,18 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include <Foundation/NSCoder.h>
-#include <Foundation/NSArchiver.h>
-#include <Foundation/NSKeyedArchiver.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSThread.h>
-#include <Foundation/NSTimer.h>
-#include <AppKit/NSWindow.h>
-#include <AppKit/NSBox.h>
-#include <AppKit/NSView.h>
-#include <AppKit/NSDrawer.h>
-#include <AppKit/NSGraphics.h>
+#import <Foundation/NSCoder.h>
+#import <Foundation/NSArchiver.h>
+#import <Foundation/NSKeyedArchiver.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSThread.h>
+#import <Foundation/NSTimer.h>
+#import "AppKit/NSWindow.h"
+#import "AppKit/NSBox.h"
+#import "AppKit/NSView.h"
+#import "AppKit/NSDrawer.h"
+#import "AppKit/NSGraphics.h"
 
 static NSNotificationCenter *nc = nil;
 
@@ -96,7 +96,6 @@ static NSNotificationCenter *nc = nil;
 		 styleMask: (unsigned int)aStyle
 		   backing: (NSBackingStoreType)bufferingType
 		     defer: (BOOL)flag
-		    screen: (NSScreen*)aScreen
 {
   if(NSIsEmptyRect(contentRect))
     {
@@ -106,8 +105,7 @@ static NSNotificationCenter *nc = nil;
   self = [super initWithContentRect: contentRect
 		styleMask: aStyle
 		backing: bufferingType
-		defer: flag
-		screen: aScreen];
+		defer: flag];
   if (self != nil)
     {
       NSRect rect = contentRect;
@@ -389,6 +387,8 @@ static NSNotificationCenter *nc = nil;
     { 
       NSRect frame = [self frameFromParentWindowFrameInState:[_drawer state]]; 
       [self setFrame: frame display: YES];
+      if (![self isVisible] && [_drawer state] != NSDrawerClosedState)
+        [self orderFront:self];
     }
   if ([self isVisible] && [_parentWindow isKeyWindow]) // do our best to maintain proper window ordering
     {
