@@ -30,17 +30,17 @@
 
 #include "config.h"
 
-#include "AppKit/NSFileWrapper.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSWorkspace.h"
-#include <Foundation/NSArray.h>
-#include <Foundation/NSData.h>
-#include <Foundation/NSValue.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSFileManager.h>
-#include <Foundation/NSArchiver.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSAutoreleasePool.h>
+#import <Foundation/NSArchiver.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSAutoreleasePool.h>
+#import <Foundation/NSData.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSFileManager.h>
+#import <Foundation/NSValue.h>
+#import "AppKit/NSFileWrapper.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSWorkspace.h"
 
 @implementation NSFileWrapper
 
@@ -339,7 +339,7 @@
 
 - (NSImage*) icon
 {
-  if (_iconImage == nil)
+  if (_iconImage == nil && [self filename])
     {
       return [[NSWorkspace sharedWorkspace] iconForFile: [self filename]];
     }
@@ -579,7 +579,7 @@
 
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
-  [aCoder encodeValueOfObjCType: @encode(GSFileWrapperType) at: &_wrapperType];
+  [aCoder encodeValueOfObjCType: @encode(int) at: &_wrapperType];
   // Dont store the file name
   [aCoder encodeObject: _preferredFilename];
   [aCoder encodeObject: _fileAttributes];
@@ -595,7 +595,7 @@
   id wrapperData;
   NSImage *iconImage;
 
-  [aDecoder decodeValueOfObjCType: @encode(GSFileWrapperType) at: &wrapperType];
+  [aDecoder decodeValueOfObjCType: @encode(int) at: &wrapperType];
   // Dont restore the file name
   preferredFilename = [aDecoder decodeObject];
   fileAttributes = [aDecoder decodeObject];
