@@ -45,7 +45,6 @@
 #import "AppKit/NSFontManager.h"
 #import "AppKit/NSView.h"
 #import "GNUstepGUI/GSFontInfo.h"
-#import "GSFusedSilicaContext.h"
 
 
 @interface NSFont (Private)
@@ -79,7 +78,7 @@ globalFontMap.
 }
 @end
 @implementation GSFontMapKey
--(unsigned int) hash
+-(NSUInteger) hash
 {
   return hash;
 }
@@ -935,7 +934,7 @@ static void setNSFont(NSString *key, NSFont *font)
   return YES;
 }
 
-- (unsigned) hash
+- (NSUInteger) hash
 {
   int i, sum;
   sum = 0;
@@ -1322,27 +1321,26 @@ static BOOL flip_hack;
       float size = [aDecoder decodeFloatForKey: @"NSSize"];
       
       DESTROY(self);
-      if ([aDecoder containsValueForKey: @"NSfFlags"])
-        {
-          int flags = [aDecoder decodeIntForKey: @"NSfFlags"];
-          // FIXME
-          if (flags == 16)
-            {
-              return RETAIN([NSFont controlContentFontOfSize: size]);
-            }
-          else if (flags == 20)
-            {
-              return RETAIN([NSFont labelFontOfSize: size]);
-            }
-          else if (flags == 22)
-            {
-              return RETAIN([NSFont titleBarFontOfSize: size]);
-            }
-        }
-
       self = RETAIN([NSFont fontWithName: name size: size]);
       if (self == nil)
         {
+	  if ([aDecoder containsValueForKey: @"NSfFlags"])
+	    {
+	      int flags = [aDecoder decodeIntForKey: @"NSfFlags"];
+	      // FIXME
+	      if (flags == 16)
+		{
+		  return RETAIN([NSFont controlContentFontOfSize: size]);
+		}
+	      else if (flags == 20)
+		{
+		  return RETAIN([NSFont labelFontOfSize: size]);
+		}
+	      else if (flags == 22)
+		{
+		  return RETAIN([NSFont titleBarFontOfSize: size]);
+		}
+	    }
           self = RETAIN([NSFont systemFontOfSize: size]);
         }
 
@@ -1493,7 +1491,7 @@ static BOOL flip_hack;
 - (void *) fontRef
 {
   if (_fontRef == nil)
-    _fontRef = [NSGraphicsContext CGFontReferenceFromFont: self];
+    _fontRef = fontInfo;
   return _fontRef;
 }
 
