@@ -419,15 +419,17 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
 
 - (id) initWithBitmapHandle: (void *)bitmap
 {
-  NSImageRep *rep = [[NSBitmapImageRep alloc] initWithBitmapHandle: bitmap];
+  NSImageRep *rep;
   
+  if (!(self = [self init]))
+    return nil;
+
+  rep = [[NSBitmapImageRep alloc] initWithBitmapHandle: bitmap];
   if (rep == nil)
     {
       RELEASE(self);
       return nil;
     }
-  if (!(self = [self init]))
-    return nil;
 
   [self addRepresentation: rep];
   RELEASE(rep);
@@ -437,15 +439,17 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
 - (id)initWithIconHandle:(void *)icon
 {
   // Only needed on MS Windows
-  NSImageRep *rep = [[NSBitmapImageRep alloc] initWithIconHandle: icon];
+  NSImageRep *rep;
   
+  if (!(self = [self init]))
+    return nil;
+
+  rep = [[NSBitmapImageRep alloc] initWithIconHandle: icon];
   if (rep == nil)
     {
       RELEASE(self);
       return nil;
     }
-  if (!(self = [self init]))
-    return nil;
 
   [self addRepresentation: rep];
   RELEASE(rep);
@@ -454,15 +458,17 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
 
 - (id) initWithContentsOfURL: (NSURL *)anURL
 {
-  NSArray *array = [NSImageRep imageRepsWithContentsOfURL: anURL];
+  NSArray *array;
 
+  if (!(self = [self init]))
+    return nil;
+
+  array = [NSImageRep imageRepsWithContentsOfURL: anURL];
   if (!array)
     {
       RELEASE(self);
       return nil;
     }
-  if (!(self = [self init]))
-    return nil;
 
   _flags.dataRetained = YES;
   [self addRepresentations: array];
@@ -477,7 +483,6 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
     return nil;
   
   reps = [NSImageRep imageRepsWithPasteboard: pasteboard];
-
   if (reps != nil)
     [self addRepresentations: reps]; 
   else
@@ -797,7 +802,6 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
   return _color;
 }
 
-
 // Using the Image 
 - (void) compositeToPoint: (NSPoint)aPoint 
                 operation: (NSCompositingOperation)op
@@ -813,8 +817,8 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
                  fromRect: (NSRect)aRect
                 operation: (NSCompositingOperation)op
 {
-  [self compositeToPoint: aPoint
-        fromRect: aRect
+  [self compositeToPoint: aPoint 
+        fromRect: aRect 
         operation: op
         fraction: 1.0];
 }
@@ -850,27 +854,27 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
 
       if (cache != nil)
         {
+          NSGraphicsContext *ctxt = GSCurrentContext();
           NSRect rect = [cache rect];
 
           NSDebugLLog(@"NSImage", @"composite rect %@ in %@", 
                       NSStringFromRect(rect), NSStringFromRect(srcRect));
+
           // Move the drawing rectangle to the origin of the image rep
           // and intersect the two rects.
           srcRect.origin.x += rect.origin.x;
           srcRect.origin.y += rect.origin.y;
           rect = NSIntersectionRect(srcRect, rect);
           
-          [GSCurrentContext() GScomposite: [[cache window] gState]
-                           toPoint: aPoint
-                           fromRect: rect
-                           operation: op
-                           fraction: delta];
+          [ctxt GScomposite: [[cache window] gState]
+                    toPoint: aPoint
+                   fromRect: rect
+                  operation: op
+                   fraction: delta];
         }
       else        
         {
-          NSRect rect;
-
-          rect = NSMakeRect(aPoint.x, aPoint.y, _size.width, _size.height);
+          NSRect rect = NSMakeRect(aPoint.x, aPoint.y, _size.width, _size.height);
           [self drawRepresentation: rep inRect: rect];
         }
     }
@@ -922,9 +926,6 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
     {
       NSRect fillrect = aRect;
 
-      if ([[NSView focusView] isFlipped])
-        fillrect.origin.y -= _size.height;
-
       [_color set];
       NSRectFill(fillrect);
 
@@ -957,10 +958,145 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
         fraction: delta];
 }
 
-- (void) drawInRect: (NSRect)dstRect
-           fromRect: (NSRect)srcRect
-          operation: (NSCompositingOperation)op
-           fraction: (float)delta
+/* New code path that delegates as much as possible to the backend and whose 
+behavior precisely matches Cocoa. */
+- (void) nativeDrawInRect: (NSRect)dstRect
+                 fromRect: (NSRect)srcRect
+                operation: (NSCompositingOperation)op
+                 fraction: (float)delta
+{
+  NSGraphicsContext *ctxt = GSCurrentContext();
+  NSSize imgSize = [self size];
+  float widthScaleFactor;
+  float heightScaleFactor;
+
+  if (NSEqualRects(srcRect, NSZeroRect))
+    {
+      srcRect.size = imgSize;
+      /* For -drawAtPoint:fromRect:operation:fraction: used with a zero rect */
+      if (NSEqualSizes(dstRect.size, NSZeroSize))
+        {
+          dstRect.size = imgSize;
+        }
+    }
+
+  if (!dstRect.size.width || !dstRect.size.height
+    || !srcRect.size.width || !srcRect.size.height)
+    return;
+
+  // Clip to image bounds
+  if (srcRect.origin.x < 0)
+    srcRect.origin.x = 0;
+  if (srcRect.origin.y < 0)
+    srcRect.origin.y = 0;
+  if (NSMaxX(srcRect) > imgSize.width)
+    srcRect.size.width = imgSize.width - srcRect.origin.x;
+  if (NSMaxY(srcRect) > imgSize.height)
+    srcRect.size.height = imgSize.height - srcRect.origin.y;
+
+  widthScaleFactor = dstRect.size.width / srcRect.size.width;
+  heightScaleFactor = dstRect.size.height / srcRect.size.height;
+
+  if (![ctxt isDrawingToScreen])
+    {
+      /* We can't composite or dissolve if we aren't drawing to a screen,
+         so we'll just draw the right part of the image in the right
+         place. */
+      NSPoint p;
+  
+      p.x = dstRect.origin.x / widthScaleFactor - srcRect.origin.x;
+      p.y = dstRect.origin.y / heightScaleFactor - srcRect.origin.y;
+
+      DPSgsave(ctxt);
+      DPSrectclip(ctxt, dstRect.origin.x, dstRect.origin.y,
+                  dstRect.size.width, dstRect.size.height);
+      DPSscale(ctxt, widthScaleFactor, heightScaleFactor);
+      [self drawRepresentation: [self bestRepresentationForDevice: nil]
+            inRect: NSMakeRect(p.x, p.y, imgSize.width, imgSize.height)];
+      DPSgrestore(ctxt);
+
+      return;
+    }
+
+  /* We cannot ask the backend to draw the image directly when the source rect 
+     doesn't cover the whole image.
+     Cairo doesn't support to specify a source rect for a surface used as a 
+     source, see cairo_set_source_surface()).
+     CoreGraphics is similarly limited, see CGContextDrawImage().
+     For now, we always use a two step process:
+     - draw the image data in a cache to apply the srcRect to inRect scaling
+     - draw the cache into the destination context
+     It might be worth to move the first step to the backend, so we don't have 
+     to create a cache window but just an intermediate surface.
+     We create a cache every time but otherwise we are more efficient than the 
+     old code path since the cache size is limited to what we actually draw 
+     and doesn't involve drawing the whole image. */
+  {
+    /* An intermediate image used to scale the image to be drawn as needed */
+    NSCachedImageRep *cache;
+    /* The scaled image graphics state we used as the source from which we 
+       draw into the destination (the current graphics context)*/
+    int gState;
+    /* The context of the cache window */
+    NSGraphicsContext *cacheCtxt;
+    /* The size of the cache window that will hold the scaled image */
+    NSSize cacheSize = NSMakeSize(imgSize.width * widthScaleFactor, 
+      imgSize.height * heightScaleFactor);
+    NSRect srcRectInCache = NSMakeRect(srcRect.origin.x * widthScaleFactor, 
+                                      srcRect.origin.y * heightScaleFactor, 
+                                     srcRect.size.width * widthScaleFactor, 
+                                   srcRect.size.height * heightScaleFactor);
+
+    cache = [[NSCachedImageRep alloc]
+                initWithSize: cacheSize
+                       depth: [[NSScreen mainScreen] depth]
+                    separate: YES
+                       alpha: YES];
+
+    [[[cache window] contentView] lockFocus];
+    cacheCtxt = GSCurrentContext();
+
+    /* Clear the cache window surface */
+    DPScompositerect(cacheCtxt, 0, 0, cacheSize.width, cacheSize.height, NSCompositeClear);
+    gState = [cacheCtxt GSDefineGState];
+
+    //NSLog(@"Draw in cache size %@", NSStringFromSize(cacheSize));
+
+    /* We must not use -drawRepresentation:inRect: because the image must drawn 
+       scaled even when -scalesWhenResized is NO */
+    [[self bestRepresentationForDevice: nil] 
+      drawInRect: NSMakeRect(0, 0, cacheSize.width, cacheSize.height)];
+    /* If we're doing a dissolve, use a DestinationIn composite to lower
+       the alpha of the pixels.  */
+    if (delta != 1.0)
+      {
+        DPSsetalpha(cacheCtxt, delta);
+        DPScompositerect(cacheCtxt, 0, 0, cacheSize.width, cacheSize.height,
+                         NSCompositeDestinationIn);
+      }
+
+    [[[cache window] contentView] unlockFocus];
+
+    //NSLog(@"Draw in %@ from %@ from cache rect %@", NSStringFromRect(dstRect), 
+    //  NSStringFromRect(srcRect), NSStringFromRect(srcRectInCache));
+
+    [ctxt GSdraw: gState
+         toPoint: dstRect.origin
+        fromRect: srcRectInCache
+       operation: op
+        fraction: delta];
+
+    [ctxt GSUndefineGState: gState];
+    DESTROY(cache);
+  }
+}
+
+/* Old code path that can probably partially be merged with the new native implementation.
+Fallback for backends other than Cairo. */
+- (void) guiDrawInRect: (NSRect)dstRect
+              fromRect: (NSRect)srcRect
+             operation: (NSCompositingOperation)op
+              fraction: (float)delta
 {
   NSGraphicsContext *ctxt = GSCurrentContext();
   NSAffineTransform *transform;
@@ -1143,6 +1279,21 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
   }
 }
 
+- (void) drawInRect: (NSRect)dstRect
+           fromRect: (NSRect)srcRect
+          operation: (NSCompositingOperation)op
+           fraction: (float)delta
+{
+  if ([GSCurrentContext() supportsDrawGState])
+  {
+    [self nativeDrawInRect: dstRect fromRect: srcRect operation: op fraction: delta];
+  }
+  else
+  {
+    [self guiDrawInRect: dstRect fromRect: srcRect operation: op fraction: delta];
+  }
+}
+
 - (void) addRepresentation: (NSImageRep *)imageRep
 {
   GSRepData *repd;
@@ -1297,14 +1448,17 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
 {
   NSImageRep* rep;
   NSMutableArray *breps;
-  NSSize dres;
   NSEnumerator *enumerator = [reps objectEnumerator];
+
+  /*
+  NSSize dres;
   NSValue *resolution = [deviceDescription objectForKey: NSDeviceResolution];
 
   if (resolution)
     dres = [resolution sizeValue];
   else
     dres = NSMakeSize(0, 0);
+  */
 
   breps = [NSMutableArray array];
   while ((rep = [enumerator nextObject]) != nil)
@@ -1606,10 +1760,8 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
         }
       if ([coder containsValueForKey: @"NSImageFlags"])
         {
-          int flags;
-          
           //FIXME
-          flags = [coder decodeIntForKey: @"NSImageFlags"];
+          //int flags = [coder decodeIntForKey: @"NSImageFlags"];
         }
       if ([coder containsValueForKey: @"NSReps"])
         {
@@ -1671,16 +1823,6 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
   return self;
 }
 
-- (id) awakeAfterUsingCoder: (NSCoder*)aDecoder
-{
-  if (_name && [nameDict objectForKey: _name]) 
-    {
-      return [nameDict objectForKey: _name];
-    }
-    
-  return self;
-}
-
 + (BOOL) canInitWithPasteboard: (NSPasteboard *)pasteboard
 {
   int i, count;
@@ -2006,8 +2148,16 @@ iterate_reps_for_types(NSArray* imageReps, SEL method)
     }
   [proxy _setResource: image];
   IF_NO_GC([[proxy retain] autorelease]);
+
+  /* Force the image to be archived by name.  This prevents
+   * problems such as when/if gorm is being used with a theme
+   * active, it will not save the image which was loaded
+   * here and will, instead save the name so that the proper
+   * image gets loaded in the future.
+   */
+  image->_flags.archiveByName = YES;
+
   [imageLock unlock];
   return (NSImage*)proxy;
 }
-
 @end
