@@ -24,22 +24,22 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include <Foundation/NSException.h>
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSInputManager.h"
-#include "AppKit/NSInputServer.h"
-#include "AppKit/NSText.h"
-#include "AppKit/NSHelpManager.h"
+#import <Foundation/NSException.h>
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSInputManager.h"
+#import "AppKit/NSInputServer.h"
+#import "AppKit/NSText.h"
+#import "AppKit/NSHelpManager.h"
 
 /* For NSBeep () */
-#include "AppKit/NSGraphics.h"
+#import "AppKit/NSGraphics.h"
 
-#include "GSKeyBindingAction.h"
-#include "GSKeyBindingTable.h"
+#import "GSKeyBindingAction.h"
+#import "GSKeyBindingTable.h"
 
 /* A table mapping character names to characters, used to interpret
    the character names found in KeyBindings dictionaries.  */
-#define CHARACTER_TABLE_SIZE 77
+#define CHARACTER_TABLE_SIZE 78
 
 static struct 
 {
@@ -124,7 +124,8 @@ character_table[CHARACTER_TABLE_SIZE] =
 
   /* Special characters by name.  Useful if you want, for example,
      to associate some special action to C-Tab or similar evils.  */
-  { @"Backspace", NSBackspaceCharacter },
+  { @"Backspace", NSDeleteCharacter },
+  { @"BackTab", NSBackTabCharacter },
   { @"Tab", NSTabCharacter },
   { @"Enter", NSEnterCharacter },
   { @"FormFeed", NSFormFeedCharacter },
@@ -289,7 +290,7 @@ static NSInputManager *currentInputManager = nil;
       NSString *c = [NSString stringWithCharacters: &character  length: 1];
       [description appendString: c];
     }
-  return description;
+  return AUTORELEASE(description);
 }
 
 - (void) loadBindingsFromFile: (NSString *)fullPath
@@ -646,6 +647,7 @@ static NSInputManager *currentInputManager = nil;
 
       switch (character)
         {
+        case NSDeleteCharacter:
         case NSBackspaceCharacter:
           [self doCommandBySelector: @selector (deleteBackward:)];
           break;
@@ -654,6 +656,10 @@ static NSInputManager *currentInputManager = nil;
           [self doCommandBySelector: @selector (cancelOperation:)];
           break;
 
+        case NSBackTabCharacter:
+          [self doCommandBySelector: @selector (insertBacktab:)];
+          break;
+
         case NSTabCharacter:
           if (flags & NSShiftKeyMask)
             {
