@@ -105,22 +105,33 @@ first. Remaining cases, highest priority first:
 
 #include <math.h>
 
-#include "AppKit/NSLayoutManager.h"
-#include "GNUstepGUI/GSLayoutManager_internal.h"
+#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSValue.h>
+#import "AppKit/NSAttributedString.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSLayoutManager.h"
+#import "AppKit/NSParagraphStyle.h"
+#import "AppKit/NSRulerMarker.h"
+#import "AppKit/NSTextContainer.h"
+#import "AppKit/NSTextStorage.h"
+#import "AppKit/NSWindow.h"
+#import "AppKit/DPSOperators.h"
+#import "AppKit/NSBezierPath.h"
 
-#include <Foundation/NSEnumerator.h>
-#include <Foundation/NSException.h>
-#include "AppKit/NSAttributedString.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSParagraphStyle.h"
-#include "AppKit/NSRulerMarker.h"
-#include "AppKit/NSTextContainer.h"
-#include "AppKit/NSTextStorage.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/DPSOperators.h"
+#import "GNUstepGUI/GSLayoutManager_internal.h"
 
 
+@interface NSLayoutManager (spelling)
+
+-(void) _drawSpellingState: (NSInteger)spellingState
+	     forGylphRange: (NSRange)range
+	  lineFragmentRect: (NSRect)fragmentRect
+    lineFragmentGlyphRange: (NSRange)fragmentGlyphRange
+	   containerOrigin: (NSPoint)containerOrigin;
+
+@end
 
 @interface NSLayoutManager (LayoutHelpers)
 -(void) _doLayoutToContainer: (int)cindex  point: (NSPoint)p;
@@ -133,6 +144,25 @@ first. Remaining cases, highest priority first:
 }
 @end
 
+/**
+ * Temporary attributes are implemented using an NSMutableAttributedString
+ * stored in the _temporaryAttributes ivar. We only care about this attributed
+ * string's attributes, not its characters, so to save space,
+ * _temporaryAttributes is initialized with an instance of the following
+ * NSMutableString subclass, which doesn't store any character data.
+ *
+ * WARNING ... it's an unofficial implementation detail that the
+ * GSMutableAttributedString class creates it's internal storage by taking
+ * a mutable copy of its initialisation argument, and we have a comment in
+ * the source to warn about it.  If we change the behavior here, we should
+ * remove the warning comment from the source in base.
+ */
+@interface GSDummyMutableString : NSMutableString
+{
+  NSUInteger _length;
+}
+- (id)initWithLength: (NSUInteger)aLength;
+@end
 
 /* Helper for searching for the line frag of a glyph. */
 #define LINEFRAG_FOR_GLYPH(glyph) \
@@ -157,12 +187,12 @@ first. Remaining cases, highest priority first:
 
 @implementation NSLayoutManager (layout)
 
--(NSPoint) locationForGlyphAtIndex: (unsigned int)glyphIndex
+-(NSPoint) locationForGlyphAtIndex: (NSUInteger)glyphIndex
 {
   NSRange r;
   NSPoint p;
   NSFont *f;
-  unsigned int i;
+  NSUInteger i;
 
   r = [self rangeOfNominallySpacedGlyphsContainingIndex: glyphIndex
 	    startLocation: &p];
@@ -213,9 +243,9 @@ container? necessary? */
 -(NSRect *) rectArrayForGlyphRange: (NSRange)glyphRange
 	  withinSelectedGlyphRange: (NSRange)selGlyphRange
 		   inTextContainer: (NSTextContainer *)container
-			 rectCount: (unsigned int *)rectCount
+			 rectCount: (NSUInteger *)rectCount
 {
-  unsigned int last;
+  NSUInteger last;
   int i;
   textcontainer_t *tc;
   linefrag_t *lf;
@@ -223,7 +253,6 @@ container? necessary? */
   float x0, x1;
   NSRect r;
 
-
   *rectCount = 0;
 
   for (tc = textcontainers, i = 0; i < num_textcontainers; i++, tc++)
@@ -320,7 +349,7 @@ container? necessary? */
 	  The end index is inside the line frag, so we need to find the
 	  exact end location.
 	  */
-	  unsigned int i;
+	  NSUInteger i;
 	  int j;
 	  linefrag_point_t *lp;
 	  glyph_run_t *r;
@@ -351,6 +380,28 @@ container? necessary? */
 	      GLYPH_STEP_FORWARD(r, i, gpos, cpos)
 	    }
 	}
+      else if (lf->pos + lf->length == last)
+	{
+	  /*
+	  The range ends in the last glyph of the line frag, so the end
+	  x-coordinate is the right edge of this glyph. This egde is
+	  equal to the right edge of the line fragment's rectangle if
+	  the glyph is invisible or a control glyph, e.g., a newline,
+	  and to the right edge of the line fragment's used rectangle
+	  otherwise.
+	  */
+	  NSUInteger i;
+	  glyph_run_t *r;
+	  unsigned int gpos, cpos;
+
+	  r = run_for_glyph_index(last - 1, glyphs, &gpos, &cpos);
+	  i = (last - 1) - gpos;
+	  if (!r->glyphs[i].isNotShown && r->glyphs[i].g &&
+	      r->glyphs[i].g != NSControlGlyph)
+	    x1 = NSMaxX(lf->used_rect);
+	  else
+	    x1 = NSMaxX(lf->rect);
+	}
       else
 	{
 	  /*
@@ -383,7 +434,7 @@ container? necessary? */
 	  if (num_rects == rect_array_size)
 	    {
 	      rect_array_size += 4;
-	      rect_array = objc_realloc(rect_array, sizeof(NSRect) * rect_array_size);
+	      rect_array = realloc(rect_array, sizeof(NSRect) * rect_array_size);
 	    }
 	  rect_array[num_rects++] = r;
 	}
@@ -400,7 +451,7 @@ container? necessary? */
 -(NSRect *) rectArrayForCharacterRange: (NSRange)charRange
 	  withinSelectedCharacterRange: (NSRange)selCharRange
 		       inTextContainer: (NSTextContainer *)container
-			     rectCount: (unsigned int *)rectCount
+			     rectCount: (NSUInteger *)rectCount
 {
   NSRange r1, r2;
 
@@ -422,7 +473,7 @@ container? necessary? */
 {
   NSRect *r;
   NSRect result;
-  unsigned int i, c;
+  NSUInteger i, c;
 
 /* TODO: This isn't correct. Need to handle glyphs that extend outside the
 line frag rect. */
@@ -1161,7 +1212,7 @@ container
   unsigned int glyph_pos, char_pos, first_char_pos;
   int i, j;
   NSRect *rects;
-  unsigned int count;
+  NSUInteger count;
   NSColor *color, *last_color;
 
   NSGraphicsContext *ctxt = GSCurrentContext();
@@ -1305,10 +1356,13 @@ attachmentSize(linefrag_t *lf, NSUInteger glyphIndex)
   la = lf->attachments;
   la_i = 0;
 
-  while (la->pos != glyphIndex && la_i < lf->num_attachments)
+  if (la)
     {
-      la++;
-      la_i++;
+      while (la->pos != glyphIndex && la_i < lf->num_attachments)
+        {
+          la++;
+          la_i++;
+        }
     }
 
   if (la_i >= lf->num_attachments)
@@ -1595,7 +1649,7 @@ attachmentSize(linefrag_t *lf, NSUInteger glyphIndex)
 		  */
 		  [self showAttachmentCell: (NSCell*)cell
 				    inRect: cellFrame
-            characterIndex: char_index];
+                            characterIndex: char_index];
 		  [f set];
 		  [color set];
 		}
@@ -1636,6 +1690,312 @@ for (i = 0; i < gbuf_len; i++) printf("   %3i : %04x\n", i, gbuf[i]); */
     }
 
 #undef GBUF_SIZE
+
+  // Draw underline where necessary
+  // FIXME: Also draw strikeout
+  {
+    const NSRange characterRange = [self characterRangeForGlyphRange: range
+						    actualGlyphRange: NULL];
+
+    for (i=characterRange.location; i<NSMaxRange(characterRange); )
+      {
+	NSRange underlinedCharacterRange;
+	id underlineValue = [[self textStorage] attribute: NSUnderlineStyleAttributeName
+						  atIndex: i
+				    longestEffectiveRange: &underlinedCharacterRange
+						  inRange: characterRange];
+	if (underlineValue != nil && [underlineValue integerValue] != NSUnderlineStyleNone)
+	  {
+	    const NSRange underlinedGylphRange = [self glyphRangeForCharacterRange: underlinedCharacterRange
+							      actualCharacterRange: NULL];
+	  
+	    // we have a range of glpyhs that need underlining, which might span
+	    // multiple line fragments, so we need to iterate though the line fragments
+	    for (j=underlinedGylphRange.location; j<NSMaxRange(underlinedGylphRange); )
+	      {
+		NSRange lineFragmentGlyphRange;
+		const NSRect lineFragmentRect = [self lineFragmentRectForGlyphAtIndex: j
+								       effectiveRange: &lineFragmentGlyphRange];
+		const NSRange rangeToUnderline = NSIntersectionRange(underlinedGylphRange, lineFragmentGlyphRange);
+
+		[self underlineGylphRange: rangeToUnderline
+			    underlineType: [underlineValue integerValue]
+			 lineFragmentRect: lineFragmentRect
+		   lineFragmentGlyphRange: lineFragmentGlyphRange
+			  containerOrigin: containerOrigin];
+
+		j = NSMaxRange(rangeToUnderline);
+	      }
+	  }
+	i += underlinedCharacterRange.length;
+      }
+  
+    // Draw spelling state (i.e. red underline for misspelled words)
+
+    if ([NSGraphicsContext currentContextDrawingToScreen])
+      {
+	for (i=characterRange.location; i<NSMaxRange(characterRange); )
+	  {
+	    NSRange underlinedCharacterRange;
+	    id underlineValue = [self temporaryAttribute: NSSpellingStateAttributeName
+					atCharacterIndex: i
+				   longestEffectiveRange: &underlinedCharacterRange
+						 inRange: characterRange];
+	    if (underlineValue != nil && [underlineValue integerValue] != 0)
+	      {
+		const NSRange underlinedGylphRange = [self glyphRangeForCharacterRange: underlinedCharacterRange
+								  actualCharacterRange: NULL];
+		
+		// we have a range of glpyhs that need underlining, which might span
+		// multiple line fragments, so we need to iterate though the line fragments
+		for (j=underlinedGylphRange.location; j<NSMaxRange(underlinedGylphRange); )
+		  {
+		    NSRange lineFragmentGlyphRange;
+		    const NSRect lineFragmentRect = [self lineFragmentRectForGlyphAtIndex: j
+									   effectiveRange: &lineFragmentGlyphRange];
+		    const NSRange rangeToUnderline = NSIntersectionRange(underlinedGylphRange, lineFragmentGlyphRange);
+		    
+		    [self _drawSpellingState: [underlineValue integerValue]
+			       forGylphRange: rangeToUnderline
+			    lineFragmentRect: lineFragmentRect
+		      lineFragmentGlyphRange: lineFragmentGlyphRange
+			     containerOrigin: containerOrigin];
+		    
+		    j = NSMaxRange(rangeToUnderline);
+		  }
+	      }
+	    i += underlinedCharacterRange.length;
+	  }
+      }
+  }
+}
+
+-(void) underlineGylphRange: (NSRange)range
+              underlineType: (NSInteger)type
+           lineFragmentRect: (NSRect)fragmentRect
+     lineFragmentGlyphRange: (NSRange)fragmentGlyphRange
+            containerOrigin: (NSPoint)containerOrigin
+{
+  // FIXME: Implement underlining by word
+  /*if ((type & NSUnderlineByWordMask) != 0)
+    {
+      NSCharacterSet *setToSkip = [NSCharacterSet whitespaceAndNewlineCharacterSet];
+      NSRange characterRange = [self characterRangeForGlyphRange: range
+						actualGlyphRange: NULL];
+      NSString *string = [[self textStorage] string];
+      NSRange examiningRange = range;
+      while (1)
+	{
+	  NSRange nextRangeToSkip = [string rangeOfCharacterInSet: setToSkip
+							   option: 0
+							    range: NSMakeRange(i, range.length - (i - range.location))];
+	  if (nextRangeToSkip.location == NSNotFound)
+	    {
+	    }
+	  else
+	    {
+
+	    }
+        }
+    }
+  else*/
+    {
+      [self drawUnderlineForGlyphRange: range
+                         underlineType: type
+                        baselineOffset: 0 // FIXME:
+		      lineFragmentRect: fragmentRect
+		lineFragmentGlyphRange: fragmentGlyphRange
+		       containerOrigin: containerOrigin];
+    }
+}
+
+
+static void GSDrawPatternLine(NSPoint start, NSPoint end, NSInteger pattern, CGFloat thickness, CGFloat phase)
+{
+  NSBezierPath *path = [NSBezierPath bezierPath];
+  // FIXME: setLineDash should take CGFloat
+  if ((pattern & NSUnderlinePatternDot) == NSUnderlinePatternDot)
+    {
+      const float dot[2] = {2.5 * thickness, 2.5 * thickness};
+      [path setLineDash: dot count: 2 phase: phase];
+    }
+  else if ((pattern & NSUnderlinePatternDash) == NSUnderlinePatternDash)
+    {
+      const float dash[2] = {10 * thickness, 5 * thickness};   
+      [path setLineDash: dash count: 2 phase: phase];
+    }
+  else if ((pattern & NSUnderlinePatternDashDot) == NSUnderlinePatternDashDot)
+    {
+      const float dashdot[4] = {10 * thickness, 3 * thickness, 3 * thickness, 3 * thickness};
+      [path setLineDash: dashdot count: 4 phase: phase];
+    }
+  else if ((pattern & NSUnderlinePatternDashDotDot) == NSUnderlinePatternDashDotDot)
+    {
+      const float dashdotdot[6] = {10 * thickness, 3 * thickness, 3 * thickness, 3 * thickness, 3 * thickness, 3 * thickness};
+      [path setLineDash: dashdotdot count: 6 phase: phase];
+    }
+
+  [path setLineWidth: thickness];
+  [path moveToPoint: start];
+  [path lineToPoint: end];
+  [path stroke];
+}
+
+-(void) drawUnderlineForGlyphRange: (NSRange)underlineRange
+                     underlineType: (NSInteger)type
+                    baselineOffset: (CGFloat)offset
+                  lineFragmentRect: (NSRect)fragmentRect
+            lineFragmentGlyphRange: (NSRange)fragmentGlyphRange
+                   containerOrigin: (NSPoint)containerOrigin
+{       
+  /*NSLog(@"drawUnderlineForGlyphRange:%@ underlineType:%d baselineOffset:%f lineFragmentRect:%@ lineFragmentGlyphRange:%@ containerOrigin:%@",
+		  NSStringFromRange(underlineRange),
+		  (int)type,
+		  (float)offset,
+		  NSStringFromRect(fragmentRect),
+		  NSStringFromRange(fragmentGlyphRange),
+		  NSStringFromPoint(containerOrigin));*/
+  
+  // We have to iterate through the attributes (again..) to find
+  // contiguous regions with the same underline color.
+
+  NSUInteger i;
+  const NSRange characterRange = [self characterRangeForGlyphRange: underlineRange
+						  actualGlyphRange: NULL];
+
+  if (!(underlineRange.location >= fragmentGlyphRange.location &&
+        NSMaxRange(underlineRange) <= NSMaxRange(fragmentGlyphRange)))
+    {
+      NSLog(@"Error, underlineRange must be inside fragmentGlyphRange");
+      return;
+    }
+
+  for (i=characterRange.location; i<NSMaxRange(characterRange); )
+    {
+      NSRange underlineColorCharacterRange, foregroundColorCharacterRange, rangeToDraw;
+      NSColor *underlineColor = (NSColor*)[[self textStorage] attribute: NSUnderlineColorAttributeName
+								atIndex: i
+						  longestEffectiveRange: &underlineColorCharacterRange
+								inRange: NSMakeRange(i, NSMaxRange(characterRange)-i)];
+      NSColor *foregroundColor = (NSColor*)[[self textStorage] attribute: NSForegroundColorAttributeName
+								 atIndex: i
+						   longestEffectiveRange: &foregroundColorCharacterRange
+								 inRange: NSMakeRange(i, NSMaxRange(characterRange)-i)];
+      /*NSLog(@"asked to underline '%@'. at %d, ul color is %@ (%@) and fg color is %@ (%@)", [[[self textStorage] string] substringWithRange: characterRange],
+	    i,
+	    underlineColor,
+	    NSStringFromRange(underlineColorCharacterRange),
+	    foregroundColor,
+	    NSStringFromRange(foregroundColorCharacterRange));*/
+
+      if (underlineColor != nil)
+	{
+	  [underlineColor set];
+	}
+      else if (foregroundColor != nil)
+	{
+	  [foregroundColor set];
+	}
+      else 
+	{
+	  [[NSColor textColor] set];
+	}
+
+      // Draw the smaller range
+      rangeToDraw = underlineColorCharacterRange.length < foregroundColorCharacterRange.length ?
+	underlineColorCharacterRange : foregroundColorCharacterRange;
+      
+      // do the actual underline
+      {
+
+	  const NSRange glyphRangeToDraw = [self glyphRangeForCharacterRange: rangeToDraw
+						       actualCharacterRange: NULL];
+
+	  // FIXME: find the largest font within the range to underline
+	  NSFont *largestFont = [self effectiveFontForGlyphAtIndex: glyphRangeToDraw.location // NOTE: GS private method
+						     range: NULL];
+
+	  const CGFloat underlineWidth = [largestFont pointSize] * 
+	    (((type & NSUnderlineStyleDouble) != 0) ? 0.05 : 0.07);
+	  
+	  NSPoint start = [self locationForGlyphAtIndex: glyphRangeToDraw.location];
+	  NSPoint end = [self locationForGlyphAtIndex: NSMaxRange(glyphRangeToDraw) - 1]; //FIXME: check length > 0
+
+	  // FIXME: remove this hack lowers the underline slightly
+	  start.y += [largestFont pointSize] * 0.07;
+	  end.y += [largestFont pointSize] * 0.07;
+
+	  end.x += [largestFont advancementForGlyph: [self glyphAtIndex: (NSMaxRange(glyphRangeToDraw) - 1)]].width;
+	  
+	  start = NSMakePoint(start.x + containerOrigin.x + fragmentRect.origin.x, start.y + containerOrigin.y + fragmentRect.origin.y);
+	  end = NSMakePoint(end.x + containerOrigin.x + fragmentRect.origin.x, end.y + containerOrigin.y + fragmentRect.origin.y);
+	  
+	  if ((type & NSUnderlineStyleDouble) == NSUnderlineStyleDouble)
+	    {
+	      GSDrawPatternLine(NSMakePoint(start.x, start.y - (underlineWidth / 2)), 
+				NSMakePoint(end.x, end.y - (underlineWidth / 2)),
+				type, underlineWidth / 2, start.x);
+	      GSDrawPatternLine(NSMakePoint(start.x, start.y + (underlineWidth / 2)), 
+				NSMakePoint(end.x, end.y + (underlineWidth / 2)),
+				type, underlineWidth / 2, start.x);
+	    }
+	  else
+	    {
+	      GSDrawPatternLine(start, end, type, underlineWidth, start.x);
+	    }
+	}
+
+      i += rangeToDraw.length;
+    }
+}
+
+@end
+
+@implementation NSLayoutManager (spelling)
+
+-(void) _drawSpellingState: (NSInteger)spellingState
+	     forGylphRange: (NSRange)range
+	  lineFragmentRect: (NSRect)fragmentRect
+    lineFragmentGlyphRange: (NSRange)fragmentGlyphRange
+	   containerOrigin: (NSPoint)containerOrigin
+{
+  NSBezierPath *path;
+  const float pattern[2] = {2.5, 1.0};
+  NSFont *largestFont = [self effectiveFontForGlyphAtIndex: range.location // NOTE: GS private method
+						     range: NULL];
+  NSPoint start = [self locationForGlyphAtIndex: range.location];
+  NSPoint end = [self locationForGlyphAtIndex: NSMaxRange(range) - 1]; //FIXME: check length > 0
+
+  if (spellingState == 0)
+    {
+      return;
+    }
+  
+  // FIXME: calculate the underline position correctly, using the font on both the start and end glyph
+  start.y += [largestFont pointSize] * 0.07;
+  end.y += [largestFont pointSize] * 0.07;
+
+  end.x += [largestFont advancementForGlyph: [self glyphAtIndex: (NSMaxRange(range) - 1)]].width;
+	  
+  start = NSMakePoint(start.x + containerOrigin.x + fragmentRect.origin.x, start.y + containerOrigin.y + fragmentRect.origin.y);
+  end = NSMakePoint(end.x + containerOrigin.x + fragmentRect.origin.x, end.y + containerOrigin.y + fragmentRect.origin.y);
+
+  path = [NSBezierPath bezierPath];
+  [path setLineDash: pattern count: 2 phase: 0];
+  [path setLineWidth: 1.5];
+  [path moveToPoint: start];
+  [path lineToPoint: end];
+
+  if ((spellingState & NSSpellingStateGrammarFlag) != 0)
+    {
+      [[NSColor greenColor] set];
+    }
+  else
+    {
+      [[NSColor redColor] set];
+    }
+
+  [path stroke];	  
 }
 
 @end
@@ -1675,6 +2035,7 @@ for (i = 0; i < gbuf_len; i++) printf("   %3i : %04x\n", i, gbuf[i]); */
 -(void) dealloc
 {
   DESTROY(_typingAttributes);
+  DESTROY(_temporaryAttributes);
   [super dealloc];
 }
 
@@ -1768,6 +2129,7 @@ dictionary.
 		   imageOrigin: NSMakePoint(0, 0)];
       [marker setRepresentedObject: tab];
       [markers addObject: marker];
+      RELEASE(marker);
     }
 
   return markers;
@@ -1933,6 +2295,32 @@ this file describes this.
   if (!(mask & NSTextStorageEditedCharacters))
     lengthChange = 0;
 
+  if (_temporaryAttributes != nil && (mask & NSTextStorageEditedCharacters) != 0)
+    {
+      int i;
+      NSArray *attrs;
+      NSRange oldRange = NSMakeRange(range.location, range.length - lengthChange);
+
+      NSString *replacementString = [[GSDummyMutableString alloc] initWithLength: range.length];
+      [_temporaryAttributes replaceCharactersInRange: oldRange
+					  withString: replacementString];
+      [replacementString release];
+      
+      // In addition, clear any temporary attributes that may have been extended
+      // over the affected range
+
+      if (range.length > 0)
+	{
+	  attrs = [[self temporaryAttributesAtCharacterIndex: range.location
+					      effectiveRange: NULL] allKeys];
+	  for (i=0; i<[attrs count]; i++)
+	    {
+	      [self removeTemporaryAttribute: [attrs objectAtIndex: i]
+			   forCharacterRange: range];
+	    }
+	}
+    }
+
   [self invalidateGlyphsForCharacterRange: invalidatedRange
 	changeInLength: lengthChange
 	actualCharacterRange: &r];
@@ -2088,12 +2476,12 @@ this file describes this.
 		}
 	      if (lf->points)
 		{
-		  objc_free(lf->points);
+		  free(lf->points);
 		  lf->points = NULL;
 		}
 	      if (lf->attachments)
 		{
-		  objc_free(lf->attachments);
+		  free(lf->attachments);
 		  lf->attachments = NULL;
 		}
 	    }
@@ -2208,12 +2596,12 @@ no_soft_invalidation:
 	    {
 	      if (lf->points)
 		{
-		  objc_free(lf->points);
+		  free(lf->points);
 		  lf->points = NULL;
 		}
 	      if (lf->attachments)
 		{
-		  objc_free(lf->attachments);
+		  free(lf->attachments);
 		  lf->attachments = NULL;
 		}
 	    }
@@ -2307,6 +2695,24 @@ no_soft_invalidation:
 }
 
 
+- (void) encodeWithCoder: (NSCoder*)aCoder
+{
+  if ([aCoder allowsKeyedCoding])
+    {
+      int flags =
+	// FIXME attribute not yet supported by GNUstep
+	//defaultAttachementScaling |
+	(backgroundLayoutEnabled ? 0x04 : 0) |
+	(showsInvisibleCharacters ? 0x08 : 0) |
+	(showsControlCharacters ? 0x10 : 0);
+
+      [aCoder encodeObject: [self textContainers] forKey: @"NSTextContainers"];
+      [aCoder encodeObject: [self textStorage] forKey: @"NSTextStorage"];
+      [aCoder encodeObject: [self delegate] forKey: @"NSDelegate"];
+      [aCoder encodeInt: flags forKey: @"NSLMFlags"];
+    }
+}
+
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
   self = [self init];
@@ -2322,9 +2728,12 @@ no_soft_invalidation:
       if ([aDecoder containsValueForKey: @"NSLMFlags"])
         {
 	  flags = [aDecoder decodeIntForKey: @"NSLMFlags"];
-	  // nothing really to do with these flags....
-	  // they are runtime flags which, even if set into the archive
-	  // they are ignored...
+
+	  // FIXME attribute not yet supported by GNUstep
+	  //defaultAttachementScaling = (NSImageScaling)(flags & 0x03);
+	  backgroundLayoutEnabled = (flags & 0x04) != 0;
+	  showsInvisibleCharacters = (flags & 0x08) != 0;
+	  showsControlCharacters = (flags & 0x10) != 0;
 	}
       [self setDelegate: delegate];
       [storage addLayoutManager: self];
@@ -2342,48 +2751,116 @@ no_soft_invalidation:
 
 @end
 
-@implementation NSLayoutManager (temporaryattributes)
-- (void) addTemporaryAttributes: (NSDictionary *)attrs forCharacterRange: (NSRange)range
-{
-  // to be implemented
-}
 
-- (void) addTemporaryAttribute: (NSString *)attr value: (id)value forCharacterRange: (NSRange)range
+@implementation GSDummyMutableString
+- (id)initWithLength: (NSUInteger)aLength
 {
-  // to be implemented
+  self = [super init];
+  if (self != nil)
+    {
+      self->_length = aLength;
+    }
+  return self;
 }
-
-- (void) setTemporaryAttributes:forCharacterRange: (NSRange)range
+- (NSUInteger)length
 {
-  // to be implemented
+  return _length;
 }
-
-- (void) removeTemporaryAttribute: (NSString *)attr forCharacterRange: (NSRange)range
+- (unichar)characterAtIndex: (NSUInteger)index
 {
-  // to be implemented
+  return 0;
 }
-
-- (id) temporaryAttribute: (NSString *)attr atCharacterIndex: (unsigned int)index effectiveRange: (NSRange)range
+- (void)replaceCharactersInRange:(NSRange)range withString:(NSString *)aString
 {
-  // to be implemented
-  return nil;
+  _length = (_length - range.length) + [aString length];
 }
-
-- (id) temporaryAttribute: (NSString *)attr atCharacterIndex: (unsigned int)index longestEffectiveRange: (NSRange*)longestRange inRange: (NSRange)range
+- (id) copyWithZone: (NSZone*)zone
 {
-  // to be implemented
-  return nil;
+  return [self mutableCopyWithZone: zone];
 }
-
-- (NSDictionary *) temporaryAttributesAtCharacterIndex: (unsigned int)index effectiveRange: (NSRange)range
+- (id) mutableCopyWithZone: (NSZone*)zone
 {
-  // to be implemented
-  return nil;
-}
-
-- (NSDictionary *) temporaryAttributesAtCharacterIndex: (unsigned int) longestEffectiveRange: (NSRange*)longestRange inRange: (NSRange)range
-{
-  // to be implemented
-  return nil;
+  return [[GSDummyMutableString allocWithZone: zone] initWithLength: _length];
 }
 @end
+
+@implementation NSLayoutManager (temporaryattributes)
+
+- (NSMutableAttributedString*) _temporaryAttributes
+{
+  if (_temporaryAttributes == nil)
+    {
+      NSString *dummyString = [[GSDummyMutableString alloc] initWithLength: [[self textStorage] length]];
+      _temporaryAttributes = [[NSMutableAttributedString alloc] initWithString: dummyString];
+      [dummyString release];
+    }
+  return _temporaryAttributes;
+}
+
+- (void) setTemporaryAttributes: (NSDictionary *)attrs 
+              forCharacterRange: (NSRange)range
+{
+  [[self _temporaryAttributes] setAttributes: attrs range: range];
+  [self invalidateDisplayForCharacterRange: range];
+}
+
+- (NSDictionary *) temporaryAttributesAtCharacterIndex: (NSUInteger)index 
+                                        effectiveRange: (NSRange*)longestRange
+{
+  return [[self _temporaryAttributes] attributesAtIndex: index effectiveRange: longestRange];
+}
+
+- (void) addTemporaryAttributes: (NSDictionary *)attrs 
+              forCharacterRange: (NSRange)range
+{
+  [[self _temporaryAttributes] addAttributes: attrs range: range];
+  [self invalidateDisplayForCharacterRange: range];
+}
+
+- (void) addTemporaryAttribute: (NSString *)attr 
+                         value: (id)value 
+             forCharacterRange: (NSRange)range
+{
+  [[self _temporaryAttributes] addAttribute: attr value: value range: range];
+  [self invalidateDisplayForCharacterRange: range];
+}
+
+- (void) removeTemporaryAttribute: (NSString *)attr 
+                forCharacterRange: (NSRange)range
+{
+  [[self _temporaryAttributes] removeAttribute: attr range: range];
+  [self invalidateDisplayForCharacterRange: range];
+}
+
+- (id) temporaryAttribute: (NSString *)attr 
+         atCharacterIndex: (NSUInteger)index 
+           effectiveRange: (NSRange*)range
+{
+  return [[self _temporaryAttributes] attribute: attr atIndex: index effectiveRange: range];
+}
+
+- (id) temporaryAttribute: (NSString *)attr 
+         atCharacterIndex: (NSUInteger)index 
+    longestEffectiveRange: (NSRange*)longestRange 
+                  inRange: (NSRange)range
+{
+  return [[self _temporaryAttributes] attribute: attr atIndex: index longestEffectiveRange: longestRange inRange: range];
+}
+
+- (NSDictionary *) temporaryAttributesAtCharacterIndex: (NSUInteger)index
+                                 longestEffectiveRange: (NSRange*)longestRange 
+                                               inRange: (NSRange)range
+{
+  return [[self _temporaryAttributes] attributesAtIndex: index longestEffectiveRange: longestRange inRange: range];
+}
+
+/**
+ * Most of this is implemented in GSLayoutManager
+ */
+- (void) setTextStorage: (NSTextStorage *)aTextStorage
+{
+  DESTROY(_temporaryAttributes);
+  [super setTextStorage: aTextStorage];
+}
+
+@end
