@@ -36,6 +36,7 @@
 #import "config.h"
 #import <Foundation/NSCoder.h>
 #import <Foundation/NSArray.h>
+#import <Foundation/NSCharacterSet.h>
 #import <Foundation/NSDebug.h>
 #import <Foundation/NSException.h>
 #import <Foundation/NSProcessInfo.h>
@@ -62,6 +63,7 @@
 #import "AppKit/NSAttributedString.h"
 
 #import "GSGuiPrivate.h"
+#import "NSDocumentFrameworkPrivate.h"
 #import "GNUstepGUI/GSTheme.h"
 
 /*
@@ -117,6 +119,7 @@
 {
   NSMenu *_the_menu;
 }
+- (void) _setmenu: (NSMenu *)menu;
 @end
 
 @interface NSMenuView (GNUstepPrivate)
@@ -132,11 +135,14 @@ static BOOL menuBarVisible = YES;
 
 @interface	NSMenu (GNUstepPrivate)
 
+- (NSString *) _name;
+- (void) _setName: (NSString *)name;
 - (NSMenuPanel *) _createWindow;
 - (NSString *) _locationKey;
 - (void) _rightMouseDisplay: (NSEvent*)theEvent;
 - (void) _setGeometry;
 - (void) _updateUserDefaults: (id) notification;
+- (void) _organizeMenu;
 
 @end
 
@@ -172,6 +178,16 @@ static BOOL menuBarVisible = YES;
 
 @implementation	NSMenu (GNUstepPrivate)
 
+- (NSString *) _name;
+{
+  return _name;
+}
+
+- (void) _setName: (NSString *)aName
+{
+  ASSIGNCOPY(_name, aName);
+}
+
 - (NSString*) _locationKey
 {
   NSInterfaceStyle style = NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil);
@@ -258,16 +274,19 @@ static BOOL menuBarVisible = YES;
           NSMutableArray *itemsToMove;
           NSImage *ti;
           float bar;
+	  
+	  if([[GSTheme theme] menuShouldShowIcon])
+	    {
+	      ti = [[NSApp applicationIconImage] copy];
+	      if (ti == nil)
+		{
+		  ti = [[NSImage imageNamed: @"GNUstep"] copy];
+		}
+	      [ti setScalesWhenResized: YES];
+	      bar = [NSMenuView menuBarHeight] - 4;
+	      [ti setSize: NSMakeSize(bar, bar)];
+	    }
 
-          ti = [[NSApp applicationIconImage] copy];
-          if (ti == nil)
-            {
-              ti = [[NSImage imageNamed: @"GNUstep"] copy];
-            }
-          [ti setScalesWhenResized: YES];
-          bar = [NSMenuView menuBarHeight] - 4;
-          [ti setSize: NSMakeSize(bar, bar)];
-          
           itemsToMove = [NSMutableArray new];
           
           if (appMenu == nil)
@@ -293,9 +312,13 @@ static BOOL menuBarVisible = YES;
                   RELEASE (appItem);
                 }
             }
-          [appItem setImage: ti];
-          RELEASE(ti);
-          
+
+	  if([[GSTheme theme] menuShouldShowIcon])
+	    {
+	      [appItem setImage: ti];
+	      RELEASE(ti);
+	    }
+
           // Collect all simple items plus "Info" and "Services"
           for (i = 1; i < [_items count]; i++)
             {
@@ -540,12 +563,8 @@ static BOOL menuBarVisible = YES;
 
 - (void) _rightMouseDisplay: (NSEvent*)theEvent
 {
-  if (_menu.horizontal == NO)
-    {
-      [self displayTransient];
-      [_view mouseDown: theEvent];
-      [self closeTransient];
-    }
+  [[GSTheme theme] rightMouseDisplay: self
+			    forEvent: theEvent];
 }
 
 @end
@@ -608,6 +627,7 @@ static BOOL menuBarVisible = YES;
   RELEASE(_view);
   RELEASE(_aWindow);
   RELEASE(_bWindow);
+  RELEASE(_name);
 
   [super dealloc];
 }
@@ -934,7 +954,7 @@ static BOOL menuBarVisible = YES;
     {
       NSMenuItem *menuItem = [_items objectAtIndex: i];
 
-      if (actionSelector == 0 || sel_eq([menuItem action], actionSelector))
+      if (actionSelector == 0 || sel_isEqual([menuItem action], actionSelector))
         {
           // There are different possibilities to implement the check here
           if ([menuItem target] == anObject)
@@ -1214,10 +1234,15 @@ static BOOL menuBarVisible = YES;
   unsigned      i;
   unsigned      count = [_items count];
   NSEventType   type = [theEvent type];
-  unsigned	modifiers = [theEvent modifierFlags];
+  unsigned int modifiers = [theEvent modifierFlags];
   NSString	*keyEquivalent = [theEvent charactersIgnoringModifiers];
+  unsigned int relevantModifiersMask = NSCommandKeyMask | NSAlternateKeyMask | NSControlKeyMask;
+  /* Take shift key into account only for control keys and arrow and function keys */
+  if ((modifiers & NSFunctionKeyMask)
+      || ([keyEquivalent length] > 0 && [[NSCharacterSet controlCharacterSet] characterIsMember:[keyEquivalent characterAtIndex:0]]))
+    relevantModifiersMask |= NSShiftKeyMask;
 
-  if (type != NSKeyDown && type != NSKeyUp) 
+  if ((type != NSKeyDown && type != NSKeyUp) || [keyEquivalent length] == 0)
     return NO;
              
   for (i = 0; i < count; i++)
@@ -1226,7 +1251,21 @@ static BOOL menuBarVisible = YES;
                                     
       if ([item hasSubmenu])
         {
+	  /* Ignore the Services submenu during menu traversal so that its key
+	     equivalents do not accidentally shadow standard key equivalents
+	     in the application's own menus. NSApp calls -performKeyEquivalent:
+	     explicitly for the Services menu when no matching key equivalent
+	     was found here (see NSApplication -sendEvent:).
+	     Note: Shadowing is no problem for a standard OpenStep menu, where
+	     the Services menu appears close to the end of the main menu, but
+	     is very likely for Macintosh or Windows 95 interface styles, where
+	     the Services menu appears in the first submenu of the main menu. */
+	  // FIXME Should really remove conflicting key equivalents from the
+	  // menus so that users don't get confused.
+	  if ([item submenu] == [NSApp servicesMenu])
+	    continue;
 	  // Recurse through submenus whether active or not.
+          // Recurse through submenus whether active or not.
           if ([[item submenu] performKeyEquivalent: theEvent])
             {
               // The event has been handled by an item in the submenu.
@@ -1235,18 +1274,18 @@ static BOOL menuBarVisible = YES;
         }
       else
         {
-	  unsigned	mask = [item keyEquivalentModifierMask];
+          unsigned int mask = [item keyEquivalentModifierMask];
 
           if ([[item keyEquivalent] isEqualToString: keyEquivalent] 
-	    && (modifiers & mask) == mask)
-	    {
-	      if ([item isEnabled])
-		{
-		  [_view performActionWithHighlightingForItemAtIndex: i];
-		}
-	      return YES;
-	    }
-	}
+            && (modifiers & relevantModifiersMask) == (mask & relevantModifiersMask))
+            {
+              if ([item isEnabled])
+                {
+                  [_view performActionWithHighlightingForItemAtIndex: i];
+                }
+              return YES;
+            }
+        }
     }
   return NO; 
 }
@@ -1451,14 +1490,12 @@ static BOOL menuBarVisible = YES;
   NSRect oldWindowFrame;
   NSRect newWindowFrame;
   NSRect menuFrame;
-  NSSize size;
 
   [_view sizeToFit];
   
   menuFrame = [_view frame];
-  size = menuFrame.size;
  
-	// Main
+  // Main
   oldWindowFrame = [_aWindow frame];
   newWindowFrame = [NSWindow frameRectForContentRect: menuFrame
                              styleMask: [_aWindow styleMask]];
@@ -1566,6 +1603,7 @@ static BOOL menuBarVisible = YES;
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
   NSString	*dTitle;
+  NSString	*dName;
   NSArray	*dItems;
   BOOL		dAuto;
   unsigned	i;
@@ -1588,15 +1626,25 @@ static BOOL menuBarVisible = YES;
 	}
       dTitle = [aDecoder decodeObjectForKey: @"NSTitle"];
       dItems = [aDecoder decodeObjectForKey: @"NSMenuItems"];
+      if ([aDecoder containsValueForKey: @"NSName"])
+        {
+	  dName = [aDecoder decodeObjectForKey: @"NSName"];
+	}
+      else
+	{
+	  dName = nil;
+	}
     }
   else
     {
       dTitle = [aDecoder decodeObject];
       dItems = [aDecoder decodeObject];
+      dName = nil;
       [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &dAuto];
     }
   self = [self initWithTitle: dTitle];
   [self setAutoenablesItems: dAuto];
+  [self _setName: dName];
 
   [self setMenuChangedMessagesEnabled: NO];
   /*
@@ -1612,6 +1660,41 @@ static BOOL menuBarVisible = YES;
   return self;
 }
 
+- (void) awakeFromNib
+{
+  NSString *name = [self _name];
+
+  if (name)
+    {
+      if ([name isEqualToString: @"_NSMainMenu"])
+        {
+          // NB This is already handled by the nib loading code
+          //[NSApp setMainMenu: self];
+        }
+      else if ([name isEqualToString: @"_NSAppleMenu"])
+        {
+          // GNUstep does not handle Apple's application menu specially
+        }
+      else if ([name isEqualToString: @"_NSWindowsMenu"])
+        {
+          [NSApp setWindowsMenu: self];
+        }
+      else if ([name isEqualToString: @"_NSServicesMenu"])
+        {
+          [NSApp setServicesMenu: self];
+        }
+      else if ([name isEqualToString: @"_NSRecentDocumentsMenu"])
+        {
+          [[NSDocumentController sharedDocumentController]
+                  _setRecentDocumentsMenu: self];
+        }
+      else if ([name isEqualToString: @"_NSFontMenu"])
+        {
+          [[NSFontManager sharedFontManager] setFontMenu: self];
+        }
+    }
+}
+
 /*
  * NSCopying Protocol
  */
@@ -1625,8 +1708,9 @@ static BOOL menuBarVisible = YES;
   for (i = 0; i < count; i++)
     {
       // This works because the copy on NSMenuItem sets the menu to nil!!!
-      [new insertItem: [[_items objectAtIndex: i] copyWithZone: zone]
-	   atIndex: i];
+      NSMenuItem *item = [[_items objectAtIndex: i] copyWithZone: zone];
+      [new insertItem: item atIndex: i];
+      RELEASE(item);
     }
   
   return new;
@@ -1691,7 +1775,11 @@ static BOOL menuBarVisible = YES;
 
 - (void) applicationDidFinishLaunching:(NSNotification *)notification
 {
-  [[GSTheme theme] updateAllWindowsWithMenu: [NSApp mainMenu]];
+  if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil) == 
+      NSWindows95InterfaceStyle)
+    {
+      [[GSTheme theme] updateAllWindowsWithMenu: [NSApp mainMenu]];
+    }
   [self _showTornOffMenuIfAny:notification];
 }
 
@@ -1793,13 +1881,26 @@ static BOOL menuBarVisible = YES;
   else
     {
       NSRect	frame = [_aWindow frame];
+      NSInterfaceStyle style;
 
       location = [_aWindow mouseLocationOutsideOfEventStream];
       location = [_aWindow convertBaseToScreen: location];
-      location.x -= frame.size.width/2;
-      if (location.x < 0)
-	location.x = 0;
-      location.y -= frame.size.height - 10;
+      location.y -= frame.size.height;
+
+      /* When using the standard NextStep/OpenStep interface style, the
+	 center of the menu's title view is placed below the mouse cursor.
+	 However, in Macintosh and Windows95 styles, menus have no visible
+	 title. To prevent the user from accidentally selecting the first
+	 item, the top left edge is placed below the mouse cursor for them. */
+      style = NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil);
+      if (style != NSWindows95InterfaceStyle &&
+	  style != NSMacintoshInterfaceStyle)
+	{
+	  location.x -= frame.size.width/2;
+	  if (location.x < 0)
+	    location.x = 0;
+	  location.y += 10;
+	}
     }
 
   [_bWindow setFrameOrigin: location];
@@ -1945,7 +2046,12 @@ static BOOL menuBarVisible = YES;
         {
           [self display];
         }
-      [[GSTheme theme] updateAllWindowsWithMenu: self];
+
+      if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil) == 
+	  NSWindows95InterfaceStyle)
+	{
+	  [[GSTheme theme] updateAllWindowsWithMenu: self];
+	}
     }
   else 
     {
@@ -1983,25 +2089,34 @@ static BOOL menuBarVisible = YES;
   NSRect    screenRect = [[theWindow screen] visibleFrame];
   NSPoint   vector    = {0.0, 0.0};
   BOOL      moveIt    = NO;
+  NSPoint location = [theWindow mouseLocationOutsideOfEventStream];
+  NSPoint pointerLoc = [theWindow convertBaseToScreen: location];
+  NSInterfaceStyle style = NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil);
   
-  // If we are the main menu forget about moving.
-  if ([self isEqual: [NSApp mainMenu]] && !_menu.transient)
+  // Don't move the main menu bar in Macintosh interface style, this is
+  // annoying (in particular, since the effective screen range is reduced
+  // by the height of the menu bar!)
+  if (style == NSMacintoshInterfaceStyle && [self isEqual: [NSApp mainMenu]])
     return;
 
   // 1 - determine the amount we need to shift in the y direction.
-  if (NSMinY (frameRect) < 0)
+  if (pointerLoc.y <= 1 && NSMinY (frameRect) < 0)
     {
       vector.y = MIN (SHIFT_DELTA, -NSMinY (frameRect));
       moveIt = YES;
     }
-  else if (NSMaxY (frameRect) > NSMaxY (screenRect))
+  /* Note: pointerLoc.y may be greater than NSMaxY(screenRect) if we have a
+     horizontal menu bar at the top of the screen (Macintosh interface style) */
+  // FIXME Don't move the horizontal menu bar downward in this case!
+  else if (pointerLoc.y >= NSMaxY(screenRect) &&
+	   NSMaxY (frameRect) > NSMaxY (screenRect))
     {
       vector.y = -MIN (SHIFT_DELTA, NSMaxY (frameRect) - NSMaxY (screenRect));
       moveIt = YES;
     }
 
   // 2 - determine the amount we need to shift in the x direction.
-  if (NSMinX (frameRect) < 0)
+  if (pointerLoc.x == 0 && NSMinX (frameRect) < 0)
     {
       vector.x = MIN (SHIFT_DELTA, -NSMinX (frameRect));
       moveIt = YES;
@@ -2009,7 +2124,8 @@ static BOOL menuBarVisible = YES;
   // Note the -3.  This is done so the menu, after shifting completely
   // has some spare room on the right hand side.  This is needed otherwise
   // the user can never access submenus of this menu.
-  else if (NSMaxX (frameRect) > NSMaxX (screenRect) - 3)
+  else if (pointerLoc.x == NSMaxX(screenRect) - 1 &&
+	   NSMaxX (frameRect) > NSMaxX (screenRect) - 3)
     {
       vector.x
 	= -MIN (SHIFT_DELTA, NSMaxX (frameRect) - NSMaxX (screenRect) + 3);
@@ -2021,9 +2137,9 @@ static BOOL menuBarVisible = YES;
       NSPoint  masterLocation;
       NSPoint  destinationPoint;
       
-      if (_menu.horizontal)
+      if (style == NSMacintoshInterfaceStyle || _menu.horizontal)
         {
-          masterLocation = [[self window] frame].origin;
+          masterLocation = frameRect.origin;
           destinationPoint.x = masterLocation.x + vector.x;
           destinationPoint.y = masterLocation.y + vector.y;
           [self nestedSetFrameOrigin: destinationPoint];
@@ -2036,6 +2152,7 @@ static BOOL menuBarVisible = YES;
           // Look for the "master" menu, i.e. the one to move from.
           for (candidateMenu = masterMenu = self;
                (candidateMenu = masterMenu->_superMenu)
+                   && !candidateMenu->_menu.horizontal
                    && (!masterMenu->_menu.is_tornoff
                        || masterMenu->_menu.transient);
                masterMenu = candidateMenu);
