@@ -25,29 +25,34 @@
 */ 
 
 #include "config.h"
-#include <Foundation/NSArray.h>
-#include <Foundation/NSCoder.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSProcessInfo.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSUserDefaults.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSCoder.h>
+#import <Foundation/NSDictionary.h>
+//#import <Foundation/NSException.h>
+#import <Foundation/NSString.h>
+//#import <Foundation/NSNotification.h>
+//#import <Foundation/NSProcessInfo.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSUserDefaults.h>
 
-#include "AppKit/NSAttributedString.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSFont.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSMenu.h"
-#include "AppKit/NSMenuItemCell.h"
-#include "AppKit/NSMenuView.h"
-#include "AppKit/NSParagraphStyle.h"
-#include "GNUstepGUI/GSTheme.h"
-#include "GSGuiPrivate.h"
+#import "AppKit/NSAttributedString.h"
+#import "AppKit/NSBezierPath.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSFont.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSMenu.h"
+#import "AppKit/NSMenuItemCell.h"
+#import "AppKit/NSMenuView.h"
+#import "AppKit/NSParagraphStyle.h"
+#import "GNUstepGUI/GSTheme.h"
+#import "GSGuiPrivate.h"
 
+static NSString *controlKeyString = @"^";
+static NSString *alternateKeyString = @"+";
+static NSString *shiftKeyString = @"/";
+static NSString *commandKeyString = @"#";
 
 @implementation NSMenuItemCell
 
@@ -55,7 +60,30 @@
 {
   if (self == [NSMenuItemCell class])
     {
+      NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
+      NSString *keyString;
+
       [self setVersion: 2];
+      keyString = [userDefaults objectForKey: @"GSControlKeyString"];
+      if (nil != keyString)
+	{
+	  controlKeyString = [keyString retain];
+	}
+      keyString = [userDefaults objectForKey: @"GSAlternateKeyString"];
+      if (nil != keyString)
+	{
+	  alternateKeyString = [keyString retain];
+	}
+      keyString = [userDefaults objectForKey: @"GSShiftKeyString"];
+      if (nil != keyString)
+	{
+	  shiftKeyString = [keyString retain];
+	}
+      keyString = [userDefaults objectForKey: @"GSCommandKeyString"];
+      if (nil != keyString)
+	{
+	  commandKeyString = [keyString retain];
+	}
     }
 }
 
@@ -66,7 +94,10 @@
 
 - (id) init
 {
-  [super init];
+  self = [super init];
+  if (nil == self)
+    return nil;
+
   [self setButtonType: NSMomentaryLightButton];
   [self setAlignment: NSLeftTextAlignment];
   [self setFont: [NSFont menuFontOfSize: 0]];
@@ -160,7 +191,7 @@
   return _menuItem;
 }
 
-- (int) tag
+- (NSInteger) tag
 {
   return [[self menuItem] tag];
 }
@@ -196,7 +227,6 @@
   NSString *key = [_menuItem keyEquivalent];
   unsigned int m = [_menuItem keyEquivalentModifierMask];
   NSString *ucKey = [key uppercaseString];
-  BOOL shift;
   unichar uchar;
 
   if ((key == nil) || [key isEqualToString: @""])
@@ -215,17 +245,21 @@
   else if ([key isEqualToString: @"\\e"])
     key = @"ESC";
   else if ([key isEqualToString: @"\\d"])
-    key=@"DEL";
+    key = @"DEL";
+
+  if (m != 0)
+    {
+      BOOL shift;
+      // shift mask and not an upper case string?
+      shift = (m & NSShiftKeyMask) & ![key isEqualToString: ucKey];
+      key = [NSString stringWithFormat:@"%@%@%@%@%@",
+                      (m & NSControlKeyMask) ? controlKeyString : @"",
+                      (m & NSAlternateKeyMask) ? alternateKeyString : @"",
+                      shift ? shiftKeyString : @"",
+                      (m & NSCommandKeyMask) ? commandKeyString : @"",
+                      key];
+    }
 
-  // shift mask and not an upper case string?
-  shift = (m & NSShiftKeyMask) & ![key isEqualToString: ucKey];
-  
-  key = [NSString stringWithFormat:@"%@%@%@%@%@",
-                  (m & NSControlKeyMask) ? @"^" : @"",
-                  (m & NSAlternateKeyMask) ? @"+" : @"",
-                  shift ? @"/" : @"",
-                  (m & NSCommandKeyMask) ? @"#" : @"",
-                  key];
   return key;
 }
 
@@ -719,8 +753,34 @@
 - (void) drawSeparatorItemWithFrame:(NSRect)cellFrame
                             inView:(NSView *)controlView
 {
-  // FIXME: This only has sense in MacOS or Windows interface styles.
-  // Maybe somebody wants to support this (Lazaro).
+  NSInterfaceStyle style = NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil);
+  if (style == NSMacintoshInterfaceStyle
+      || style == NSWindows95InterfaceStyle)
+    {
+      NSBezierPath *path = [NSBezierPath bezierPath];
+      NSPoint start = NSMakePoint(3, cellFrame.size.height/2 + 
+				  cellFrame.origin.y);
+      NSPoint end   = NSMakePoint(cellFrame.size.width - 3, 
+				  cellFrame.size.height/2 + 
+				  cellFrame.origin.y);
+
+      [[NSColor blackColor] set];
+
+      [path moveToPoint: start];
+      [path lineToPoint: end];
+
+      [path stroke];
+      /*
+      NSRect lineFrame = NSMakeRect(cellFrame.origin.x,
+				    cellFrame.origin.y 
+				    + cellFrame.size.height/2,
+				    cellFrame.size.width,
+				    1);
+      NSBox *line = [[NSBox alloc] initWithFrame: lineFrame];
+      [controlView addSubview:line];
+      RELEASE(line);
+      */
+    }
 }
 
 - (void) drawStateImageWithFrame: (NSRect)cellFrame
@@ -898,6 +958,8 @@
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
   self = [super initWithCoder: aDecoder];
+  if (nil == self)
+    return nil;
 
   if ([aDecoder allowsKeyedCoding])
     {
