@@ -68,7 +68,7 @@ typedef struct _GSCellRect {
 #ifdef GSIArray
 #undef GSIArray
 #endif
-#import <GNUstepBase/GSIArray.h>
+#include <GNUstepBase/GSIArray.h>
 
 static NSMapTable *viewInfo = 0;
 
@@ -188,8 +188,8 @@ static NSMapTable *viewInfo = 0;
 
 - (id) initAsTearOff
 {
-  [self initWithFrame: NSZeroRect];
-  if (!self)
+  self = [self initWithFrame: NSZeroRect];
+  if (nil == self)
     return nil;
         
   if (_attachedMenu)
@@ -306,6 +306,7 @@ static NSMapTable *viewInfo = 0;
 
       scRect.size.height = [NSMenuView menuBarHeight];
       [self setFrameSize: scRect.size];
+      [self setNeedsSizing: YES];
     }
   else if (flag == NO && _horizontal == YES)
     {
@@ -317,6 +318,7 @@ static NSMapTable *viewInfo = 0;
           NSZoneFree(NSDefaultMallocZone(), a);
           NSMapRemove(viewInfo, self);
         }
+      [self setNeedsSizing: YES];
     }
 
   _horizontal = flag;
@@ -405,7 +407,10 @@ static NSMapTable *viewInfo = 0;
 
 - (NSMenuItemCell*) menuItemCellForItemAtIndex: (int)index
 {
-  return [_itemCells objectAtIndex: index];
+  if (index < [_itemCells count])
+    return [_itemCells objectAtIndex: index];
+  else
+    return nil;
 }
 
 - (NSMenuView*) attachedMenuView
@@ -583,13 +588,19 @@ static NSMapTable *viewInfo = 0;
 {
   BOOL needTitleView;
   BOOL rootIsAppMenu;
+  NSInterfaceStyle style;
 
   NSDebugLLog (@"NSMenu", @"update called on menu view");
 
   /*
    * Ensure that a title view exists only if needed.
    */
-  if (_attachedMenu == nil)
+  style = NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil);
+  if (style == NSWindows95InterfaceStyle || style == NSMacintoshInterfaceStyle)
+    {
+      needTitleView = NO;
+    }
+  else if (_attachedMenu == nil)
     {
       needTitleView = NO;
     }
@@ -649,6 +660,9 @@ static NSMapTable *viewInfo = 0;
 
 - (void) sizeToFit
 {
+  BOOL isPullDown =
+    [_attachedMenu _ownedByPopUp] && [[_attachedMenu _owningPopUp] pullsDown];
+
   if (_horizontal == YES)
     {
       unsigned i;
@@ -665,7 +679,17 @@ static NSMapTable *viewInfo = 0;
 */
       _cellSize.height = [NSMenuView menuBarHeight];
 
-      for (i = 0; i < howMany; i++)
+      if (howMany && isPullDown)
+        {
+          GSCellRect elem;
+          elem.rect = NSMakeRect (currentX,
+                                  0,
+                                  (2 * _horizontalEdgePad),
+                                  _cellSize.height);
+          GSIArrayAddItem(cellRects, (GSIArrayItem)elem);
+          currentX += 2 * _horizontalEdgePad;
+        }
+      for (i = isPullDown ? 1 : 0; i < howMany; i++)
         {
           GSCellRect elem;
           NSMenuItemCell *aCell = [self menuItemCellForItemAtIndex: i];
@@ -722,8 +746,8 @@ static NSMapTable *viewInfo = 0;
         {
           menuBarHeight += _leftBorderOffset;
         }
-      
-      for (i = 0; i < howMany; i++)
+
+      for (i = isPullDown ? 1 : 0; i < howMany; i++)
         {
           float aStateImageWidth;
           float aTitleWidth;
@@ -787,6 +811,8 @@ static NSMapTable *viewInfo = 0;
           if (anImageWidth)
             popupImageWidth = anImageWidth;
         }
+      if (isPullDown && howMany)
+        howMany -= 1;
       
       // Cache the needed widths.
       _stateImageWidth = neededStateImageWidth;
@@ -926,6 +952,13 @@ static NSMapTable *viewInfo = 0;
       [self sizeToFit];
     } 
 
+  // The first item of a pull down menu holds its title and isn't displayed
+  if (index == 0 && [_attachedMenu _ownedByPopUp] &&
+      [[_attachedMenu _owningPopUp] pullsDown])
+    {
+      return NSZeroRect;
+    }
+
   if (_horizontal == YES)
     {
       GSCellRect aRect;
@@ -1013,7 +1046,7 @@ static NSMapTable *viewInfo = 0;
           NSPoint subOrigin = [_window convertBaseToScreen: aRect.origin];
 
           return NSMakePoint (NSMaxX(frame),
-            subOrigin.y - NSHeight(submenuFrame) - 3 +
+            subOrigin.y - NSHeight(submenuFrame) - 2 +
             2*[NSMenuView menuBarHeight]);
         }
       else if ([self _rootIsHorizontal: 0] == YES)
@@ -1038,7 +1071,9 @@ static NSMapTable *viewInfo = 0;
 	[_attachedMenu indexOfItemWithSubmenu: aSubmenu]] toView: nil];
       NSPoint subOrigin = [_window convertBaseToScreen: aRect.origin];
 
-      return NSMakePoint(subOrigin.x, subOrigin.y - NSHeight(submenuFrame));
+      /*We add +1 for don't lose the track when the user move the
+	mouse from the horizontal menu to a submenu.*/
+      return NSMakePoint(subOrigin.x, subOrigin.y - NSHeight(submenuFrame) + 1);
     }
 }
 
@@ -1054,7 +1089,7 @@ static NSMapTable *viewInfo = 0;
 {
   NSRect r;
   NSRect cellFrame;
-  NSRect screenFrame;
+  NSRect popUpFrame;
   int items = [_itemCells count];
   BOOL growHeight = YES;
   BOOL resizeCell = NO;
@@ -1062,6 +1097,8 @@ static NSMapTable *viewInfo = 0;
 
   // Make sure the menu entries are up to date
   [self update];
+  if (_needsSizing)
+    [self sizeToFit];
 
   /* FIXME: Perhaps all of this belongs into NSPopupButtonCell and
      should be used to determine the proper screenRect to pass on into
@@ -1136,83 +1173,148 @@ static NSMapTable *viewInfo = 0;
   if (resizeScreenRect)
     {
       cellFrame = [self convertRect: cellFrame toView: nil];
-      screenRect.size = cellFrame.size;
     }
 
   /*
    * Compute the frame
    */
-  screenFrame = screenRect;
+  popUpFrame.origin = screenRect.origin;
+  popUpFrame.size = cellFrame.size;
   if (items > 0)
     {
       float f;
 
       if (_horizontal == NO)
         {
-          f = screenRect.size.height * (items - 1);
-          screenFrame.size.height += f + _leftBorderOffset;
-          screenFrame.origin.y -= f;
-          screenFrame.size.width += _leftBorderOffset;
-          screenFrame.origin.x -= _leftBorderOffset;
+          f = cellFrame.size.height * (items - 1);
+          popUpFrame.size.height += f + _leftBorderOffset;
+          popUpFrame.origin.y -= f;
+          popUpFrame.size.width += _leftBorderOffset;
+          popUpFrame.origin.x -= _leftBorderOffset;
+
+	  // If the menu is a pull down menu the first item, which would
+	  // appear at the top of the menu, holds the title and is omitted
+	  if ([_attachedMenu _ownedByPopUp])
+	    {
+	      if ([[_attachedMenu _owningPopUp] pullsDown])
+		{
+		  popUpFrame.size.height -= cellFrame.size.height;
+		  popUpFrame.origin.y += cellFrame.size.height;
+		}
+	    }
+
           // Compute position for popups, if needed
           if (selectedItemIndex != -1) 
             {
-              screenFrame.origin.y
-                  += screenRect.size.height * selectedItemIndex;
+              popUpFrame.origin.y
+                  += cellFrame.size.height * selectedItemIndex;
             }
         }
       else
         {
-          f = screenRect.size.width * (items - 1);
-          screenFrame.size.width += f;
+          f = cellFrame.size.width * (items - 1);
+          popUpFrame.size.width += f;
+
+	  // If the menu is a pull down menu the first item holds the
+	  // title and is omitted
+	  if ([_attachedMenu _ownedByPopUp])
+	    {
+	      if ([[_attachedMenu _owningPopUp] pullsDown])
+		{
+		  popUpFrame.size.width -= cellFrame.size.width;
+		}
+	    }
+
           // Compute position for popups, if needed
           if (selectedItemIndex != -1) 
             {
-              screenFrame.origin.x -= screenRect.size.width * selectedItemIndex;
+              popUpFrame.origin.x -= cellFrame.size.width * selectedItemIndex;
             }
         }
     }  
   
   // Update position, if needed, using the preferredEdge
-  if ([_attachedMenu _ownedByPopUp])
+  if (selectedItemIndex == -1)
     {
-      // Per Cocoa documentation, the selected cell should always show up
-      // over the button for popups.   For pull down menus, the preferred 
-      // edge is relevant.   This is also apparent from testing under Cocoa.
-      if ([[_attachedMenu _owningPopUp] pullsDown])
+      NSRect screenFrame;
+
+      if (screen == nil)
+	screen = [NSScreen mainScreen];
+      screenFrame = [screen frame];
+
+      popUpFrame.origin.y -= cellFrame.size.height;
+      if (edge == NSMinYEdge || edge == NSMaxYEdge)
 	{
-	  // Handle the Y edge...
-	  /*
-	  if ([[[_attachedMenu _owningPopUp] controlView] isFlipped])
+	  NSRect minYFrame = popUpFrame;
+	  NSRect maxYFrame = popUpFrame;
+
+	  // show menu above or below the cell depending on the preferred edge
+	  // if the menu would be partially off screen on that edge use the
+	  // opposite edge or at least the one where more space is left
+	  maxYFrame.origin.y +=
+	    maxYFrame.size.height + screenRect.size.height - _leftBorderOffset;
+
+	  if (edge == NSMinYEdge)
 	    {
-	      if (edge == NSMaxYEdge)
+	      if ((NSMinY(minYFrame) < NSMinY(screenFrame))
+		  && ((NSMaxY(maxYFrame) <= NSMaxY(screenFrame))
+		      || (NSMaxY(screenFrame) - NSMaxY(screenRect) >
+			  NSMinY(screenRect) - NSMinY(screenFrame))))
 		{
-		  screenFrame.origin.y += screenRect.size.height;  
+		  edge = NSMaxYEdge;
 		}
 	    }
 	  else
 	    {
-	      if (edge == NSMinYEdge)
+	      if ((NSMaxY(maxYFrame) > NSMaxY(screenFrame))
+		  && ((NSMinY(minYFrame) >= NSMinY(screenFrame))
+		      || (NSMaxY(screenFrame) - NSMaxY(screenRect) <
+			  NSMinY(screenRect) - NSMinY(screenFrame))))
 		{
-		  screenFrame.origin.y += screenRect.size.height;  
+		  edge = NSMinYEdge;
 		}
 	    }
-	  */
+	  popUpFrame = edge == NSMinYEdge ? minYFrame : maxYFrame;
+	}
+      else
+	{
+	  NSRect minXFrame = popUpFrame;
+	  NSRect maxXFrame = popUpFrame;
 
-	  // Handle the X edge...
-	  if (edge == NSMaxXEdge)
+	  minXFrame.origin.y += screenRect.size.height;
+	  minXFrame.origin.x -= minXFrame.size.width;
+
+	  maxXFrame.origin.y += screenRect.size.height;
+	  maxXFrame.origin.x += screenRect.size.width + _leftBorderOffset;
+
+	  // show menu on the opposite edge if it does not fit on screen on
+	  // the preferred edge
+	  if (edge == NSMinXEdge)
 	    {
-	      screenFrame.origin.x += screenRect.size.width;
+	      if ((NSMinX(minXFrame) < NSMinX(screenFrame))
+		  && ((NSMaxX(maxXFrame) <= NSMaxX(screenFrame))
+		      || (NSMaxX(screenFrame) - NSMaxX(screenRect) >
+			  NSMinX(screenRect) - NSMinX(screenFrame))))
+		{
+		  edge = NSMaxXEdge;
+		}
 	    }
-	  else if (edge == NSMinXEdge)
+	  else
 	    {
-	      screenFrame.origin.x -= screenRect.size.width;
+	      if ((NSMaxX(maxXFrame) > NSMaxX(screenFrame))
+		  && ((NSMinX(minXFrame) >= NSMinX(screenFrame))
+		      || (NSMaxX(screenFrame) - NSMaxX(screenRect) <
+			  NSMinX(screenRect) - NSMinX(screenFrame))))
+		{
+		  edge = NSMinXEdge;
+		}
 	    }
+	  popUpFrame = edge == NSMinXEdge ? minXFrame : maxXFrame;
 	}
     }
 
   // Get the frameRect
-  r = [NSWindow frameRectForContentRect: screenFrame
+  r = [NSWindow frameRectForContentRect: popUpFrame
                 styleMask: [_window styleMask]];
   
   // Set the window frame
@@ -1229,27 +1331,10 @@ static NSMapTable *viewInfo = 0;
 
 - (void) drawRect: (NSRect)rect
 {
-  int        i;
-  int        howMany = [_itemCells count];
-
-  [[GSTheme theme] drawBackgroundForMenuView: self
-                                   withFrame: _bounds
-                                   dirtyRect: rect
-                                  horizontal: _horizontal];
- 
-  // Draw the menu cells.
-  for (i = 0; i < howMany; i++)
-    {
-      NSRect aRect;
-      NSMenuItemCell *aCell;
-      
-      aRect = [self rectOfItemAtIndex: i];
-      if (NSIntersectsRect(rect, aRect) == YES)
-        {
-          aCell = [self menuItemCellForItemAtIndex: i];
-          [aCell drawWithFrame: aRect inView: self];
-        }
-    }
+  [[GSTheme theme] drawMenuRect: rect
+		   inView: self
+		   isHorizontal: _horizontal
+		   itemCells: _itemCells];
 }
 
 /*
@@ -1304,22 +1389,34 @@ static NSMapTable *viewInfo = 0;
 - (BOOL) _executeItemAtIndex: (int)indexOfActionToExecute
 	       removeSubmenu: (BOOL)subMenusNeedRemoving
 {
+  NSInterfaceStyle style =
+    NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self);
+  
+  /*If we have menu in window, close the menu after select
+    an option*/
+  if (style == NSWindows95InterfaceStyle)
+    {
+      [[[[NSApp mainWindow] menu] attachedMenu] close];
+    }
+  
   if (indexOfActionToExecute >= 0
       && [_attachedMenu attachedMenu] != nil && [_attachedMenu attachedMenu] ==
       [[_items_link objectAtIndex: indexOfActionToExecute] submenu])
     {
-#if 1
-      if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self)
-	  == NSMacintoshInterfaceStyle)
+      if (style == NSMacintoshInterfaceStyle)
         {
-          /*
-           * FIXME ... always remove submenus in mac mode ... this is not
-           * quite the way the mac behaves, but it's closer than the normal
-           * behavior.
-           */
-          subMenusNeedRemoving = YES;
+	  // On Macintosh, clicking on or releasing the mouse over a
+	  // submenu item always closes the menu (if it is open) and
+	  // ends menu tracking. We do the same here, too.
+          [self detachSubmenu];
+	  return YES;
         }
-#endif
+
+      if (style == NSWindows95InterfaceStyle)
+	{
+	  return YES;
+	}
+
       if (subMenusNeedRemoving)
         {
           [self detachSubmenu];
@@ -1327,27 +1424,11 @@ static NSMapTable *viewInfo = 0;
       // Clicked on a submenu.
       return NO;
     }
-  
-  if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self)
-      == NSMacintoshInterfaceStyle)
-    {
-      NSMenu *tmp = _attachedMenu;
-      
-      do
-        {
-          if ([tmp isEqual: [NSApp mainMenu]] == NO)
-            {
-              [tmp close];
-            }
-          tmp = [tmp supermenu];
-        }
-      while (tmp != nil);
-    }
 
   return YES;
 }
 
-- (BOOL) trackWithEvent: (NSEvent*)event
+- (BOOL) _trackWithEvent: (NSEvent*)event
 {
   unsigned eventMask = NSPeriodicMask;
   NSDate *theDistantFuture = [NSDate distantFuture];
@@ -1389,6 +1470,14 @@ static NSMapTable *viewInfo = 0;
       end = NSLeftMouseUp;
       eventMask |= NSLeftMouseUpMask | NSLeftMouseDraggedMask;
       eventMask |= NSLeftMouseDownMask;
+
+      /*We need know if the user press a modifier key to close the menu
+	when the menu is in a window*/
+      if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self) ==
+	  NSWindows95InterfaceStyle)
+	{
+	  eventMask |= NSFlagsChangedMask;
+	}
     }
   else
     {
@@ -1398,7 +1487,15 @@ static NSMapTable *viewInfo = 0;
       eventMask |= NSLeftMouseDownMask;
     }
 
-  if ([self isHorizontal] == YES)
+  /* We attempt to mimic Mac OS X behavior for pop up menus. If the user
+     presses the mouse button over a pop up button and then drags the mouse
+     over the menu, the menu is closed when the user releases the mouse. On the
+     other hand, when the user clicks on the button and then moves the mouse
+     the menu is closed upon the next mouse click. */
+  if ([self isHorizontal] == YES ||
+      ([[self menu] _ownedByPopUp] == YES &&
+       NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self)
+       == NSMacintoshInterfaceStyle))
     {
       /*
        * Ignore the first mouse up if nothing interesting has happened.
@@ -1407,6 +1504,16 @@ static NSMapTable *viewInfo = 0;
     }
   do
     {
+      /*Close the menu if the user press a modifier key and menu
+	is in a window*/
+      if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self) ==
+	  NSWindows95InterfaceStyle && type == NSFlagsChanged)
+	{
+	  [self setHighlightedItemIndex: -1];
+	  [[[[NSApp mainWindow] menu] attachedMenu] close];
+	  return NO;
+	}
+
       if (type == end)
         {
           shouldFinish = YES;
@@ -1436,14 +1543,15 @@ static NSMapTable *viewInfo = 0;
           if ([_attachedMenu isPartlyOffScreen])
             {
               NSPoint pointerLoc = [_window convertBaseToScreen: location];
+              NSRect screenFrame = [[_window screen] visibleFrame];
               /*
                * The +/-1 in the y - direction is because the flipping
                * between X-coordinates and GNUstep coordinates let the
                * GNUstep screen coordinates start with 1.
                */
               if (pointerLoc.x == 0 || pointerLoc.y == 1
-                || pointerLoc.x == [[_window screen] frame].size.width - 1
-                || pointerLoc.y == [[_window screen] frame].size.height)
+                || pointerLoc.x == screenFrame.size.width - 1
+                || pointerLoc.y == screenFrame.size.height)
                 [_attachedMenu shiftOnScreen];
             }
 
@@ -1483,7 +1591,7 @@ static NSMapTable *viewInfo = 0;
                 = [_window convertBaseToScreen: location];
 
               /*
-               * 3a - Check if moved into one of the ancester menus.
+               * 3a - Check if moved into one of the ancestor menus.
                *      This is tricky, there are a few possibilities:
                *          We are a transient attached menu of a
                *          non-transient menu
@@ -1520,7 +1628,7 @@ static NSMapTable *viewInfo = 0;
                     setHighlightedItemIndex: -1];
                   
                   candidateMenuResult = [[candidateMenu menuRepresentation]
-                           trackWithEvent: original];
+                           _trackWithEvent: original];
                   return candidateMenuResult;
                 }
 
@@ -1534,7 +1642,7 @@ static NSMapTable *viewInfo = 0;
                   BOOL subMenuResult;
 
                   subMenuResult
-                    = [[self attachedMenuView] trackWithEvent: original];
+                    = [[self attachedMenuView] _trackWithEvent: original];
                   if (subMenuResult
                     && wasTransient == [_attachedMenu isTransient])
                     {
@@ -1542,6 +1650,37 @@ static NSMapTable *viewInfo = 0;
                     }
                   return subMenuResult;
                 }
+	      
+	      /*We track the menu correctly when this is located
+		in a window*/
+	      if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self) ==
+		  NSWindows95InterfaceStyle &&
+		  ![[self menu] isTransient] &&
+		  ![[self menu] _ownedByPopUp])
+		{
+                  // If the user moves the mouse into the main window
+                  // horizontal menu, start tracking again.
+                  NSWindow *mainWindow = [NSApp mainWindow];
+                  NSMenuView *mainWindowMenuView = [[mainWindow menu]
+                    menuRepresentation];
+                  NSPoint locationInMainWindow = [mainWindow 
+                    convertScreenToBase: locationInScreenCoordinates];
+		  if ([mainWindowMenuView 
+                        hitTest: locationInMainWindow] != nil)
+		    {
+                      int index = [mainWindowMenuView indexOfItemAtPoint: 
+                        [mainWindowMenuView 
+                          convertPoint: locationInMainWindow
+                              fromView: nil]];
+                      if (index != -1 &&
+                          index != [mainWindowMenuView highlightedItemIndex])
+                        {
+		          [self setHighlightedItemIndex: -1];
+		          return [mainWindowMenuView
+			       _trackWithEvent: original];
+                        }
+		    }
+		}
             }
 
           // 4 - We changed the selected item and should update.
@@ -1598,6 +1737,7 @@ static NSMapTable *viewInfo = 0;
    * 4 - Perform the action if there is one.
    */
 
+  // FIXME
   [NSEvent stopPeriodicEvents];
 
   /*
@@ -1633,6 +1773,9 @@ static NSMapTable *viewInfo = 0;
       return YES;
     }
 
+  // Before executing the action, uncapture the mouse
+  [_window _releaseMouse: self];
+
   if([self _executeItemAtIndex: indexOfActionToExecute
 	   removeSubmenu: subMenusNeedRemoving] == NO)
     {
@@ -1654,6 +1797,23 @@ static NSMapTable *viewInfo = 0;
   return YES;
 }
 
+- (BOOL) trackWithEvent: (NSEvent*)event
+{
+  BOOL result = NO;
+
+  // Capture the mouse so we get clicks outside the menus and
+  // GNUstep windows.
+  [_window _captureMouse: self];
+  NS_DURING
+    result = [self _trackWithEvent: event];
+  NS_HANDLER
+    [_window _releaseMouse: self];
+    [localException raise];
+  NS_ENDHANDLER
+  [_window _releaseMouse: self];
+  return result;
+}
+
 /**
    This method is called when the user clicks on a button in the menu.
          Or, if a right click happens and the app menu is brought up.
