@@ -52,8 +52,8 @@
 #import "AppKit/NSMatrix.h"
 #import "AppKit/NSOpenPanel.h"
 
-#include "GSGuiPrivate.h"
-#include "GNUstepGUI/GSTheme.h"
+#import "GSGuiPrivate.h"
+#import "GNUstepGUI/GSTheme.h"
 
 static NSString	*
 pathToColumn(NSBrowser *browser, int column)
@@ -442,8 +442,19 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 /*
  * Running the NSOpenPanel
  */
-/**<p>Displays the open panel in a modal session, filtering for
-   files that have the specified types </p>
+
+/**<p>Displays the open panel in a modal session, showing the current directory
+   (or last selected), and filtering for files that matches the allowed file 
+   types.</p>
+   <p>See Also: -runModalForDirectory:file:types: and -allowedFileTypes</p>
+*/
+- (int) runModal
+{
+  return [self runModalForTypes: [self allowedFileTypes]];
+}
+
+/**<p>Displays the open panel in a modal session, showing the current directory 
+   (or last selected), and filtering for files that have the specified types.</p>
    <p>See Also: -runModalForDirectory:file:types:</p>
 */
 - (int) runModalForTypes: (NSArray *)fileTypes
@@ -604,7 +615,9 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
-  [super initWithCoder: aDecoder];
+  self = [super initWithCoder: aDecoder];
+  if (nil == self)
+    return nil;
 
   [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_canChooseDirectories];
   [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_canChooseFiles];
