@@ -123,6 +123,7 @@ static NSImage *unexpandable  = nil;
 - (void) _openItem: (id)item;
 - (void) _closeItem: (id)item;
 - (void) _removeChildren: (id)startitem;
+- (void) _noteNumberOfRowsChangedBelowItem: (id)item by: (int)n;
 @end
 
 @interface	NSOutlineView (Private)
@@ -252,27 +253,20 @@ static NSImage *unexpandable  = nil;
           object: self
           userInfo: infoDict];
 
-      // collapse...
-      [self _closeItem: item];
-
-      // Send out the notification to let observers know that this has
-      // occured.
-      [nc postNotificationName: NSOutlineViewItemDidCollapseNotification
-          object: self
-          userInfo: infoDict];
-
-      // recursively find all children and call this method to open them.
+      // recursively find all children and call this method to close them.
+      // Note: The children must be collapsed before their parent item so
+      // that the selected row indexes are properly updated (and in particular
+      // are valid when we post our notifications).
       if (collapseChildren) // collapse all
         {
-          NSMutableArray *allChildren = nil;
-          int numchild = 0;
-          int index = 0;
+          int index, numChildren;
+          NSMutableArray *allChildren;
           id sitem = (item == nil) ? (id)[NSNull null] : (id)item;
 
           allChildren = NSMapGet(_itemDict, sitem);
-          numchild = [allChildren count];
+          numChildren = [allChildren count];
 
-          for (index = 0; index < numchild; index++)
+          for (index = 0; index < numChildren; index++)
             {
               id child = [allChildren objectAtIndex: index];
 
@@ -282,7 +276,16 @@ static NSImage *unexpandable  = nil;
                 }
             }
         }
-      [self noteNumberOfRowsChanged];
+
+      // collapse...
+      [self _closeItem: item];
+
+      // Send out the notification to let observers know that this has
+      // occured.
+      [nc postNotificationName: NSOutlineViewItemDidCollapseNotification
+          object: self
+          userInfo: infoDict];
+
       // Should only mark the rect below the closed item for redraw
       [self setNeedsDisplay: YES];
     }
@@ -342,15 +345,14 @@ static NSImage *unexpandable  = nil;
       // recursively find all children and call this method to open them.
       if (expandChildren) // expand all
         {
-          NSMutableArray *allChildren = nil;
-          int numchild = 0;
-          int index = 0;
+          int index, numChildren;
+          NSMutableArray *allChildren;
           id sitem = (item == nil) ? (id)[NSNull null] : (id)item;
 
           allChildren = NSMapGet(_itemDict, sitem);
-          numchild = [allChildren count];
+          numChildren = [allChildren count];
 
-          for (index = 0; index < numchild; index++)
+          for (index = 0; index < numChildren; index++)
             {
               id child = [allChildren objectAtIndex: index];
 
@@ -361,12 +363,29 @@ static NSImage *unexpandable  = nil;
             }
         }
 
-      [self noteNumberOfRowsChanged];
       // Should only mark the rect below the expanded item for redraw
       [self setNeedsDisplay: YES];
     }
 }
 
+- (NSRect) frameOfOutlineCellAtRow: (NSInteger)row
+{
+  NSRect frameRect;
+
+  if (![self isExpandable: [self itemAtRow: row]])
+    return NSZeroRect;
+
+  frameRect = [self frameOfCellAtColumn: 0
+                                    row: row];
+  
+  if (_indentationMarkerFollowsCell)
+    {
+      frameRect.origin.x += _indentationPerLevel * [self levelForRow: row];
+    }
+
+  return frameRect;
+}
+
 /**
  * Returns whether or not the indentation marker or "knob" is indented
  * along with the content inside the cell.
@@ -380,7 +399,7 @@ static NSImage *unexpandable  = nil;
  * Returns the amount of indentation, in points, for each level
  * of the tree represented by the outline view.
  */
-- (float) indentationPerLevel
+- (CGFloat) indentationPerLevel
 {
   return _indentationPerLevel;
 }
@@ -418,7 +437,7 @@ static NSImage *unexpandable  = nil;
  * Returns the item at a given row. If no item exists for the given row,
  * returns nil.
  */
-- (id) itemAtRow: (int)row
+- (id) itemAtRow: (NSInteger)row
 {
   if ((row >= [_items count]) || (row < 0))
     {
@@ -430,12 +449,12 @@ static NSImage *unexpandable  = nil;
 /**
  * Returns the level for a given item.
  */
-- (int) levelForItem: (id)item
+- (NSInteger) levelForItem: (id)item
 {
   if (item != nil)
     {
       id object = NSMapGet(_levelOfItems, item);
-      return [object intValue];
+      return [object integerValue];
     }
 
   return -1;
@@ -444,7 +463,7 @@ static NSImage *unexpandable  = nil;
 /**
  * Returns the level for the given row.
  */
-- (int) levelForRow: (int)row
+- (NSInteger) levelForRow: (NSInteger)row
 {
   return [self levelForItem: [self itemAtRow: row]];
 }
@@ -457,6 +476,29 @@ static NSImage *unexpandable  = nil;
   return _outlineTableColumn;
 }
 
+/**
+ * Returns the parent of the given item or nil
+ */
+- (id) parentForItem: (id)item
+{
+  NSArray *allKeys = NSAllMapTableKeys(_itemDict);
+  NSEnumerator *en = [allKeys objectEnumerator];
+  NSInteger index;
+  id parent;
+
+  while ((parent = [en nextObject]))
+    {
+      NSMutableArray *childArray = NSMapGet(_itemDict, parent);
+
+      if ((index = [childArray indexOfObject: item]) != NSNotFound)
+        {
+          return (parent == [NSNull null]) ? (id)nil : (id)parent;
+        }
+    }
+
+  return nil;
+}
+
 /**
  * Causes an item to be reloaded.  This is the equivalent of calling
  * [NSOutlineView-reloadItem:reloadChildren:] with reloadChildren set to NO.
@@ -473,7 +515,7 @@ static NSImage *unexpandable  = nil;
  */
 - (void) reloadItem: (id)item reloadChildren: (BOOL)reloadChildren
 {
-  int index;
+  NSInteger index;
   id parent;
   BOOL expanded;
   id dsobj = nil;
@@ -490,7 +532,7 @@ static NSImage *unexpandable  = nil;
 
       if ((index = [childArray indexOfObject: object]) != NSNotFound)
         {
-          parent = (parent == [NSNull null]) ? (id)nil :  (id)parent;
+          parent = (parent == [NSNull null]) ? (id)nil : (id)parent;
           dsobj = [_dataSource outlineView: self
                                child: index
                                ofItem: parent];
@@ -514,7 +556,6 @@ static NSImage *unexpandable  = nil;
       if (expanded)
         {
           [self _openItem: dsobj];
-          [self noteNumberOfRowsChanged];
         }
     }
   [self setNeedsDisplay: YES];
@@ -524,9 +565,9 @@ static NSImage *unexpandable  = nil;
  * Returns the corresponding row in the outline view for the given item.
  * Returns -1 if item is nil or not found.
  */
-- (int) rowForItem: (id)item
+- (NSInteger) rowForItem: (id)item
 {
-  int row;
+  NSInteger row;
   if (item == nil)
     return -1;
 
@@ -599,7 +640,7 @@ static NSImage *unexpandable  = nil;
 /**
  * Sets the amount, in points, that each level is to be indented by.
  */
-- (void)setIndentationPerLevel: (float)newIndentLevel
+- (void)setIndentationPerLevel: (CGFloat)newIndentLevel
 {
   _indentationPerLevel = newIndentLevel;
 }
@@ -677,8 +718,6 @@ static NSImage *unexpandable  = nil;
                                    64);
 
   // reload all the open items...
-  [self _loadDictionaryStartingWith: nil
-        atLevel: -1];
   [self _openItem: nil];
   [super reloadData];
 }
@@ -717,6 +756,7 @@ static NSImage *unexpandable  = nil;
   [super encodeWithCoder: aCoder];
   if ([aCoder allowsKeyedCoding] == NO)
     {
+      float indentation = _indentationPerLevel;
       [aCoder encodeValueOfObjCType: @encode(BOOL)
                                  at: &_autoResizesOutlineColumn];
       [aCoder encodeValueOfObjCType: @encode(BOOL)
@@ -724,7 +764,7 @@ static NSImage *unexpandable  = nil;
       [aCoder encodeValueOfObjCType: @encode(BOOL)
                                  at: &_autosaveExpandedItems];
       [aCoder encodeValueOfObjCType: @encode(float)
-                                 at: &_indentationPerLevel];
+                                 at: &indentation];
       [aCoder encodeConditionalObject: _outlineTableColumn];
     }
 }
@@ -748,6 +788,7 @@ static NSImage *unexpandable  = nil;
     }
   else
     {
+      float indentation;
       // overrides outline defaults with archived values
       [aDecoder decodeValueOfObjCType: @encode(BOOL)
                                    at: &_autoResizesOutlineColumn];
@@ -756,7 +797,8 @@ static NSImage *unexpandable  = nil;
       [aDecoder decodeValueOfObjCType: @encode(BOOL)
                                    at: &_autosaveExpandedItems];
       [aDecoder decodeValueOfObjCType: @encode(float)
-                                   at: &_indentationPerLevel];
+                                   at: &indentation];
+      _indentationPerLevel = indentation;
       _outlineTableColumn = [aDecoder decodeObject];
     }
   return self;
@@ -799,13 +841,15 @@ static NSImage *unexpandable  = nil;
           && location.x >= position
                 && location.x <= position + [image size].width)
         {
+          BOOL withChildren =
+	    ([theEvent modifierFlags] & NSAlternateKeyMask) ? YES : NO;
           if (![self isItemExpanded: item])
             {
-              [self expandItem: item];
+              [self expandItem: item expandChildren: withChildren];
             }
           else
             {
-              [self collapseItem: item];
+              [self collapseItem: item collapseChildren: withChildren];
             }
           return;
         }
@@ -887,8 +931,8 @@ static NSImage *unexpandable  = nil;
       if (tb == _outlineTableColumn)
         {
           NSImage *image = nil;
-          int level = 0;
-          float indentationFactor = 0.0;
+          NSInteger level = 0;
+          CGFloat indentationFactor = 0.0;
           // float originalWidth = drawingRect.size.width;
 
           // display the correct arrow...
@@ -909,17 +953,7 @@ static NSImage *unexpandable  = nil;
           level = [self levelForItem: item];
           indentationFactor = _indentationPerLevel * level;
           imageCell = [[NSCell alloc] initImageCell: image];
-
-          if (_indentationMarkerFollowsCell)
-            {
-              imageRect.origin.x = drawingRect.origin.x + indentationFactor;
-              imageRect.origin.y = drawingRect.origin.y;
-            }
-          else
-            {
-              imageRect.origin.x = drawingRect.origin.x;
-              imageRect.origin.y = drawingRect.origin.y;
-            }
+          imageRect = [self frameOfOutlineCellAtRow: rowIndex];
 
           if ([_delegate respondsToSelector: @selector(outlineView:willDisplayOutlineCell:forTableColumn:item:)])
             {
@@ -936,9 +970,9 @@ static NSImage *unexpandable  = nil;
               imageRect.size.height = [image size].height;
               [imageCell drawWithFrame: imageRect inView: self];
               drawingRect.origin.x
-                += indentationFactor + [image size].width + 5;
+                += indentationFactor + imageRect.size.width + 5;
               drawingRect.size.width
-                -= indentationFactor + [image size].width + 5;
+                -= indentationFactor + imageRect.size.width + 5;
             }
           else
             {
@@ -978,7 +1012,7 @@ static NSImage *unexpandable  = nil;
 }
 
 - (void) setDropItem: (id)item
-      dropChildIndex: (int)childIndex
+      dropChildIndex: (NSInteger)childIndex
 {
 
   if (item != nil && [_items indexOfObject: item] == NSNotFound)
@@ -1018,7 +1052,9 @@ static NSImage *unexpandable  = nil;
 }
 
 // TODO: Move the part that starts at 'Compute the indicator rect area' to GSTheme
-- (void) drawDropAboveIndicatorWithDropItem: (id)currentDropItem atRow: (int)row childDropIndex: (int)currentDropIndex
+- (void) drawDropAboveIndicatorWithDropItem: (id)currentDropItem 
+                                      atRow: (int)row 
+                             childDropIndex: (int)currentDropIndex
 {
   int level = 0;
   NSBezierPath *path = nil;
@@ -1136,10 +1172,12 @@ static NSImage *unexpandable  = nil;
 
 /* Returns the row whose item is the parent that owns the child at the given row. 
 Also returns the child index relative to this parent. */
-- (int) parentRowForRow: (int)row atLevel: (int)level andReturnChildIndex: (int *)childIndex
+- (NSInteger) _parentRowForRow: (NSInteger)row 
+                       atLevel: (NSInteger)level 
+           andReturnChildIndex: (NSInteger *)childIndex
 {
-  int i;
-  int lvl;
+  NSInteger i;
+  NSInteger lvl;
 
   *childIndex = 0;
 
@@ -1173,29 +1211,29 @@ Also returns the child index relative to this parent. */
    * The insertion row is identical to the hovered row, except when p is in 
    * the hovered row bottom part (the last quarter).
    */
-  int row;
+  NSInteger row;
   /* A row can be divided into 4 vertically stacked portions.
    * We call each portion a quarter. 
    * verticalQuarterPosition is the number of quarters that exists between the 
    * top left origin (NSOutlineView is flipped) and the hovered row (precisely 
    * up to the quarter occupied by the pointer in this row). 
    */
-  int verticalQuarterPosition;
+  NSInteger verticalQuarterPosition;
   /* An indentation unit can be divided into 2 portions (left and right).
    * We call each portion a half.
    * We use it to compute the insertion level. */
-  int horizontalHalfPosition;
+  NSInteger horizontalHalfPosition;
   /* The quarter (0, 1, 2 or 3) occupied by the pointer within the hovered row
    * (not in the insertion row). */
-  int positionInRow;
+  NSInteger positionInRow;
   /* The previous row level (the row before the insertion row) */
-  int levelBefore;
+  NSInteger levelBefore;
   /* The next row level (the row after the insertion row) */
-  int levelAfter;
+  NSInteger levelAfter;
   /* The insertion level that may vary with the horizontal pointer position, 
    * when the pointer is between two rows and the bottom row is a parent.
    */
-  int level;
+  NSInteger level;
   NSDragOperation dragOperation = [sender draggingSourceOperationMask];
 
   ASSIGN(lastDragUpdate, [NSDate date]);
@@ -1243,9 +1281,9 @@ Also returns the child index relative to this parent. */
   if ((lastVerticalQuarterPosition != verticalQuarterPosition)
     || (lastHorizontalHalfPosition != horizontalHalfPosition))
     {
-      int minInsertionLevel = levelAfter;
-      int maxInsertionLevel = levelBefore;
-      int pointerInsertionLevel = rint((float)horizontalHalfPosition / 2.);
+      NSInteger minInsertionLevel = levelAfter;
+      NSInteger maxInsertionLevel = levelBefore;
+      NSInteger pointerInsertionLevel = rint((float)horizontalHalfPosition / 2.);
 
       /* Save positions to avoid executing this code when the general
        * position of the mouse is unchanged.
@@ -1302,10 +1340,10 @@ Also returns the child index relative to this parent. */
 	}
       else /* Drop above */
 	{
-          int childIndex = 0;
-          int parentRow = [self parentRowForRow: row 
-                                        atLevel: level 
-                            andReturnChildIndex: &childIndex];
+          NSInteger childIndex = 0;
+          NSInteger parentRow = [self _parentRowForRow: row 
+                                               atLevel: level 
+                                   andReturnChildIndex: &childIndex];
 
 	  //NSLog(@"found %d (proposed childIndex = %d)", parentRow, childIndex);
 
@@ -1407,6 +1445,31 @@ Also returns the child index relative to this parent. */
   return YES;
 }
 
+- (NSArray*) namesOfPromisedFilesDroppedAtDestination: (NSURL *)dropDestination
+{
+  if ([_dataSource respondsToSelector:
+                    @selector(outlineView:namesOfPromisedFilesDroppedAtDestination:forDraggedItems:)])
+    {
+      int count = [_selectedRows count];
+      NSMutableArray *itemArray = [NSMutableArray arrayWithCapacity: count];
+      unsigned int index = [_selectedRows firstIndex];
+      
+      while (index != NSNotFound)
+        {
+          [itemArray addObject: [self itemAtRow: index]];
+          index = [_selectedRows indexGreaterThanIndex: index];
+        }
+
+      return [_dataSource outlineView: self
+                          namesOfPromisedFilesDroppedAtDestination: dropDestination
+                          forDraggedItems: itemArray];
+    }
+  else
+    {
+      return nil;
+    }
+}
+
 // Autosave methods...
 - (void) setAutosaveName: (NSString *)name
 {
@@ -1421,7 +1484,7 @@ Also returns the child index relative to this parent. */
 {
   NSText *t;
   NSTableColumn *tb;
-  NSRect drawingRect, imageRect;
+  NSRect drawingRect;
   unsigned length = 0;
   int level = 0;
   float indentationFactor = 0.0;
@@ -1505,8 +1568,9 @@ Also returns the child index relative to this parent. */
       id item = nil;
       NSImage *image = nil;
       NSCell *imageCell = nil;
+      NSRect imageRect;
 
-      item = [self itemAtRow: _editedRow];
+      item = [self itemAtRow: rowIndex];
       // determine which image to use...
       if ([self isItemExpanded: item])
         {
@@ -1519,38 +1583,46 @@ Also returns the child index relative to this parent. */
 
       if (![self isExpandable: item])
         {
-//          image = unexpandable;
-	  image = nil;
+          image = unexpandable;
         }
 
       level = [self levelForItem: item];
       indentationFactor = _indentationPerLevel * level;
-
       // create the image cell..
       imageCell = [[NSCell alloc] initImageCell: image];
-      if (_indentationMarkerFollowsCell)
+      imageRect = [self frameOfOutlineCellAtRow: rowIndex];
+
+      if ([_delegate respondsToSelector: @selector(outlineView:willDisplayOutlineCell:forTableColumn:item:)])
         {
-          imageRect.origin.x = drawingRect.origin.x + indentationFactor;
-          imageRect.origin.y = drawingRect.origin.y;
+          [_delegate outlineView: self
+                     willDisplayOutlineCell: imageCell
+                     forTableColumn: tb
+                     item: item];
+        }
+
+
+      if ([imageCell image])
+        {
+
+          imageRect.size.width = [image size].width;
+          imageRect.size.height = [image size].height;
+          
+          // draw...
+          [self lockFocus];
+          [imageCell drawWithFrame: imageRect inView: self];
+          [self unlockFocus];
+          
+          // move the drawing rect over like in the drawRow routine...
+          drawingRect.origin.x += indentationFactor + 5 + imageRect.size.width;
+          drawingRect.size.width -= indentationFactor + 5 + imageRect.size.width;
         }
       else
         {
-          imageRect.origin.x = drawingRect.origin.x;
-          imageRect.origin.y = drawingRect.origin.y;
+          // move the drawing rect over like in the drawRow routine...
+          drawingRect.origin.x += indentationFactor;
+          drawingRect.size.width -= indentationFactor;
         }
 
-      imageRect.size.width = [image size].width;
-      imageRect.size.height = [image size].height;
-
-      // draw...
-      [self lockFocus];
-      [imageCell drawWithFrame: imageRect inView: self];
-      [self unlockFocus];
-
-      // move the drawing rect over like in the drawRow routine...
-      drawingRect.origin.x += indentationFactor + 5 + [image size].width;
-      drawingRect.size.width -= indentationFactor + 5 + [image size].width;
-
       RELEASE(imageCell);
     }
 
@@ -1667,10 +1739,10 @@ Also returns the child index relative to this parent. */
 
 - (void) _didChangeSortDescriptors: (NSArray *)oldSortDescriptors
 {
-  if ([_delegate 
+  if ([_dataSource 
 	respondsToSelector: @selector(outlineView:sortDescriptorsDidChange:)])
     {
-      [_delegate outlineView: self sortDescriptorsDidChange: oldSortDescriptors];
+      [_dataSource outlineView: self sortDescriptorsDidChange: oldSortDescriptors];
     }
 }
 
@@ -1871,6 +1943,20 @@ Also returns the child index relative to this parent. */
     }
 }
 
+- (BOOL) _isItemLoaded: (id)item
+{
+  id sitem = (item == nil) ? (id)[NSNull null] : (id)item;
+  id object = NSMapGet(_itemDict, sitem);
+
+  // NOTE: We could store the loaded items in a map to ensure we only load 
+  // the children of item when it gets expanded for the first time. This would
+  // allow to write: return (NSMapGet(_loadedItemDict, sitem) != nil);
+  // The last line isn't truly correct because it implies an item without 
+  // children will get incorrectly reloaded automatically on each 
+  // expand/collapse.
+  return ([object count] != 0);
+}
+
 - (void) _loadDictionaryStartingWith: (id) startitem
                              atLevel: (int) level
 {
@@ -1879,12 +1965,16 @@ Also returns the child index relative to this parent. */
   id sitem = (startitem == nil) ? (id)[NSNull null] : (id)startitem;
   NSMutableArray *anarray = nil;
 
-  /* Check to see if item is expandable before getting the number of
-   * items. For macos compatibility the topmost item (startitem==nil)
+  /* Check to see if item is expandable and expanded before getting the number 
+   * of items. For macos compatibility the topmost item (startitem==nil)
    * is always considered expandable and must not be checked.
+   * We must load the item only if expanded, otherwise an outline view is not 
+   * usable with a big tree structure. For example, an outline view to browse 
+   * file system would try to traverse every file/directory on -reloadData.
    */
-  if (startitem == nil
+  if ((startitem == nil
     || [_dataSource outlineView: self isItemExpandable: startitem])
+    && [self isItemExpanded: startitem])
     {
       num = [_dataSource outlineView: self
 			 numberOfChildrenOfItem: startitem];
@@ -1912,12 +2002,11 @@ Also returns the child index relative to this parent. */
 
 - (void)_closeItem: (id)item
 {
-  int numchildren = 0;
-  int i = 0;
+  int i, numChildren;
   NSMutableArray *removeAll = [NSMutableArray array];
 
   [self _collectItemsStartingWith: item into: removeAll];
-  numchildren = [removeAll count];
+  numChildren = [removeAll count];
 
   // close the item...
   if (item != nil)
@@ -1927,30 +2016,35 @@ Also returns the child index relative to this parent. */
 
   // For the close method it doesn't matter what order they are
   // removed in.
-  for (i = 0; i < numchildren; i++)
+  for (i = 0; i < numChildren; i++)
     {
       id child = [removeAll objectAtIndex: i];
       [_items removeObject: child];
     }
+  [self _noteNumberOfRowsChangedBelowItem: item by: -numChildren];
 }
 
 - (void)_openItem: (id)item
 {
-  int numchildren = 0;
-  int i = 0;
-  int insertionPoint = 0;
-  id object = nil;
+  int i, insertionPoint, numChildren, numDescendants;
+  id object;
   id sitem = (item == nil) ? (id)[NSNull null] : (id)item;
 
-  object = NSMapGet(_itemDict, sitem);
-  numchildren = [object count];
-
   // open the item...
   if (item != nil)
     {
       [_expandedItems addObject: item];
     }
 
+  // Load the children of the item if needed
+  if ([self _isItemLoaded: item] == NO)
+    {
+      [self _loadDictionaryStartingWith: item atLevel: [self levelForItem: item]];
+    }
+
+  object = NSMapGet(_itemDict, sitem);
+  numChildren = numDescendants = [object count];
+
   insertionPoint = [_items indexOfObject: item];
   if (insertionPoint == NSNotFound)
     {
@@ -1961,7 +2055,7 @@ Also returns the child index relative to this parent. */
       insertionPoint++;
     }
 
-  for (i = numchildren-1; i >= 0; i--)
+  for (i = numChildren-1; i >= 0; i--)
     {
       id obj = NSMapGet(_itemDict, sitem);
       id child = [obj objectAtIndex: i];
@@ -1969,12 +2063,13 @@ Also returns the child index relative to this parent. */
       // Add all of the children...
       if ([self isItemExpanded: child])
         {
+          int i, numItems;
           NSMutableArray *insertAll = [NSMutableArray array];
-          int i = 0, numitems = 0;
 
           [self _collectItemsStartingWith: child into: insertAll];
-          numitems = [insertAll count];
-           for (i = numitems-1; i >= 0; i--)
+          numItems = [insertAll count];
+          numDescendants += numItems;
+          for (i = numItems-1; i >= 0; i--)
             {
               [_items insertObject: [insertAll objectAtIndex: i]
                       atIndex: insertionPoint];
@@ -1984,18 +2079,19 @@ Also returns the child index relative to this parent. */
       // Add the parent
       [_items insertObject: child atIndex: insertionPoint];
     }
+
+  [self _noteNumberOfRowsChangedBelowItem: item by: numDescendants];
 }
 
 - (void) _removeChildren: (id)startitem
 {
-  int numchildren = 0;
-  int i = 0;
+  int i, numChildren;
   id sitem = (startitem == nil) ? (id)[NSNull null] : (id)startitem;
   NSMutableArray *anarray;
 
   anarray = NSMapGet(_itemDict, sitem);
-  numchildren = [anarray count];
-  for (i = 0; i < numchildren; i++)
+  numChildren = [anarray count];
+  for (i = 0; i < numChildren; i++)
     {
       id child = [anarray objectAtIndex: i];
 
@@ -2005,6 +2101,91 @@ Also returns the child index relative to this parent. */
       [_expandedItems removeObject: child];
     }
   [anarray removeAllObjects];
+  [self _noteNumberOfRowsChangedBelowItem: startitem by: -numChildren];
+}
+
+- (void) _noteNumberOfRowsChangedBelowItem: (id)item by: (int)numItems
+{
+  BOOL selectionDidChange = NO;
+  int rowIndex, nextIndex;
+
+  // check for trivial case
+  if (numItems == 0)
+    return;
+
+  // if a row below item is selected, update the selected row indexes
+  /* Note: We update the selected row indexes directly instead of calling
+   * -selectRowIndexes:extendingSelection: to avoid posting bogus selection
+   * did change notifications. */
+  rowIndex = [_items indexOfObject: item];
+  rowIndex = (rowIndex == NSNotFound) ? 0 : rowIndex + 1;
+  nextIndex = [_selectedRows indexGreaterThanOrEqualToIndex: rowIndex];
+  if (nextIndex != NSNotFound)
+    {
+      if (numItems > 0)
+	{
+	  [_selectedRows shiftIndexesStartingAtIndex: rowIndex by: numItems];
+	  if (_selectedRow >= rowIndex)
+	    {
+	      _selectedRow += numItems;
+	    }
+	}
+      else
+        {
+	  numItems = -numItems;
+	  [_selectedRows shiftIndexesStartingAtIndex: rowIndex + numItems
+						  by: -numItems];
+	  if (nextIndex < rowIndex + numItems)
+	    {
+	      /* Don't post the notification here, as the table view is in
+	       * an inconsistent state. */
+	      selectionDidChange = YES;
+	    }
+
+	  /* If the selection becomes empty after removing items and the
+	   * receiver does not allow empty selections, select the root item. */
+          if ([_selectedRows firstIndex] == NSNotFound &&
+              [self allowsEmptySelection] == NO)
+            {
+              [_selectedRows addIndex: 0];
+            }
+
+	  if (_selectedRow >= rowIndex + numItems)
+	    {
+	      _selectedRow -= numItems;
+	    }
+	  else if (_selectedRow >= rowIndex)
+	    {
+	      /* If the item at _selectedRow was removed, we arbitrarily choose
+	       * another selected item (if there is still any). The policy
+	       * implemented below chooses the index most close to item. */
+	      int r1 = [_selectedRows indexLessThanIndex: rowIndex];
+	      int r2 = [_selectedRows indexGreaterThanOrEqualToIndex: rowIndex];
+	      if (r1 != NSNotFound && r2 != NSNotFound)
+		{
+		  _selectedRow = (rowIndex - r1) <= (r2 - rowIndex) ? r1 : r2;
+		}
+	      else if (r1 != NSNotFound)
+		{
+		  _selectedRow = r1;
+		}
+	      else if (r2 != NSNotFound)
+		{
+		  _selectedRow = r2;
+		}
+	      else
+		{
+		  _selectedRow = -1;
+		}
+	    }
+        }
+    }
+
+  [self noteNumberOfRowsChanged];
+  if (selectionDidChange)
+    {
+      [self _postSelectionDidChangeNotification];
+    }
 }
 
 @end
