@@ -27,12 +27,9 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-/* To keep the allocation counts valid when swizzling the class in
-[NSMutableParagraphStyle -copyWithZone:]. */
-#include <Foundation/NSDebug.h>
-
-#include <Foundation/NSException.h>
-#include "AppKit/NSParagraphStyle.h"
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSException.h>
+#import "AppKit/NSParagraphStyle.h"
 
 @implementation NSTextTab
 
@@ -139,7 +136,7 @@
     return NSOrderedSame;
 }
 
-- (unsigned) hash
+- (NSUInteger) hash
 {
   unsigned val = (unsigned)_location;
 
@@ -473,9 +470,8 @@ static NSParagraphStyle	*defaultStyle = nil;
     {
       unsigned	count;
       
-      [aCoder decodeValueOfObjCType: @encode(NSTextAlignment) at: &_alignment];
-      [aCoder decodeValueOfObjCType: @encode(NSLineBreakMode) 
-	      at: &_lineBreakMode];
+      [aCoder decodeValueOfObjCType: @encode(int) at: &_alignment];
+      [aCoder decodeValueOfObjCType: @encode(int) at: &_lineBreakMode];
       [aCoder decodeValueOfObjCType: @encode(float) at: &_firstLineHeadIndent];
       [aCoder decodeValueOfObjCType: @encode(float) at: &_headIndent];
       [aCoder decodeValueOfObjCType: @encode(float) at: &_lineSpacing];
@@ -498,7 +494,7 @@ static NSParagraphStyle	*defaultStyle = nil;
           [aCoder decodeArrayOfObjCType: @encode(float)
                   count: count
                   at: locations];
-          [aCoder decodeArrayOfObjCType: @encode(NSTextTabType)
+          [aCoder decodeArrayOfObjCType: @encode(int)
                   count: count
                   at: types];
           for (i = 0; i < count; i++)
@@ -531,9 +527,8 @@ static NSParagraphStyle	*defaultStyle = nil;
     {
       unsigned	count;
       
-      [aCoder encodeValueOfObjCType: @encode(NSTextAlignment) at: &_alignment];
-      [aCoder encodeValueOfObjCType: @encode(NSLineBreakMode) 
-	      at: &_lineBreakMode];
+      [aCoder encodeValueOfObjCType: @encode(int) at: &_alignment];
+      [aCoder encodeValueOfObjCType: @encode(int) at: &_lineBreakMode];
       [aCoder encodeValueOfObjCType: @encode(float) at: &_firstLineHeadIndent];
       [aCoder encodeValueOfObjCType: @encode(float) at: &_headIndent];
       [aCoder encodeValueOfObjCType: @encode(float) at: &_lineSpacing];
@@ -563,7 +558,7 @@ static NSParagraphStyle	*defaultStyle = nil;
           [aCoder encodeArrayOfObjCType: @encode(float)
                   count: count
                   at: locations];
-          [aCoder encodeArrayOfObjCType: @encode(NSTextTabType)
+          [aCoder encodeArrayOfObjCType: @encode(int)
                   count: count
                   at: types];
         }
@@ -601,7 +596,7 @@ static NSParagraphStyle	*defaultStyle = nil;
   return [_tabStops isEqualToArray: other->_tabStops];
 }
 
-- (unsigned int) hash
+- (NSUInteger) hash
 {
   return _alignment + _lineBreakMode;
 }
@@ -806,16 +801,14 @@ static NSParagraphStyle	*defaultStyle = nil;
 
 - (id) copyWithZone: (NSZone*)aZone
 {
-  NSMutableParagraphStyle	*c;
+  NSMutableParagraphStyle *c;
 
   c = (NSMutableParagraphStyle*)NSCopyObject (self, 0, aZone);
-  GSDebugAllocationRemove(c->isa, c);
-  c->isa = [NSParagraphStyle class];
-  GSDebugAllocationAdd(c->isa, c);
+  GSClassSwizzle(c, [NSParagraphStyle class]);
   c->_tabStops = [_tabStops mutableCopyWithZone: aZone];
   c->_textBlocks = [_textBlocks mutableCopyWithZone: aZone];
   c->_textLists = [_textLists mutableCopyWithZone: aZone];
-   return c;
+  return c;
 }
 
 @end
