@@ -65,7 +65,10 @@ Class _nspopupbuttonCellClass = 0;
       [self setCellClass: [NSPopUpButtonCell class]];
 
       [self exposeBinding: NSSelectedIndexBinding];
-    } 
+      [self exposeBinding: NSSelectedTagBinding];
+      [self setKeys: [NSArray arrayWithObject: NSSelectedIndexBinding] 
+            triggerChangeNotificationsForDependentKey: NSSelectedTagBinding];
+   } 
 }
 
 + (Class) cellClass
@@ -257,7 +260,7 @@ this to return nil to indicate that we have no context menu.
 /**<p>Returns the tag of the selected item</p>
    <p>See Also: [NSPopUpButtonCell-indexOfSelectedItem]</p>
 */
-- (int) selectedTag
+- (NSInteger) selectedTag
 {
   return [[_cell selectedItem] tag];
 }
@@ -568,6 +571,10 @@ this to return nil to indicate that we have no context menu.
     {
       [self selectItemAtIndex: [anObject intValue]];
     }
+  else if ([aKey isEqual: NSSelectedTagBinding])
+    {
+      [self selectItemWithTag: [anObject integerValue]];
+    }
   else
     {
       [super setValue: anObject forKey: aKey];
@@ -580,6 +587,10 @@ this to return nil to indicate that we have no context menu.
     {
       return [NSNumber numberWithInt: [self indexOfSelectedItem]];
     }
+  else if ([aKey isEqual: NSSelectedTagBinding])
+    {
+      return [NSNumber numberWithInteger: [self selectedTag]];
+    }
   else
     {
       return [super valueForKey: aKey];
