@@ -117,7 +117,7 @@ static NSImage *_pbc_image[5];
   [self _initMenu];
   [self setPullsDown: flag];
   _pbcFlags.usesItemFromMenu = YES;
-  [self setPreferredEdge: NSMinYEdge];
+  [self setPreferredEdge: NSMaxYEdge];
   [self setArrowPosition: NSPopUpArrowAtCenter];
 
   if ([stringValue length] > 0)
@@ -147,6 +147,11 @@ static NSImage *_pbc_image[5];
    * so the local reference to it is invalid and will result in a 
    * segmentation fault. 
    */
+  if (_menu != nil)
+    {
+      NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
+      [nc removeObserver: self name: nil object: _menu];
+    }
   _selectedItem = nil;
   [super dealloc];
 }
@@ -157,6 +162,8 @@ static NSImage *_pbc_image[5];
  */ 
 - (void) setMenu: (NSMenu *)menu
 {
+  NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
+
   if (_menu == menu)
     {
       return;
@@ -165,6 +172,9 @@ static NSImage *_pbc_image[5];
   if (_menu != nil)
     {
       [_menu _setOwnedByPopUp: nil];
+      [nc removeObserver: self
+                    name: nil
+                  object: _menu];
     }
   ASSIGN(_menu, menu);
   if (_menu != nil)
@@ -174,6 +184,18 @@ static NSImage *_pbc_image[5];
        * popupbutton code in super class NSMenuItemCell
        */
       [self setMenuView: [_menu menuRepresentation]];
+      [nc addObserver: self
+             selector: @selector(synchronizeTitleAndSelectedItem)
+                 name: NSMenuDidAddItemNotification
+               object: _menu];
+      [nc addObserver: self
+             selector: @selector(synchronizeTitleAndSelectedItem)
+                 name: NSMenuDidRemoveItemNotification
+               object: _menu];
+      [nc addObserver: self
+             selector: @selector(synchronizeTitleAndSelectedItem)
+                 name: NSMenuDidChangeItemNotification
+               object: _menu];
     }
   else 
     {
@@ -184,6 +206,15 @@ static NSImage *_pbc_image[5];
   [self synchronizeTitleAndSelectedItem];
 }
 
+- (void)setMenuView: (NSMenuView *)aMenuView
+{
+  // NB Don't call NSMenuItemCell's implementation here because it also
+  //    changes various items of the receiver, which is reasonable for a
+  //    menu item cell that is part of a menu, but not for a pop up button
+  //    cell.
+  _menuView = aMenuView;
+}
+
 /**
  * Return the menu for the popup.
  */
@@ -193,7 +224,7 @@ static NSImage *_pbc_image[5];
 }
 
 /**
- * Returns YES, if this is a pull-down 
+ * Set the pull-down state
  */
 - (void) setPullsDown: (BOOL)flag
 {
@@ -209,6 +240,7 @@ static NSImage *_pbc_image[5];
   _pbcFlags.pullsDown = flag;
 
   [self setMenuItem: item];
+  [self synchronizeTitleAndSelectedItem];
 }
 
 /**
@@ -293,11 +325,11 @@ static NSImage *_pbc_image[5];
 
   if (flag)
     {
-      [selectedItem setState: NSOffState];
+      [selectedItem setState: NSOnState];
     }
   else
     {
-      [selectedItem setState: NSOnState];
+      [selectedItem setState: NSOffState];
     }
 
   _pbcFlags.altersStateOfSelectedItem = flag;
@@ -562,7 +594,7 @@ static NSImage *_pbc_image[5];
           return nil;
         }
 
-      if (_pbcFlags.preferredEdge == NSMinYEdge)
+      if (_pbcFlags.preferredEdge == NSMaxYEdge)
         {
           return _pbc_image[1];
         }
@@ -570,7 +602,7 @@ static NSImage *_pbc_image[5];
         {
           return _pbc_image[2];
         }
-      else if (_pbcFlags.preferredEdge == NSMaxYEdge)
+      else if (_pbcFlags.preferredEdge == NSMinYEdge)
         {
           return _pbc_image[3];
         }
@@ -622,7 +654,13 @@ static NSImage *_pbc_image[5];
   id<NSMenuItem> oldSelectedItem = _selectedItem;
 
   if (_selectedItem == item)
-    return;
+    {
+      // pull-down should set highlighted item even when selection is unchanged
+      if (_pbcFlags.pullsDown)
+        [[_menu menuRepresentation] setHighlightedItemIndex: 
+                   [_menu indexOfItem: _selectedItem]];
+      return;
+    }
 
   if (_selectedItem != nil)
     {
@@ -634,6 +672,13 @@ static NSImage *_pbc_image[5];
 
   _selectedItem = item;
 
+  /* Set the item in the menu */
+  /* Note: Must do this before changing the state of the selected item, since
+   * the change will recursively invoke synchronizeTitleAndSelectedItem, which
+   * otherwise would select the old item again */
+  [[_menu menuRepresentation] setHighlightedItemIndex: 
+                   [_menu indexOfItem: _selectedItem]];
+
   if (_selectedItem != nil)
     {
       if (_pbcFlags.altersStateOfSelectedItem)
@@ -642,12 +687,11 @@ static NSImage *_pbc_image[5];
         }
     }
 
-  /* Set the item in the menu */
-  [[_menu menuRepresentation] setHighlightedItemIndex: 
-                   [_menu indexOfItem: _selectedItem]];
-
-  [[_menu menuRepresentation] setNeedsDisplayForItemAtIndex:
-                   [_menu indexOfItem: oldSelectedItem]];
+  if (oldSelectedItem)
+    {
+      [[_menu menuRepresentation] setNeedsDisplayForItemAtIndex:
+                       [_menu indexOfItem: oldSelectedItem]];
+    }
 }
 
 - (void) selectItemAtIndex: (int)index
@@ -669,6 +713,32 @@ static NSImage *_pbc_image[5];
   [self selectItem: anItem];
 }
 
+- (NSString *)title
+{
+  id <NSMenuItem> selectedItem;
+
+  if (!_pbcFlags.usesItemFromMenu)
+    {
+      selectedItem = _menuItem;
+    }
+  else if (_pbcFlags.pullsDown)
+    {
+      if ([_menu numberOfItems] == 0)
+	{
+	  selectedItem = _menuItem;
+	}
+      else
+	{
+	  selectedItem = [_menu itemAtIndex: 0];
+	}
+    }
+  else
+    {
+      selectedItem = [self selectedItem];
+    }
+  return [selectedItem title]; 
+}
+
 - (void) setTitle: (NSString *)aString
 {
   id <NSMenuItem> anItem;
@@ -688,6 +758,7 @@ static NSImage *_pbc_image[5];
       else
         {
           anItem = [_menu itemAtIndex: 0];
+          [anItem setTitle: aString];
         }
     }
   else
@@ -813,6 +884,7 @@ static NSImage *_pbc_image[5];
 - (void) attachPopUpWithFrame: (NSRect)cellFrame
                        inView: (NSView *)controlView
 {
+  NSRectEdge            preferredEdge = _pbcFlags.preferredEdge;
   NSNotificationCenter  *nc = [NSNotificationCenter defaultCenter];
   NSWindow              *cvWin = [controlView window];
   NSMenuView            *mr = [_menu menuRepresentation];
@@ -830,25 +902,38 @@ static NSImage *_pbc_image[5];
 
   if (_pbcFlags.pullsDown)
     selectedItem = -1;
-  else 
-    selectedItem = [self indexOfSelectedItem];
+  else
+    {
+      selectedItem = [self indexOfSelectedItem];
+      if (selectedItem == -1)
+	{
+	  selectedItem = 0;
+	}
+    }
 
   if (selectedItem > 0)
     {
       [mr setHighlightedItemIndex: selectedItem];
     }
 
-  // Ask the MenuView to attach the menu to this rect
-  [mr setWindowFrameForAttachingToRect: cellFrame
-      onScreen: [cvWin screen]
-      preferredEdge: _pbcFlags.preferredEdge
-      popUpSelectedItem: selectedItem];
+  if ([controlView isFlipped])
+    {
+      if (preferredEdge == NSMinYEdge)
+	{
+	  preferredEdge = NSMaxYEdge;
+	}
+      else if (preferredEdge == NSMaxYEdge)
+	{
+	  preferredEdge = NSMinYEdge;
+	}
+    }
 
-  // Set to be above the main window
-  [cvWin addChildWindow: [mr window] ordered: NSWindowAbove];
-
-  // Last, display the window
-  [[mr window] orderFrontRegardless];
+  // display the menu item...
+  [[GSTheme theme] displayPopUpMenu: mr
+		      withCellFrame: cellFrame
+		  controlViewWindow: cvWin
+		      preferredEdge: preferredEdge
+		       selectedItem: selectedItem];
 
   [nc addObserver: self
       selector: @selector(_handleNotification:)
@@ -913,6 +998,8 @@ static NSImage *_pbc_image[5];
   // Attach the popUp
   [self attachPopUpWithFrame: cellFrame
                       inView: controlView];
+  if ([[GSTheme theme] doesProcessEventsForPopUpMenu])
+    return YES; // the theme handles the events, so we're done
   
   p = [[controlView window] convertBaseToScreen: [theEvent locationInWindow]];
   p = [menuWindow convertScreenToBase: p];
@@ -981,6 +1068,9 @@ static NSImage *_pbc_image[5];
 
   /* We need to calc our size to get images placed correctly */
   [self calcSize];
+  [[GSTheme theme] drawPopUpButtonCellInteriorWithFrame: cellFrame
+		   withCell: self
+		   inView: controlView];
   [super drawInteriorWithFrame: cellFrame inView: controlView];
 
   /* Unset the item to restore balance if a new was created */
@@ -1120,6 +1210,11 @@ static NSImage *_pbc_image[5];
 
   if ([aDecoder allowsKeyedCoding])
     {
+      /* First decode menu, menu items must be available to set the selection */
+      menu = [aDecoder decodeObjectForKey: @"NSMenu"];
+      [self setMenu: nil];
+      [self setMenu: menu];
+
       if ([aDecoder containsValueForKey: @"NSAltersState"])
         {
           BOOL alters = [aDecoder decodeBoolForKey: @"NSAltersState"];
@@ -1156,12 +1251,6 @@ static NSImage *_pbc_image[5];
 	  BOOL pullDown = [aDecoder decodeBoolForKey: @"NSPullDown"];
 	  [self setPullsDown: pullDown];
 	}
-
-      menu = [aDecoder decodeObjectForKey: @"NSMenu"];
-      [self setMenu: nil];
-      [self setMenu: menu];
-      [self setAutoenablesItems: NO];
-      [self setEnabled: YES];
     }
   else
     {
@@ -1212,7 +1301,7 @@ static NSImage *_pbc_image[5];
         }
       if (version < 3)
         {
-          [self setPreferredEdge: NSMinYEdge];
+          [self setPreferredEdge: NSMaxYEdge];
           [self setArrowPosition: NSPopUpArrowAtCenter];
         }
       [self selectItem: selectedItem];
@@ -1230,6 +1319,10 @@ static NSImage *_pbc_image[5];
  */
 - (void) _popUpItemAction: (id)sender
 {
+  // first, if sender is one of our items, set it as our selected item
+  int index = [_menu indexOfItem:sender];
+  if (index != NSNotFound)
+    [self selectItemAtIndex:index];
   [NSApp sendAction: [self action] to: [self target] from: self];
 }
 
