@@ -36,31 +36,31 @@
 #include <limits.h>
 #include <math.h>
 #include "config.h"
-#include <Foundation/NSString.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSData.h>
-#include <Foundation/NSFileManager.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSPathUtilities.h>
-#include <Foundation/NSTask.h>
-#include <Foundation/NSThread.h>
-#include <Foundation/NSUserDefaults.h>
-#include <Foundation/NSValue.h>
-#include "AppKit/AppKitExceptions.h"
-#include "AppKit/NSAffineTransform.h"
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSGraphicsContext.h"
-#include "AppKit/NSView.h"
-#include "AppKit/NSPrinter.h"
-#include "AppKit/NSPrintPanel.h"
-#include "AppKit/NSPrintInfo.h"
-#include "AppKit/NSPrintOperation.h"
-#include "AppKit/NSWorkspace.h"
-#include "AppKit/PSOperators.h"
-#include "GNUstepGUI/GSEPSPrintOperation.h"
-#include "GNUstepGUI/GSPDFPrintOperation.h"
-#include "GNUstepGUI/GSPrintOperation.h"
-#include "GSGuiPrivate.h"
+#import <Foundation/NSString.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSData.h>
+#import <Foundation/NSFileManager.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSPathUtilities.h>
+#import <Foundation/NSTask.h>
+#import <Foundation/NSThread.h>
+#import <Foundation/NSUserDefaults.h>
+#import <Foundation/NSValue.h>
+#import "AppKit/AppKitExceptions.h"
+#import "AppKit/NSAffineTransform.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSGraphicsContext.h"
+#import "AppKit/NSView.h"
+#import "AppKit/NSPrinter.h"
+#import "AppKit/NSPrintPanel.h"
+#import "AppKit/NSPrintInfo.h"
+#import "AppKit/NSPrintOperation.h"
+#import "AppKit/NSWorkspace.h"
+#import "AppKit/PSOperators.h"
+#import "GNUstepGUI/GSEPSPrintOperation.h"
+#import "GNUstepGUI/GSPDFPrintOperation.h"
+#import "GNUstepGUI/GSPrintOperation.h"
+#import "GSGuiPrivate.h"
 
 #define NSNUMBER(a) [NSNumber numberWithInt: (a)]
 #define NSFNUMBER(a) [NSNumber numberWithFloat: (a)]
