@@ -123,7 +123,6 @@ static NSPrintPanel *shared_instance;
 		 styleMask: (unsigned int)aStyle
 		   backing: (NSBackingStoreType)bufferingType
 		     defer: (BOOL)flag
-		    screen: (NSScreen*)aScreen
 {
   unsigned int i;
   id control;
@@ -134,8 +133,7 @@ static NSPrintPanel *shared_instance;
   self = [super initWithContentRect: contentRect
 		 styleMask: aStyle
 		   backing: bufferingType
-		     defer: flag
-		    screen: aScreen];
+		     defer: flag];
   if (self == nil)
     return nil;
 
@@ -256,7 +254,7 @@ static NSPrintPanel *shared_instance;
   NSSavePanel *sp;
 
   sp = [NSSavePanel savePanel];
-  [sp setRequiredFileType: @"ps"];
+  [sp setAllowedFileTypes: [NSArray arrayWithObjects: @"pdf", @"ps", nil]];
   result = [sp runModal];
   if (result == NSOKButton)
     {
@@ -318,6 +316,16 @@ static NSPrintPanel *shared_instance;
   [NSApp stopModalWithCode: (_picked == NSCancelButton) ? NSCancelButton :  NSOKButton];
 }
 
+- (void) _pickedPrinter: (id)sender
+{
+  NSPrintInfo* info = [[NSPrintOperation currentOperation] printInfo]; 
+  NSString *name = [sender titleOfSelectedItem];
+  NSPrinter *printer = [NSPrinter printerWithName: name];
+ 
+  [info setPrinter: printer];
+  [self updateFromPrintInfo];
+}
+
 - (void) _pickedPage: (id)sender
 {
   id pageMatrix = CONTROL(self, NSPPPageChoiceMatrix);
@@ -383,7 +391,23 @@ static NSPrintPanel *shared_instance;
   dict = [info dictionary];
 
   /* Setup printer information */
-  [CONTROL(self, NSPPNameField) setStringValue: [printer name] ];
+  {
+    NSArray *printerNames = [NSPrinter printerNames];
+    NSInteger i;
+    NSPopUpButton *button = CONTROL(self, NSPPNameField);
+    [button removeAllItems];
+    
+    for (i=0; i<[printerNames count]; i++)
+      {
+      NSString *printerName = [printerNames objectAtIndex: i];
+      [button addItemWithTitle: printerName];
+      if ([[printer name] isEqual: printerName])
+	{
+	  [button selectItemAtIndex: i];
+	}
+    }
+  }
+
   [CONTROL(self, NSPPNoteField) setStringValue: [printer note] ];
   [CONTROL(self, NSPPStatusField) setStringValue: @"Idle" ];
 
