@@ -50,14 +50,15 @@
 #import "AppKit/NSImage.h"
 #import "AppKit/NSImageView.h"
 #import "AppKit/NSMatrix.h"
+#import "AppKit/NSMenu.h"
 #import "AppKit/NSPasteboard.h"
 #import "AppKit/NSDragging.h"
 #import "AppKit/NSSavePanel.h"
 #import "AppKit/NSTextField.h"
 #import "AppKit/NSWorkspace.h"
 
-#include "GSGuiPrivate.h"
-#include "GNUstepGUI/GSTheme.h"
+#import "GSGuiPrivate.h"
+#import "GNUstepGUI/GSTheme.h"
 
 #define _SAVE_PANEL_X_PAD	5
 #define _SAVE_PANEL_Y_PAD	4
@@ -137,7 +138,7 @@ setPath(NSBrowser *browser, NSString *path)
       return NSDragOperationNone;
     }
 
-  return NSDragOperationAll;
+  return NSDragOperationEvery;
 }
     
 - (BOOL) performDragOperation: (id<NSDraggingInfo>)sender
@@ -180,6 +181,7 @@ setPath(NSBrowser *browser, NSString *path)
   NSBox *bar;
   NSButton *button;
   NSImage *image;
+  NSImageView *imageView;
   NSRect r;
   id lastKeyView;
 
@@ -194,9 +196,12 @@ setPath(NSBrowser *browser, NSString *path)
   // minimum size we want it to have.  Then, we resize it at the 
   // comfortable size of (384, 426).
   //
-  [super initWithContentRect: NSMakeRect (100, 100, 308, 317)
-	 styleMask: (NSTitledWindowMask | NSResizableWindowMask) 
-	 backing: 2 defer: YES];
+  self = [super initWithContentRect: NSMakeRect (100, 100, 308, 317)
+                          styleMask: (NSTitledWindowMask | NSResizableWindowMask) 
+                            backing: 2 defer: YES];
+  if (nil == self)
+    return nil;
+
   [self setMinSize: [self frame].size];
 
   r = NSMakeRect (0, 0, 308, 317);
@@ -231,6 +236,14 @@ setPath(NSBrowser *browser, NSString *path)
   [_browser setMinColumnWidth: 140];
   [_topView addSubview: _browser];
   [_browser release];
+  
+  _showsHiddenFilesMenu = [[NSMenu alloc] initWithTitle: @""];
+  [_showsHiddenFilesMenu insertItemWithTitle: _(@"Show Hidden Files") action:@selector(_toggleShowsHiddenFiles:) keyEquivalent:@"" atIndex:0];
+  [[_showsHiddenFilesMenu itemAtIndex: 0] setTarget: self];
+  [[_showsHiddenFilesMenu itemAtIndex: 0] setState: [self showsHiddenFiles]];
+  [_browser setMenu: _showsHiddenFilesMenu];
+  [_showsHiddenFilesMenu release];
+
 
   r = NSMakeRect (8, 39, 291, 21);
   _form = [NSForm new];
@@ -333,17 +346,13 @@ setPath(NSBrowser *browser, NSString *path)
   [_browser setTarget: _okButton];
 
   r = NSMakeRect (8, 261, 48, 48);
-  button = [[NSButton alloc] initWithFrame: r]; 
   image = [[NSApplication sharedApplication] applicationIconImage];
-  [button setImage: image];
-  [button setBordered: NO];
-  [button setEnabled: NO];
-  [[button cell] setImageDimsWhenDisabled: NO];
-  [button setImagePosition: NSImageOnly];
-  [button setAutoresizingMask: NSViewMinYMargin];
-  [button setTag: NSFileHandlingPanelImageButton];
-  [_topView addSubview: button];
-  [button release];
+  imageView = [[NSImageView alloc] initWithFrame: r];
+  [imageView setAutoresizingMask: NSViewMinYMargin];
+  [imageView setImage:image];
+  [imageView setTag: NSFileHandlingPanelImageButton];
+  [_topView addSubview: imageView];
+  [imageView release];
 
   r = NSMakeRect (67, 276, 200, 14);
   _titleField = [[NSTextField alloc] initWithFrame: r]; 
@@ -376,6 +385,12 @@ setPath(NSBrowser *browser, NSString *path)
   return self;
 }
 
+- (void) _toggleShowsHiddenFiles: (id)sender
+{
+  NSMenuItem *menuItem = (NSMenuItem*)sender;
+  [self setShowsHiddenFiles: ![menuItem state]];
+}
+
 - (void) _getOriginalSize
 {
   /* Used in setMinSize: */
@@ -697,7 +712,9 @@ selectCellWithString: (NSString*)title
 // complicated initializations, you get a simple panel from super.
 -(id) init
 {
-  [self _initWithoutGModel];
+  self = [self _initWithoutGModel];
+  if (nil == self)
+    return nil;
 
 /*
  * All these are set automatically  
@@ -1434,6 +1451,21 @@ selectCellWithString: (NSString*)title
   _isExtensionHidden = flag;
 }
 
+- (BOOL) showsHiddenFiles
+{
+  return _showsHiddenFiles;
+}
+
+- (void) setShowsHiddenFiles: (BOOL) flag
+{
+  if (flag != _showsHiddenFiles)
+    {
+      _showsHiddenFiles = flag;
+      [[_showsHiddenFilesMenu itemAtIndex: 0] setState: flag];
+      [self _reloadBrowser];
+    }
+}
+
 - (BOOL) isExpanded
 {
   // FIXME
@@ -1446,6 +1478,8 @@ selectCellWithString: (NSString*)title
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
   self = [super initWithCoder: aDecoder];
+  if (nil == self)
+    return nil;
 
   // TODO
   return self;
@@ -1631,8 +1665,10 @@ createRowsForColumn: (int)column
   {
     NSString *h;
     NSArray *hiddenFiles = nil;
-    BOOL gsSavePanelHideDotFiles;
     
+    // FIXME: Use NSFileManager to tell us what files are hidden/non-hidden
+    // rather than having it hardcoded here
+
     if ([files containsObject: @".hidden"] == YES)
       {
 	/* We need to remove files listed in the xxx/.hidden file.  */
@@ -1641,19 +1677,13 @@ createRowsForColumn: (int)column
 	hiddenFiles = [h componentsSeparatedByString: @"\n"];
       }
 
-    /* We need to remove files starting with `.' (dot), but only if
-       the user asked for it in the defaults.  Perhaps we could add a
-       button turning on/off display of hidden files ?  */
+    /* Alse remove files starting with `.' (dot) */
 
-    /* NB: GWorkspace is using this same user default to determine
-       whether to hide or not dot files.  */
-    gsSavePanelHideDotFiles = [[NSUserDefaults standardUserDefaults]
-				boolForKey: @"GSFileBrowserHideDotFiles"];
-    
     /* Now copy the files array into a mutable array - but only if
        strictly needed.  */
-    if (hiddenFiles != nil  ||  gsSavePanelHideDotFiles)
+    if (!_showsHiddenFiles)
       {
+	int j;
 	/* We must make a mutable copy of the array because the API
 	   says that NSFileManager -directoryContentsAtPath: return a
 	   NSArray, not a NSMutableArray, so we shouldn't expect it to
@@ -1666,22 +1696,20 @@ createRowsForColumn: (int)column
 	    [mutableFiles removeObjectsInArray: hiddenFiles];
 	  }
 	
-	if (gsSavePanelHideDotFiles)
+	
+	/* Don't use i which is unsigned.  */
+	j = [mutableFiles count] - 1;
+	
+	while (j >= 0)
 	  {
-	    /* Don't use i which is unsigned.  */
-	    int j = [mutableFiles count] - 1;
+	    NSString *file = (NSString *)[mutableFiles objectAtIndex: j];
 	    
-	    while (j >= 0)
+	    if ([file hasPrefix: @"."])
 	      {
-		NSString *file = (NSString *)[mutableFiles objectAtIndex: j];
-		
-		if ([file hasPrefix: @"."])
-		  {
-		    /* NSLog (@"Removing dot file %@", file); */
-		    [mutableFiles removeObjectAtIndex: j];
-		  }
-		j--;
+		/* NSLog (@"Removing dot file %@", file); */
+		[mutableFiles removeObjectAtIndex: j];
 	      }
+	    j--;
 	  }
 	
 	files = mutableFiles;
