@@ -30,26 +30,26 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSUserDefaults.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSUserDefaults.h>
 
-#include "AppKit/NSColor.h"
-#include "AppKit/NSColorList.h"
-#include "AppKit/NSCell.h"
-#include "AppKit/NSClipView.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/NSInterfaceStyle.h"
-#include "AppKit/NSRulerView.h"
-#include "AppKit/NSScroller.h"
-#include "AppKit/NSScrollView.h"
-#include "AppKit/NSTableHeaderView.h"
-#include "AppKit/NSTableView.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/PSOperators.h"
-#include "GNUstepGUI/GSTheme.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSColorList.h"
+#import "AppKit/NSCell.h"
+#import "AppKit/NSClipView.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSInterfaceStyle.h"
+#import "AppKit/NSRulerView.h"
+#import "AppKit/NSScroller.h"
+#import "AppKit/NSScrollView.h"
+#import "AppKit/NSTableHeaderView.h"
+#import "AppKit/NSTableView.h"
+#import "AppKit/NSWindow.h"
+#import "AppKit/PSOperators.h"
+#import "GNUstepGUI/GSTheme.h"
 
 @interface NSClipView (Private)
 - (void) _scrollToPoint: (NSPoint)aPoint;
@@ -61,7 +61,7 @@
 //
 typedef struct _scrollViewFlags 
 {
-#ifdef WORDS_BIGENDIAN
+#if GS_WORDS_BIGENDIAN == 1
   unsigned int __unused6:14;
   unsigned int __unused5:1;
   unsigned int autohidesScrollers:1;
@@ -368,7 +368,7 @@ static float scrollerWidth;
 
           [self setVerticalScroller: scroller];
           RELEASE(scroller);
-          if (_contentView && !_contentView->_rFlags.flipped_view)
+          if (_contentView && ![_contentView isFlipped])
             [_vertScroller setFloatValue: 1];
         }
       [self addSubview: _vertScroller];
@@ -379,11 +379,20 @@ static float scrollerWidth;
   [self tile];
 }
 
+/**
+ <p> Return wether scroller autohiding is set or not. </p>
+ <p>See Also: -setAutohidesScrollers:</p>
+*/
 - (BOOL) autohidesScrollers
 {
   return _autohidesScrollers;
 }
 
+/**
+ <p>Sets whether the view hides the scrollers (horizontal and/or vertical independendently) if they are not needed.</p>
+ <p>If the content fits inside the clip view on the X or Y axis or both, the respective scroller is removed and additional space is gained.</p>
+ <p>See Also: -autohidesScrollers</p>
+ */
 - (void) setAutohidesScrollers: (BOOL)flag
 {
   _autohidesScrollers = flag;
@@ -439,14 +448,14 @@ static float scrollerWidth;
       amount = - _vLineScroll * deltaY;
     }
 
-  if (_contentView != nil && !_contentView->_rFlags.flipped_view)
+  if (_contentView != nil && ![_contentView isFlipped])
     {
       /* If view is flipped reverse the scroll direction */
       amount = -amount;
     }
   NSDebugLLog (@"NSScrollView", 
     @"increment/decrement: amount = %f, flipped = %d",
-    amount, _contentView ? _contentView->_rFlags.flipped_view : 0);
+    amount, _contentView ? [_contentView isFlipped] : 0);
 
   point.y = clipViewBounds.origin.y + amount;
 
@@ -503,7 +512,7 @@ static float scrollerWidth;
     }
   point = clipViewBounds.origin;
   amount = _vLineScroll;
-  if (_contentView != nil && !_contentView->_rFlags.flipped_view)
+  if (_contentView != nil && ![_contentView isFlipped])
     {
       amount = -amount;
     }
@@ -531,7 +540,7 @@ static float scrollerWidth;
     }
   point = clipViewBounds.origin;
   amount = _vLineScroll;
-  if (_contentView != nil && !_contentView->_rFlags.flipped_view)
+  if (_contentView != nil && ![_contentView isFlipped])
     {
       amount = -amount;
     }
@@ -575,7 +584,7 @@ static float scrollerWidth;
   amount = clipViewBounds.size.height - _vPageScroll;
   amount = (amount < 0) ? 0 : amount;
 
-  if (_contentView != nil && !_contentView->_rFlags.flipped_view)
+  if (_contentView != nil && ![_contentView isFlipped])
     {
       amount = -amount;
     }
@@ -618,7 +627,7 @@ static float scrollerWidth;
    */
   amount = clipViewBounds.size.height - _vPageScroll;
   amount = (amount < 0) ? 0 : amount;
-  if (_contentView != nil && !_contentView->_rFlags.flipped_view)
+  if (_contentView != nil && ![_contentView isFlipped])
     {
       amount = -amount;
     }
@@ -701,14 +710,14 @@ static float scrollerWidth;
         }
       else
         {
-          if (_contentView != nil && !_contentView->_rFlags.flipped_view)
+          if (_contentView != nil && ![_contentView isFlipped])
             {
               /* If view is flipped reverse the scroll direction */
               amount = -amount;
             }
           NSDebugLLog (@"NSScrollView", 
                        @"increment/decrement: amount = %f, flipped = %d",
-            amount, _contentView ? _contentView->_rFlags.flipped_view : 0);
+            amount, _contentView ? [_contentView isFlipped] : 0);
           point.y = clipViewBounds.origin.y + amount;
         }
     }
@@ -722,7 +731,7 @@ static float scrollerWidth;
         }
       else
         {
-          if (_contentView != nil && !_contentView->_rFlags.flipped_view)
+          if (_contentView != nil && ![_contentView isFlipped])
             floatValue = 1 - floatValue;
           point.y = floatValue * (documentRect.size.height
             - clipViewBounds.size.height);
@@ -751,7 +760,7 @@ static float scrollerWidth;
       documentRect = [_contentView documentRect];
     }
   point = documentRect.origin;
-  if (_contentView != nil && !_contentView->_rFlags.flipped_view)
+  if (_contentView != nil && ![_contentView isFlipped])
     {
       point.y = NSMaxY(documentRect) - NSHeight(clipViewBounds);
       if (point.y < 0)
@@ -776,7 +785,7 @@ static float scrollerWidth;
       documentRect = [_contentView documentRect];
     }
   point = documentRect.origin;
-  if (_contentView == nil || _contentView->_rFlags.flipped_view)
+  if (_contentView == nil || [_contentView isFlipped])
     {
       point.y = NSMaxY(documentRect) - NSHeight(clipViewBounds);
       if (point.y < 0)
@@ -848,7 +857,7 @@ static float scrollerWidth;
           floatValue = (clipViewBounds.origin.y - documentFrame.origin.y)
             / (documentFrame.size.height - clipViewBounds.size.height);
 
-          if (!_contentView->_rFlags.flipped_view)
+          if (![_contentView isFlipped])
             {
               floatValue = 1 - floatValue;
             }
@@ -1075,7 +1084,7 @@ static float scrollerWidth;
     }
 
   /* Determine edge positions.  */
-  if (_rFlags.flipped_view)
+  if ([self isFlipped])
     {
       topEdge = NSMinYEdge;
       bottomEdge = NSMaxYEdge;
@@ -1207,87 +1216,8 @@ static float scrollerWidth;
 
 - (void) drawRect: (NSRect)rect
 {
-  NSGraphicsContext *ctxt = GSCurrentContext();
-  GSTheme	*theme = [GSTheme theme];
-  NSColor	*color;
-  NSString	*name;
-  BOOL hasInnerBorder = ![[NSUserDefaults standardUserDefaults]
-			   boolForKey: @"GSScrollViewNoInnerBorder"];
-
-  name = [theme nameForElement: self];
-  if (name == nil)
-    {
-      name = @"NSScrollView";
-    }
-  color = [theme colorNamed: name state: GSThemeNormalState];
-  if (color == nil)
-    {
-      color = [NSColor controlDarkShadowColor];
-    }
-  
-  switch (_borderType)
-    {
-      case NSNoBorder:
-        break;
-
-      case NSLineBorder:
-        [color set];
-        NSFrameRect(_bounds);
-        break;
-
-      case NSBezelBorder:
-        [theme drawGrayBezel: _bounds withClip: rect];
-        break;
-
-      case NSGrooveBorder:
-        [theme drawGroove: _bounds withClip: rect];
-        break;
-    }
-
-  if (hasInnerBorder)
-    {
-      [color set];
-      DPSsetlinewidth(ctxt, 1);
-
-      if (_hasVertScroller)
-	{
-	  NSInterfaceStyle style;
-
-	  style = NSInterfaceStyleForKey(@"NSScrollViewInterfaceStyle", nil);
-	  if (style == NSMacintoshInterfaceStyle
-	      || style == NSWindows95InterfaceStyle)
-	    {
-	      DPSmoveto(ctxt, [_vertScroller frame].origin.x - 1, 
-			[_vertScroller frame].origin.y - 1);
-	    }
-	  else
-	    {
-	      DPSmoveto(ctxt, [_vertScroller frame].origin.x + scrollerWidth, 
-			[_vertScroller frame].origin.y - 1);
-	    }
-	  DPSrlineto(ctxt, 0, [_vertScroller frame].size.height + 1);
-	  DPSstroke(ctxt);
-	}
-
-      if (_hasHorizScroller)
-	{
-	  float ypos;
-	  float scrollerY = [_horizScroller frame].origin.y;
-
-	  if (_rFlags.flipped_view)
-	    {
-	      ypos = scrollerY - 1;
-	    }
-	  else
-	    {
-	      ypos = scrollerY + scrollerWidth + 1;
-	    }
-
-	  DPSmoveto(ctxt, [_horizScroller frame].origin.x - 1, ypos);
-	  DPSrlineto(ctxt, [_horizScroller frame].size.width + 1, 0);
-	  DPSstroke(ctxt);
-	}
-    }
+   [[GSTheme theme] drawScrollViewRect: rect
+		    inView: self];
 }
 
 - (NSRect) documentVisibleRect
@@ -1328,7 +1258,7 @@ static float scrollerWidth;
 {
   [_contentView setDocumentView: aView];
 
-  if (_contentView && !_contentView->_rFlags.flipped_view)
+  if (_contentView && ![_contentView isFlipped])
     {
       [_vertScroller setFloatValue: 1];
     }
@@ -1548,7 +1478,7 @@ static float scrollerWidth;
   else
     {
       [aCoder encodeObject: _contentView];
-      [aCoder encodeValueOfObjCType: @encode(NSBorderType) at: &_borderType];
+      [aCoder encodeValueOfObjCType: @encode(int) at: &_borderType];
       [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_scrollsDynamically];
       [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_rulersVisible];
       [aCoder encodeValueOfObjCType: @encode(float) at: &_hLineScroll];
@@ -1679,7 +1609,7 @@ static float scrollerWidth;
       int version = [aDecoder versionForClassName: @"NSScrollView"];
       NSDebugLLog(@"NSScrollView", @"NSScrollView: start decoding\n");
       _contentView = [aDecoder decodeObject];
-      [aDecoder decodeValueOfObjCType: @encode(NSBorderType) at: &_borderType];
+      [aDecoder decodeValueOfObjCType: @encode(int) at: &_borderType];
       [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_scrollsDynamically];
       [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_rulersVisible];
       [aDecoder decodeValueOfObjCType: @encode(float) at: &_hLineScroll];
