@@ -264,14 +264,16 @@ static int __documentTag = 0;
 //
 - init
 {
-  NSArray *userLanguages = [NSUserDefaults userLanguages];  
+  NSArray *languages;
 
   self = [super init];
   if (self == nil)
     return nil;
 
+  languages = [[NSUserDefaults standardUserDefaults]
+    stringArrayForKey: @"NSLanguages"];
   // Set the language to the default for the user.
-  _language = [userLanguages objectAtIndex: 0];
+  _language = RETAIN([languages objectAtIndex: 0]);
   _wrapFlag = NO;
   _position = 0;
   _spellPanel = nil;
@@ -294,6 +296,7 @@ static int __documentTag = 0;
 
 - (void)dealloc
 {
+  RELEASE(_language);
   RELEASE(_ignoredWords);
   RELEASE(_serverProxy);
   [super dealloc];
@@ -325,15 +328,14 @@ static int __documentTag = 0;
 		 language:(NSString *)language
 {
   int count = 0;
-  NSRange r = NSMakeRange(0,0);
   id<NSSpellServerPrivateProtocol> proxy = [self _serverProxy];
 
   if (proxy != nil)
-    r = [proxy _findMisspelledWordInString: aString
-	       language: _language
-	       ignoredWords: nil
-	       wordCount: &count
-	       countOnly: YES];
+    [proxy _findMisspelledWordInString: aString
+                              language: _language
+                          ignoredWords: nil
+                             wordCount: &count
+                             countOnly: YES];
 
   return count;
 }
@@ -342,7 +344,7 @@ static int __documentTag = 0;
 		      startingAt:(int)startingOffset
 {
   int wordCount = 0;
-  NSRange r = NSMakeRange(0,0);
+  NSRange r;
 
   r = [self checkSpellingOfString: stringToCheck
   	               startingAt: startingOffset
@@ -539,7 +541,7 @@ inSpellDocumentWithTag:(int)tag
   [self _populateAccessoryView];
 }
 
-- _findNext: (id)sender
+- (void) _findNext: (id)sender
 {
   BOOL processed = [[[NSApp mainWindow] firstResponder]
 		       tryToPerform: @selector(checkSpelling:)
@@ -549,20 +551,17 @@ inSpellDocumentWithTag:(int)tag
     {
       NSLog(@"No responder found");
     }
-
-  return self;
 }
 
-- _learn: (id)sender
+- (void) _learn: (id)sender
 {
   NSString *word = [_wordField stringValue];
-  BOOL result = NO;
 
   // Call server and record the learned word.
   NS_DURING
     {
-      result = [[self _serverProxy] _learnWord: word
-				    inDictionary: _language];
+      [[self _serverProxy] _learnWord: word
+                         inDictionary: _language];
     }
   NS_HANDLER
     {
@@ -571,21 +570,18 @@ inSpellDocumentWithTag:(int)tag
   NS_ENDHANDLER
 
   [self _findNext: sender];
-
-  return self;
 }
 
-- _forget: (id)sender
+- (void) _forget: (id)sender
 {
   NSString *word = [_wordField stringValue];
-  BOOL result = NO;
 
   // Call the server and remove the word from the learned
   // list.
   NS_DURING
     {
-      result = [[self _serverProxy] _forgetWord: word
-				    inDictionary: _language];
+      [[self _serverProxy] _forgetWord: word
+                          inDictionary: _language];
     }
   NS_HANDLER
     {
@@ -594,11 +590,9 @@ inSpellDocumentWithTag:(int)tag
   NS_ENDHANDLER
 
   [self _findNext: sender];
-
-  return self;
 }
 
-- _ignore: (id)sender
+- (void) _ignore: (id)sender
 {
   BOOL processed = [[[NSApp mainWindow] firstResponder]
 		       tryToPerform: @selector(ignoreSpelling:)
@@ -610,18 +604,15 @@ inSpellDocumentWithTag:(int)tag
     }
 
   [self _findNext: sender];
-
-  return self;
 }
 
-- _guess: (id)sender
+- (void) _guess: (id)sender
 {
   // Fill in the view...
   [self _populateAccessoryView];
-  return self;
 }
 
-- _correct: (id)sender
+- (void) _correct: (id)sender
 {
   BOOL processed = [[[NSApp mainWindow] firstResponder]
 		       tryToPerform: @selector(changeSpelling:)
@@ -632,11 +623,9 @@ inSpellDocumentWithTag:(int)tag
       NSLog(@"No responder found");
     }
   [self _findNext: sender];
-
-  return self;
 }
 
-- _switchDictionary: (id)sender
+- (void) _switchDictionary: (id)sender
 {
   id<NSSpellServerPrivateProtocol> proxy = nil;
   NSString *language = nil;
@@ -658,11 +647,9 @@ inSpellDocumentWithTag:(int)tag
 	  [_dictionaryPulldown selectItemWithTitle: _language];
 	}
     }
-
-  return self;
 }
 
-- _highlightGuess: (id)sender
+- (void) _highlightGuess: (id)sender
 {
   NSString *selectedGuess = nil;
 
@@ -670,8 +657,6 @@ inSpellDocumentWithTag:(int)tag
   [_ignoreButton setEnabled: NO];
   [_guessButton setEnabled: YES];
   [_wordField setStringValue: selectedGuess];
-
-  return self;
 }
 
 - (void) awakeFromNib
@@ -698,6 +683,7 @@ inSpellDocumentWithTag:(int)tag
   [_correctButton setAlternateImage: [NSImage imageNamed: @"common_retH"]];
   [_spellPanel makeFirstResponder: _correctButton];
   [_spellPanel setBecomesKeyOnlyIfNeeded: YES];
+  [_spellPanel setFloatingPanel: YES];
 }
 @end
 
