@@ -32,26 +32,26 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSArray.h>
-#include <Foundation/NSCoder.h>
-#include <Foundation/NSDecimalNumber.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSGeometry.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSRunLoop.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSUserDefaults.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSCoder.h>
+#import <Foundation/NSDecimalNumber.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSGeometry.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSRunLoop.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSUserDefaults.h>
 
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSSplitView.h"
-#include "AppKit/NSWindow.h"
-#include "GSGuiPrivate.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSSplitView.h"
+#import "AppKit/NSWindow.h"
+#import "GSGuiPrivate.h"
 
 static NSNotificationCenter *nc = nil;
 
@@ -402,7 +402,8 @@ static NSNotificationCenter *nc = nil;
 - (void) mouseDown: (NSEvent*)theEvent
 {
   NSApplication *app = [NSApplication sharedApplication];
-  NSPoint p, op;
+  NSPoint p = NSZeroPoint, 
+    op = NSZeroPoint;
   NSEvent *e;
   NSRect r, r1, bigRect, vis;
   id v = nil, prev = nil;
@@ -720,7 +721,6 @@ static NSNotificationCenter *nc = nil;
         {
           [_dividerColor set];
           NSHighlightRect(oldRect);
-          lit = NO;
         }
       [self unlockFocus];
     }
@@ -879,8 +879,7 @@ static NSNotificationCenter *nc = nil;
           newSize = NSMakeSize(NSWidth(_bounds), newHeight);
           newPoint = NSMakePoint(0.0, running);
           running += newHeight + _dividerWidth;
-          [views[i] setFrameSize: newSize];
-          [views[i] setFrameOrigin: newPoint];
+          [views[i] setFrame: NSMakeRect(newPoint.x, newPoint.y, newSize.width, newSize.height)];
         }
     }
   else
@@ -903,8 +902,7 @@ static NSNotificationCenter *nc = nil;
           newSize = NSMakeSize(newWidth, NSHeight(_bounds));
           newPoint = NSMakePoint(running, 0.0);
           running += newWidth + _dividerWidth;
-          [views[i] setFrameSize: newSize];
-          [views[i] setFrameOrigin: newPoint];
+          [views[i] setFrame: NSMakeRect(newPoint.x, newPoint.y, newSize.width, newSize.height)];
         }
     }
 
@@ -957,7 +955,7 @@ static inline NSPoint centerSizeInRect(NSSize innerSize, NSRect outerRect)
    * Images are always drawn with their bottom-left corner at the origin
    * so we must adjust the position to take account of a flipped view.
    */
-  if (_rFlags.flipped_view)
+  if ([self isFlipped])
     {
       dimpleOrigin.y += dimpleSize.height;
     }
