@@ -29,21 +29,22 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <Foundation/NSArray.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSRunLoop.h>
-#include <Foundation/NSUserDefaults.h>
-#include "AppKit/NSTableHeaderCell.h"
-#include "AppKit/NSTableHeaderView.h"
-#include "AppKit/NSTableColumn.h"
-#include "AppKit/NSTableView.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/NSEvent.h"
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSScrollView.h"
-#include "AppKit/NSGraphics.h"
-#include "GSGuiPrivate.h"
+#import <Foundation/NSArray.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSRunLoop.h>
+#import <Foundation/NSUserDefaults.h>
+#import "AppKit/NSTableHeaderCell.h"
+#import "AppKit/NSTableHeaderView.h"
+#import "AppKit/NSTableColumn.h"
+#import "AppKit/NSTableView.h"
+#import "AppKit/NSWindow.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSColor.h"
+#import "AppKit/NSScrollView.h"
+#import "AppKit/NSGraphics.h"
+#import "GSGuiPrivate.h"
+#import "GNUstepGUI/GSTheme.h"
 
 /*
  * Number of pixels in either direction that will be counted as a hit 
@@ -165,91 +166,8 @@
  */
 - (void)drawRect: (NSRect)aRect
 {
-  NSArray *columns;
-  int firstColumnToDraw;
-  int lastColumnToDraw;
-  NSRect drawingRect;
-  NSTableColumn *column;
-  NSTableColumn *highlightedTableColumn;
-  float width;
-  int i;
-  NSCell *cell;
-
-  if (_tableView == nil)
-    return;
-
-  firstColumnToDraw = [self columnAtPoint: NSMakePoint (aRect.origin.x,
-                                                        aRect.origin.y)];
-  if (firstColumnToDraw == -1)
-    firstColumnToDraw = 0;
-
-  lastColumnToDraw = [self columnAtPoint: NSMakePoint (NSMaxX (aRect),
-                                                       aRect.origin.y)];
-  if (lastColumnToDraw == -1)
-    lastColumnToDraw = [_tableView numberOfColumns] - 1;
-
-  drawingRect = [self headerRectOfColumn: firstColumnToDraw];
-  
-  columns = [_tableView tableColumns];
-  highlightedTableColumn = [_tableView highlightedTableColumn];
-  
-  for (i = firstColumnToDraw; i < lastColumnToDraw; i++)
-    {
-      column = [columns objectAtIndex: i];
-      width = [column width];
-      drawingRect.size.width = width;
-      cell = [column headerCell];
-      if ((column == highlightedTableColumn)
-          || [_tableView isColumnSelected: i])
-        {
-          [cell setHighlighted: YES];
-        }
-      else
-        {
-          [cell setHighlighted: NO];
-        }
-      [cell drawWithFrame: drawingRect
-                           inView: self];
-      drawingRect.origin.x += width;
-    }
-  if (lastColumnToDraw == [_tableView numberOfColumns] - 1)
-    {
-      column = [columns objectAtIndex: lastColumnToDraw];
-      width = [column width] - 1;
-      drawingRect.size.width = width;
-      cell = [column headerCell];
-      if ((column == highlightedTableColumn)
-          || [_tableView isColumnSelected: lastColumnToDraw])
-        {
-          [cell setHighlighted: YES];
-        }
-      else
-        {
-          [cell setHighlighted: NO];
-        }
-      [cell drawWithFrame: drawingRect
-                           inView: self];
-      drawingRect.origin.x += width;
-    }
-  else
-    {
-      column = [columns objectAtIndex: lastColumnToDraw];
-      width = [column width];
-      drawingRect.size.width = width;
-      cell = [column headerCell];
-      if ((column == highlightedTableColumn)
-          || [_tableView isColumnSelected: lastColumnToDraw])
-        {
-          [cell setHighlighted: YES];
-        }
-      else
-        {
-          [cell setHighlighted: NO];
-        }
-      [cell drawWithFrame: drawingRect
-                           inView: self];
-      drawingRect.origin.x += width;
-    }
+  [[GSTheme theme] drawTableHeaderRect: aRect
+		   inView: self];
 }
 
 /**
@@ -725,7 +643,6 @@
                 {
                   NSHighlightRect(highlightRect);
                   [_window flushWindow];
-                  lit = NO;
                 }
               [self unlockFocus];
             }
