@@ -32,19 +32,20 @@
 
 #include "config.h"
 
-#include <Foundation/NSFormatter.h>
-#include <Foundation/NSNotification.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSValue.h>
+#import <Foundation/NSFormatter.h>
+#import <Foundation/NSNotification.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSValue.h>
 
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSCursor.h"
-#include "AppKit/NSGraphics.h"
-#include "AppKit/NSTextField.h"
-#include "AppKit/NSTextFieldCell.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/NSKeyValueBinding.h"
-#include "GSBindingHelpers.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSCursor.h"
+#import "AppKit/NSEvent.h"
+#import "AppKit/NSGraphics.h"
+#import "AppKit/NSTextField.h"
+#import "AppKit/NSTextFieldCell.h"
+#import "AppKit/NSWindow.h"
+#import "AppKit/NSKeyValueBinding.h"
+#import "GSBindingHelpers.h"
 
 static NSNotificationCenter *nc;
 
@@ -179,6 +180,24 @@ static Class textFieldCellClass;
              so we need to call it before setting up the _text_object */
           length = [[self stringValue] length];
           _text_object = [_cell setUpFieldEditorAttributes: text];
+	  if (sender == self)
+	  {
+	    /* Special case: If -selectText: is called in response to making
+	       the receiver first responder with a mouse click, don't select
+	       the whole text. First of all, this is redundant since the field
+	       editor will receive the mouse click and immediately collapse the
+	       selection by placing the insertion point at the mouse location.
+	       Furthermore, we thus preserve the current selection on X and
+	       therefore the user can easily paste it into the text field with
+	       a middle mouse click. */
+	    NSEvent *event = [_window currentEvent];
+	    if ([event type] == NSLeftMouseDown &&
+		NSPointInRect([event locationInWindow],
+			      [self convertRect:[self bounds] toView:nil]))
+	      {
+		length = 0;
+	      }
+	  }
           [_cell selectWithFrame: _bounds
                  inView: self
                  editor: _text_object
@@ -769,13 +788,17 @@ static Class textFieldCellClass;
 {
   if ([binding hasPrefix: NSEditableBinding])
     {
+      GSKeyValueBinding *kvb;
+
       [self unbind: binding];
-      [[GSKeyValueAndBinding alloc] initWithBinding: NSEditableBinding 
-                                    withName: binding 
-                                    toObject: anObject
-                                    withKeyPath: keyPath
-                                    options: options
-                                    fromObject: self];
+      kvb = [[GSKeyValueAndBinding alloc] initWithBinding: NSEditableBinding 
+                                                 withName: binding 
+                                                 toObject: anObject
+                                              withKeyPath: keyPath
+                                                  options: options
+                                               fromObject: self];
+      // The binding will be retained in the binding table
+      RELEASE(kvb);
     }
   else
     {
