@@ -599,6 +599,10 @@ static GSValidationCenter *vc = nil;
 {
   GSToolbarCustomizationPalette *palette;
 
+  if (![self allowsUserCustomization])
+    {
+      return;
+    }
   if (_customizationPaletteIsRunning)
     {
       NSLog(@"Customization palette is already running for toolbar: %@", self);
@@ -907,7 +911,7 @@ static GSValidationCenter *vc = nil;
 
       displayMode = (NSToolbarDisplayMode)[[config objectForKey: @"displayMode"] intValue];
       [self setDisplayMode: displayMode];
-      sizeMode = (NSToolbarDisplayMode)[[config objectForKey: @"sizeMode"] intValue];
+      sizeMode = (NSToolbarSizeMode)[[config objectForKey: @"sizeMode"] intValue];
       [self setSizeMode: sizeMode];
     }
 
@@ -1083,7 +1087,7 @@ static GSValidationCenter *vc = nil;
   NSArray *linked;
   id toolbar;
   
-  linked = [isa _toolbarsWithIdentifier: [self identifier]];
+  linked = [[self class] _toolbarsWithIdentifier: [self identifier]];
     
   if (linked != nil && [linked count] > 0)
     {
