@@ -317,7 +317,7 @@ NSString *GSMovableToolbarItemPboardType = @"GSMovableToolbarItemPboardType";
   //nothing to do
 }
 
-- (unsigned int) draggingSourceOperationMaskForLocal: (BOOL)isLocal
+- (NSDragOperation) draggingSourceOperationMaskForLocal: (BOOL)isLocal
 {
   return isLocal ? NSDragOperationGeneric : NSDragOperationNone;
 }
@@ -613,18 +613,21 @@ NSString *GSMovableToolbarItemPboardType = @"GSMovableToolbarItemPboardType";
   if ([view superview] == nil) // Show the view to eventually hide it later
     [self addSubview: view];
 	
-  if([view respondsToSelector:@selector(sizeToFit)])
+  if ([view respondsToSelector: @selector(sizeToFit)])
     {
       NSSize newSize, minSize = [_toolbarItem minSize];
-	  [view performSelector:@selector(sizeToFit)];
-	  newSize = [view frame].size;
-	  if (newSize.width < minSize.width || newSize.height < minSize.height)
-	    {
-		  newSize.width = MAX(newSize.width, minSize.width);
-		  newSize.height = MAX(newSize.height, minSize.height);
-		  [view setFrameSize:newSize];
-		}
+      
+      [(id)view sizeToFit];
+      newSize = [view frame].size;
+      if (minSize.width > 0 || newSize.height < minSize.height)
+        {
+          if (minSize.width > 0)
+			newSize.width = minSize.width;
+          newSize.height = MAX(newSize.height, minSize.height);
+          [view setFrameSize: newSize];
+        }
     }
+
   // Adjust the layout in accordance with NSToolbarSizeMode
   switch ([[_toolbarItem toolbar] sizeMode])
     {
@@ -805,7 +808,7 @@ NSString *GSMovableToolbarItemPboardType = @"GSMovableToolbarItemPboardType";
   //nothing to do
 }
 
-- (unsigned int) draggingSourceOperationMaskForLocal: (BOOL)isLocal
+- (NSDragOperation) draggingSourceOperationMaskForLocal: (BOOL)isLocal
 {
   return isLocal ? NSDragOperationGeneric : NSDragOperationNone;
 }
@@ -1384,8 +1387,10 @@ NSString *GSMovableToolbarItemPboardType = @"GSMovableToolbarItemPboardType";
       NSSize size;
 
       size = [view frame].size;
-      [self setMinSize: size];
-      [self setMaxSize: size];
+      if (NSEqualSizes(NSZeroSize, _minSize))
+          [self setMinSize: size];
+      if (NSEqualSizes(NSZeroSize, _maxSize))
+          [self setMaxSize: size];
 
       [_view setToolTip: _toolTip];
 
@@ -1495,7 +1500,7 @@ NSString *GSMovableToolbarItemPboardType = @"GSMovableToolbarItemPboardType";
 
 - (BOOL) _isFlexibleSpace
 {
-  return [self isKindOfClass:[GSToolbarFlexibleSpaceItem class]];
+  return NO;
 }
 
 - (BOOL) _selectable
@@ -1576,22 +1581,36 @@ NSString *GSMovableToolbarItemPboardType = @"GSMovableToolbarItemPboardType";
 {
   NSToolbarItem *new = [[NSToolbarItem allocWithZone: zone] 
     initWithItemIdentifier: _itemIdentifier];
+  NSString *toolTip;
+  NSImage *image;
+  NSString *label;
+  NSMenuItem *item;
 
   // Copy all items individually...
   [new setTarget: [self target]];
   [new setAction: [self action]];
-  [new setToolTip: [[self toolTip] copyWithZone: zone]];
+
+  toolTip = [[self toolTip] copyWithZone: zone];
+  [new setToolTip: toolTip];
+  RELEASE(toolTip);
   [new setTag: [self tag]];
-  [new setImage: [[self image] copyWithZone: zone]];
+  image = [[self image] copyWithZone: zone];
+  [new setImage: image];
+  RELEASE(image);
   [new setEnabled: [self isEnabled]];
-  [new setPaletteLabel: [[self paletteLabel] copyWithZone: zone]];
-  [new setLabel: [[self label] copyWithZone: zone]];
+  label = [[self paletteLabel] copyWithZone: zone];
+  [new setPaletteLabel: label];
+  RELEASE(label);
+  label = [[self label] copyWithZone: zone];
+  [new setLabel: label];
+  RELEASE(label);
   [new setMinSize: [self minSize]];
   [new setMaxSize: [self maxSize]];
   [new setAutovalidates: [self autovalidates]];
   [new setVisibilityPriority: [self visibilityPriority]];
-  [new setMenuFormRepresentation: [[self menuFormRepresentation] 
-                                      copyWithZone: zone]];
+  item = [[self menuFormRepresentation] copyWithZone: zone];
+  [new setMenuFormRepresentation: item];
+  RELEASE(item);
 
   if ([self view] != nil)
     {
