@@ -2,7 +2,7 @@
 
    <abstract>Workspace class</abstract>
 
-   Copyright (C) 1996-1999, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1996-2010 Free Software Foundation, Inc.
 
    Author: Scott Christley <scottc@net-community.com>
    Date: 1996
@@ -30,9 +30,14 @@
    Boston, MA 02110-1301, USA.
 */ 
 
-#include "config.h"
+#import "config.h"
 
-#if	defined(HAVE_GETMNTENT) && defined (MNT_MEMB)
+#if	defined(HAVE_GETMNTINFO)
+#include <unistd.h>
+#include <sys/types.h>
+#include <sys/param.h>
+#include <sys/mount.h>
+#elif	defined(HAVE_GETMNTENT) && defined (MNT_MEMB)
 #if	defined(HAVE_MNTENT_H)
 #include <mntent.h>
 #elif defined(HAVE_SYS_MNTENT_H)
@@ -42,37 +47,37 @@
 #endif
 #endif
 
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSData.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSHost.h>
-#include <Foundation/NSLock.h>
-#include <Foundation/NSDistributedLock.h>
-#include <Foundation/NSPathUtilities.h>
-#include <Foundation/NSUserDefaults.h>
-#include <Foundation/NSTask.h>
-#include <GNUstepBase/NSTask+GNUstepBase.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSFileManager.h>
-#include <Foundation/NSNotificationQueue.h>
-#include <Foundation/NSDistributedNotificationCenter.h>
-#include <Foundation/NSConnection.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSProcessInfo.h>
-#include <Foundation/NSThread.h>
-#include <Foundation/NSURL.h>
-#include <Foundation/NSValue.h>
-#include "AppKit/NSWorkspace.h"
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSPasteboard.h"
-#include "AppKit/NSView.h"
-#include "AppKit/NSPanel.h"
-#include "AppKit/NSWindow.h"
-#include "AppKit/NSScreen.h"
-#include "GNUstepGUI/GSServicesManager.h"
-#include "GNUstepGUI/GSDisplayServer.h"
-#include "GSGuiPrivate.h"
+#import <Foundation/NSBundle.h>
+#import <Foundation/NSData.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSHost.h>
+#import <Foundation/NSLock.h>
+#import <Foundation/NSDistributedLock.h>
+#import <Foundation/NSPathUtilities.h>
+#import <Foundation/NSUserDefaults.h>
+#import <Foundation/NSTask.h>
+#import <GNUstepBase/NSTask+GNUstepBase.h>
+#import <Foundation/NSException.h>
+#import <Foundation/NSFileManager.h>
+#import <Foundation/NSNotificationQueue.h>
+#import <Foundation/NSDistributedNotificationCenter.h>
+#import <Foundation/NSConnection.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSProcessInfo.h>
+#import <Foundation/NSThread.h>
+#import <Foundation/NSURL.h>
+#import <Foundation/NSValue.h>
+#import "AppKit/NSWorkspace.h"
+#import "AppKit/NSApplication.h"
+#import "AppKit/NSImage.h"
+#import "AppKit/NSPasteboard.h"
+#import "AppKit/NSView.h"
+#import "AppKit/NSPanel.h"
+#import "AppKit/NSWindow.h"
+#import "AppKit/NSScreen.h"
+#import "GNUstepGUI/GSServicesManager.h"
+#import "GNUstepGUI/GSDisplayServer.h"
+#import "GSGuiPrivate.h"
 
 /* Informal protocol for method to ask an app to open a URL.
  */
@@ -88,15 +93,18 @@
 
 #define PosixExecutePermission	(0111)
 
+static NSMutableDictionary *folderPathIconDict = nil;
+static NSMutableDictionary *folderIconCache = nil;
+
 static NSImage	*folderImage = nil;
-static NSImage	*homeImage = nil;
 static NSImage	*multipleFiles = nil;
-static NSImage	*rootImage = nil;
 static NSImage	*unknownApplication = nil;
 static NSImage	*unknownTool = nil;
 
 
 static NSString	*GSWorkspaceNotification = @"GSWorkspaceNotification";
+static NSString *GSWorkspacePreferencesChanged =
+    @"GSWorkspacePreferencesChanged";
 
 /*
  * Depending on the 'active' flag this returns either the currently
@@ -400,6 +408,10 @@ static id GSLaunched(NSNotification *notification, BOOL active)
 - (BOOL) _extension: (NSString*)ext
                role: (NSString*)role
 	        app: (NSString**)app;
+- (BOOL) _scheme: (NSString*)scheme
+	    role: (NSString*)role
+	     app: (NSString**)app;
+- (void) _workspacePreferencesChanged: (NSNotification *)aNotification;
 
 // application communication
 - (BOOL) _launchApplication: (NSString*)appName
@@ -512,6 +524,9 @@ static NSDictionary		*applications = nil;
 
 static NSString			*extPrefPath = nil;
 static NSDictionary		*extPreferences = nil;
+
+static NSString			*urlPrefPath = nil;
+static NSDictionary		*urlPreferences = nil;
 // FIXME: Won't work for MINGW32
 static NSString			*_rootPath = @"/";
 
@@ -562,11 +577,28 @@ static NSString			*_rootPath = @"/";
 		}
 	    }
 	  
+	  /*
+	   *	Load URL scheme preferences.
+	   */
+	  urlPrefPath = [service
+	    stringByAppendingPathComponent: @".GNUstepURLPrefs"];
+	  RETAIN(urlPrefPath);
+	  if ([mgr isReadableFileAtPath: urlPrefPath] == YES)
+	    {
+	      data = [NSData dataWithContentsOfFile: urlPrefPath];
+	      if (data)
+		{
+		  dict = [NSDeserializer deserializePropertyListFromData: data
+					 mutableContainers: NO];
+		  urlPreferences = RETAIN(dict);
+		}
+	    }
+	  
 	  /*
 	   *	Load cached application information.
 	   */
 	  appListPath = [service
-			  stringByAppendingPathComponent: @".GNUstepAppList"];
+	    stringByAppendingPathComponent: @".GNUstepAppList"];
 	  RETAIN(appListPath);
 	  if ([mgr isReadableFileAtPath: appListPath] == YES)
 	    {
@@ -628,6 +660,14 @@ static NSString			*_rootPath = @"/";
 
 - (id) init
 {
+  NSArray *documentDir;
+  NSArray *libraryDirs;
+  NSArray *sysAppDir;
+  NSArray *downloadDir;
+  NSArray *desktopDir;
+  NSString *sysDir;
+  int i;
+
   if (sharedWorkspace != self)
     {
       RELEASE(self);
@@ -640,6 +680,15 @@ static NSString			*_rootPath = @"/";
     name: NSUserDefaultsDidChangeNotification
     object: nil];
 
+  /* There's currently no way of knowing if things have changed due to
+   * apps being installed etc ... so we actually poll regularly.
+   */
+  [[NSNotificationCenter defaultCenter]
+    addObserver: self
+    selector: @selector(_workspacePreferencesChanged:)
+    name: @"GSHousekeeping"
+    object: nil];
+
   _workspaceCenter = [_GSWorkspaceCenter new];
   _iconMap = [NSMutableDictionary new];
   _launched = [NSMutableDictionary new];
@@ -647,6 +696,68 @@ static NSString			*_rootPath = @"/";
     {
       [self findApplications];
     }
+  [_workspaceCenter
+    addObserver: self
+    selector: @selector(_workspacePreferencesChanged:)
+    name: GSWorkspacePreferencesChanged
+    object: nil];
+
+  /* icon association and caching */
+  folderPathIconDict = [[NSMutableDictionary alloc] initWithCapacity:5];
+
+  documentDir = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,
+    NSUserDomainMask, YES);
+  downloadDir = NSSearchPathForDirectoriesInDomains(NSDownloadsDirectory,
+    NSUserDomainMask, YES);
+  desktopDir = NSSearchPathForDirectoriesInDomains(NSDesktopDirectory,
+    NSUserDomainMask, YES);
+  libraryDirs = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory,
+    NSAllDomainsMask, YES);
+  sysAppDir = NSSearchPathForDirectoriesInDomains(NSApplicationDirectory,
+    NSSystemDomainMask, YES);
+ 
+  /* we try to guess a System directory and check if looks like one */
+  sysDir = nil;
+  if ([sysAppDir count] > 0)
+    {
+      sysDir = [[sysAppDir objectAtIndex: 0] stringByDeletingLastPathComponent];
+      if (![[sysDir lastPathComponent] isEqualToString: @"System"])
+	sysDir = nil;
+    }
+
+  if (sysDir != nil)
+    [folderPathIconDict setObject: @"GSFolder" forKey: sysDir];
+
+  [folderPathIconDict setObject: @"HomeDirectory"
+    forKey: NSHomeDirectory()];
+  [folderPathIconDict setObject: @"ImageFolder"
+    forKey: [NSHomeDirectory () stringByAppendingPathComponent: @"Images"]];
+  [folderPathIconDict setObject: @"MusicFolder"
+    forKey: [NSHomeDirectory () stringByAppendingPathComponent: @"Music"]];
+  /* it would be nice to use different root icons... */
+  [folderPathIconDict setObject: @"Root_PC" forKey: _rootPath];
+
+  for (i = 0; i < [libraryDirs count]; i++)
+    {
+      [folderPathIconDict setObject: @"LibraryFolder"
+	forKey: [libraryDirs objectAtIndex: i]];
+    }
+  for (i = 0; i < [documentDir count]; i++)
+    {
+      [folderPathIconDict setObject: @"DocsFolder"
+	forKey: [documentDir objectAtIndex: i]];
+    }
+  for (i = 0; i < [downloadDir count]; i++)
+    {
+      [folderPathIconDict setObject: @"DownloadFolder"
+	forKey: [downloadDir objectAtIndex: i]];
+    }
+  for (i = 0; i < [desktopDir count]; i++)
+    {
+      [folderPathIconDict setObject: @"Desktop"
+	forKey: [desktopDir objectAtIndex: i]];
+    }
+  folderIconCache = [[NSMutableDictionary alloc] init];
 
   return self;
 }
@@ -811,18 +922,10 @@ static NSString			*_rootPath = @"/";
     }
   else
     {
-      NSDictionary	*map;
       NSString		*appName;
       NSPasteboard      *pb;
 
-      /* Look up an application to handle this URL scheme.
-       * We get a dictionary containing all apps for the scheme.
-       * FIXME ... we just use the last app in the dictionary,
-       * but we really ought to have some way of saying which we want.
-       */
-      map = [applications objectForKey: @"GSSchemesMap"];
-      map = [map objectForKey: [[url scheme] lowercaseString]];
-      appName = [[map allKeys] lastObject];
+      appName = [self getBestAppInRole: nil forScheme: [url scheme]];
       if (appName != nil)
 	{
 	  id		app;
@@ -986,8 +1089,49 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 		      description: (NSString **)description
 			     type: (NSString **)fileSystemType
 {
+#if defined (HAVE_GETMNTINFO)
+  /* FIXME Check for presence of statfs call explicitly. Not all systems
+     with getmntinfo do have a statfs calls. In particular, NetBSD offers
+     only a statvfs calls for compatibility with POSIX. Other BSDs and
+     Linuxes have statvfs as well, but this returns less information than
+     the 4.4BSD statfs call. The NetBSD statvfs, on the other hand, is just
+     a statfs in disguise, i.e., it provides all information available in
+     the 4.4BSD statfs call. Therefore, we go ahead an just #define statfs
+     as statvfs on NetBSD.
+     Note that the POSIX statvfs is not really helpful for us here. The
+     only information that could be extracted from the data returned by
+     that syscall is the ST_RDONLY flag. There is no owner field nor a
+     typename.
+     The statvfs call on Solaris returns a structure that includes a
+     non-standard f_basetype field, which provides the name of the
+     underlying file system type.
+  */
+#if defined (__NetBSD__) && __NetBSD_Version__ >= 300000000 
+#define statfs statvfs
+#define f_flags f_flag
+#endif
+  uid_t uid;
+  struct statfs m;
+  NSStringEncoding enc;
+
+  if (statfs([fullPath fileSystemRepresentation], &m))
+    return NO;
+
+  uid = geteuid();
+  enc = [NSString defaultCStringEncoding];
+  *removableFlag = NO; // FIXME
+  *writableFlag = (m.f_flags & MNT_RDONLY) == 0;
+  *unmountableFlag =
+    (m.f_flags & MNT_ROOTFS) == 0 && (uid == 0 || uid == m.f_owner);
+  *description = @"filesystem"; // FIXME
+  *fileSystemType =
+    [[NSString alloc] initWithCString: m.f_fstypename encoding: enc];
+
+  return YES;
+#else
   // FIXME
   return NO;
+#endif
 }
 
 /**
@@ -1103,15 +1247,12 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
   fileType = [attributes objectForKey: NSFileType];
   if ([fileType isEqual: NSFileTypeDirectory] == YES)
     {
-      NSString		*iconPath = nil;
-      BOOL		isApplication = NO;
+      NSString *iconPath = nil;
       
       if ([pathExtension isEqualToString: @"app"]
 	|| [pathExtension isEqualToString: @"debug"]
 	|| [pathExtension isEqualToString: @"profile"])
 	{
-          isApplication = YES;
-	  
 	  image = [self appIconForApp: fullPath];
 	  
 	  if (image == nil)
@@ -1151,35 +1292,33 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 	  image = [self _iconForExtension: pathExtension];
 	  if (image == nil || image == [self unknownFiletypeImage])
 	    {
-	      if ([fullPath isEqual: _rootPath])
-		{
-		  if (rootImage == nil)
-		    {
-		      rootImage = RETAIN([NSImage _standardImageWithName:
-			@"Root_PC"]);
-		    }
+	      NSString *iconName;
 
-		  image = rootImage;
-		}
-	      else if ([fullPath isEqual: NSHomeDirectory ()])
+	      iconName = [folderPathIconDict objectForKey: fullPath];
+	      if (iconName != nil)
 		{
-		  if (homeImage == nil)
+		  NSImage *iconImage;
+
+		  iconImage = [folderIconCache objectForKey: iconName];
+		  if (iconImage == nil)
 		    {
-		      homeImage = RETAIN([NSImage _standardImageWithName:
-			@"HomeDirectory"]);
+		      iconImage = [NSImage _standardImageWithName: iconName];
+		      /* the dictionary retains the image */
+		      [folderIconCache setObject: iconImage forKey: iconName];
 		    }
-		  image = homeImage;
+		  image = iconImage;
 		}
 	      else
 		{
 		  if (folderImage == nil)
 		    {
 		      folderImage = RETAIN([NSImage _standardImageWithName:
-			@"Folder"]);
+						      @"Folder"]);
 		    }
 		  image = folderImage;
 		}
 	    }
+
 	}
     }
   else
@@ -1312,9 +1451,6 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 - (void) findApplications
 {
   static NSString	*path = nil;
-  NSFileManager		*mgr = [NSFileManager defaultManager];
-  NSData		*data;
-  NSDictionary		*dict;
   NSTask		*task;
 
   /*
@@ -1330,31 +1466,9 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
     {
       [task waitUntilExit];
     }
-  if ([mgr isReadableFileAtPath: extPrefPath] == YES)
-    {
-      data = [NSData dataWithContentsOfFile: extPrefPath];
-      if (data)
-	{
-	  dict = [NSDeserializer deserializePropertyListFromData: data
-					       mutableContainers: NO];
-	  ASSIGN(extPreferences, dict);
-	}
-    }
-
-  if ([mgr isReadableFileAtPath: appListPath] == YES)
-    {
-      data = [NSData dataWithContentsOfFile: appListPath];
-      if (data)
-	{
-	  dict = [NSDeserializer deserializePropertyListFromData: data
-					       mutableContainers: NO];
-	  ASSIGN(applications, dict);
-	}
-    }
-  /*
-   *	Invalidate the cache of icons for file extensions.
-   */
-  [_iconMap removeAllObjects];
+  [self _workspacePreferencesChanged:
+     [NSNotification notificationWithName: GSWorkspacePreferencesChanged
+				   object: self]];
 }
 
 /**
@@ -1678,9 +1792,37 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
       NSZoneFree(NSDefaultMallocZone(), base);
     }
 
+#elif defined (HAVE_GETMNTINFO)
+  NSFileManager	*mgr = [NSFileManager defaultManager];
+  unsigned int	i, n;
+  struct statfs	*m;
+
+  n = getmntinfo(&m, MNT_NOWAIT);
+  names = [NSMutableArray arrayWithCapacity: n];
+  for (i = 0; i < n; i++)
+    {
+      /* NB For now assume that all local volumes are mounted from a device
+         with an entry /dev and this is not the case for any pseudo
+	 filesystems.
+      */
+      if (strncmp(m[i].f_mntfromname, "/dev/", 5) == 0)
+        {
+	  [names addObject:
+		   [mgr stringWithFileSystemRepresentation: m[i].f_mntonname
+			length: strlen(m[i].f_mntonname)]];
+        }
+    }
 #elif	defined(HAVE_GETMNTENT) && defined (MNT_MEMB)
-  NSFileManager		*mgr = [NSFileManager defaultManager];
-  FILE		*fptr = fopen("/etc/mtab", "r");
+  /* FIXME Solaris uses /etc/mnttab instead of /etc/mtab, but defines
+   * MNTTAB to that path.
+   * FIXME We won't get here on Solaris at all because it defines the
+   * mntent struct in sys/mnttab.h instead of sys/mntent.h.
+   */
+# ifndef MNTTAB
+#  define MNTTAB "/etc/mtab"
+# endif
+  NSFileManager	*mgr = [NSFileManager defaultManager];
+  FILE		*fptr = fopen(MNTTAB, "r");
   struct mntent	*m;
 
 
@@ -1708,7 +1850,7 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
   
   // get reserved names....
   reservedMountNames = [[NSUserDefaults standardUserDefaults] objectForKey: @"GSReservedMountNames"];
-  if(reservedMountNames == nil)
+  if (reservedMountNames == nil)
     {
       reservedMountNames = [NSArray arrayWithObjects: 
 				      @"proc",@"devpts",
@@ -1720,7 +1862,7 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 					     forKey: @"GSReservedMountNames"];
     }
 
-  mtab = [NSString stringWithContentsOfFile:mtabPath];
+  mtab = [NSString stringWithContentsOfFile: mtabPath];
   mounts = [mtab componentsSeparatedByString: @"\n"];
 
   names = [NSMutableArray arrayWithCapacity: [mounts count]];
@@ -1945,7 +2087,7 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
     }
   
   iconPath = [[bundle infoDictionary] objectForKey: @"NSIcon"];
-  if(iconPath == nil)
+  if (iconPath == nil)
     {
       /*
        * Try the CFBundleIconFile property.
@@ -2106,7 +2248,15 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
   RELEASE(extPreferences);
   extPreferences = map;
   data = [NSSerializer serializePropertyList: extPreferences];
-  [data writeToFile: extPrefPath atomically: YES];
+  if ([data writeToFile: extPrefPath atomically: YES])
+    {
+      [_workspaceCenter postNotificationName: GSWorkspacePreferencesChanged
+				      object: self];
+    }
+  else
+    {
+      NSLog(@"Update %@ of failed", extPrefPath);
+    }
 }
 
 /**
@@ -2137,7 +2287,111 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
   RELEASE(extPreferences);
   extPreferences = map;
   data = [NSSerializer serializePropertyList: extPreferences];
-  [data writeToFile: extPrefPath atomically: YES];
+  if ([data writeToFile: extPrefPath atomically: YES])
+    {
+      [_workspaceCenter postNotificationName: GSWorkspacePreferencesChanged
+				      object: self];
+    }
+  else
+    {
+      NSLog(@"Update %@ of failed", extPrefPath);
+    }
+}
+
+/**
+ * Gets the applications cache (generated by the make_services tool)
+ * and looks up the special entry that contains a dictionary of all
+ * URL schemes recognised by GNUstep applications.  Then finds the
+ * dictionary of applications that can handle our scheme and returns
+ * it.
+ */
+- (NSDictionary*) infoForScheme: (NSString*)scheme
+{
+  NSDictionary  *map;
+
+  scheme = [scheme lowercaseString];
+  map = [applications objectForKey: @"GSSchemesMap"];
+  return [map objectForKey: scheme];
+}
+
+/**
+ * Returns the 'best' application to open a file with the specified URL
+ * scheme using the given role.  If the role is nil then apps which can
+ * edit are preferred but viewers are also acceptable.  Uses a user preferred
+ * app or picks any good match.
+ */
+- (NSString*) getBestAppInRole: (NSString*)role
+		     forScheme: (NSString*)scheme
+{
+  NSString	*appName = nil;
+
+  if ([self _scheme: scheme role: role app: &appName] == NO)
+    {
+      appName = nil;
+    }
+  return appName;
+}
+
+/**
+ * Sets up a user preference for which app should be used to open files
+ * of the specified URL scheme
+ */
+- (void) setBestApp: (NSString*)appName
+	     inRole: (NSString*)role
+	  forScheme: (NSString*)scheme
+{
+  NSMutableDictionary	*map;
+  NSMutableDictionary	*inf;
+  NSData		*data;
+
+  scheme = [scheme lowercaseString];
+  if (urlPreferences != nil)
+    map = [urlPreferences mutableCopy];
+  else
+    map = [NSMutableDictionary new];
+
+  inf = [[map objectForKey: scheme] mutableCopy];
+  if (inf == nil)
+    {
+      inf = [NSMutableDictionary new];
+    }
+  if (appName == nil)
+    {
+      if (role == nil)
+	{
+	  NSString	*iconPath = [inf objectForKey: @"Icon"];
+
+	  RETAIN(iconPath);
+	  [inf removeAllObjects];
+	  if (iconPath)
+	    {
+	      [inf setObject: iconPath forKey: @"Icon"];
+	      RELEASE(iconPath);
+	    }
+	}
+      else
+	{
+	  [inf removeObjectForKey: role];
+	}
+    }
+  else
+    {
+      [inf setObject: appName forKey: (role ? (id)role : (id)@"Editor")];
+    }
+  [map setObject: inf forKey: scheme];
+  RELEASE(inf);
+  RELEASE(urlPreferences);
+  urlPreferences = map;
+  data = [NSSerializer serializePropertyList: urlPreferences];
+  if ([data writeToFile: urlPrefPath atomically: YES])
+    {
+      [_workspaceCenter postNotificationName: GSWorkspacePreferencesChanged
+				      object: self];
+    }
+  else
+    {
+      NSLog(@"Update %@ of failed", urlPrefPath);
+    }
 }
 
 @end
@@ -2149,20 +2403,20 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
   NSDictionary	*typeInfo = [extInfo objectForKey: appName];
   NSString	*file = [typeInfo objectForKey: @"NSIcon"];
   
-  //
-  // If the NSIcon entry isn't there and the CFBundle entries are,
-  // get the first icon in the list if it's an array, or assign
-  // the icon to file if it's a string.
-  //
-  // FIXME: CFBundleTypeExtensions/IconFile can be arrays which assign
-  // multiple types to icons.  This needs to be handled eventually.
-  //
-  if(file == nil)
+  /*
+   * If the NSIcon entry isn't there and the CFBundle entries are,
+   * get the first icon in the list if it's an array, or assign
+   * the icon to file if it's a string.
+   *
+   * FIXME: CFBundleTypeExtensions/IconFile can be arrays which assign
+   * multiple types to icons.  This needs to be handled eventually.
+   */
+  if (file == nil)
     {
       id icon = [typeInfo objectForKey: @"CFBundleTypeIconFile"];
-      if([icon isKindOfClass: [NSArray class]])
+      if ([icon isKindOfClass: [NSArray class]])
 	{
-	  if([icon count])
+	  if ([icon count])
 	    {
 	      file = [icon objectAtIndex: 0];
 	    }
@@ -2245,8 +2499,9 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 
   absolute = [[NSURL fileURLWithPath: [file stringByStandardizingPath]] 
 		 absoluteString];
-  // This compensates for a feature we have in NSURL, that is there to have 
-  // MacOSX compatibility.
+  /* This compensates for a feature we have in NSURL, that is there to have 
+   * MacOSX compatibility.
+   */
   if ([absolute hasPrefix:  @"file://localhost/"])
     {
       absolute = [@"file:///" stringByAppendingString: 
@@ -2302,12 +2557,12 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 	   */
 	  if (prefs)
 	    {
-	      if ((appName = [extInfo objectForKey: @"Editor"]) != nil)
+	      if ((appName = [prefs objectForKey: @"Editor"]) != nil)
 		{
 		  icon = [self _extIconForApp: appName info: extInfo];
 		}
 	      if (icon == nil
-		&& (appName = [extInfo objectForKey: @"Viewer"]) != nil)
+		&& (appName = [prefs objectForKey: @"Viewer"]) != nil)
 		{
 		  icon = [self _extIconForApp: appName info: extInfo];
 		}
@@ -2507,6 +2762,200 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
     }
 }
 
+- (BOOL) _scheme: (NSString*)scheme
+	    role: (NSString*)role
+	     app: (NSString**)app
+{
+  NSEnumerator	*enumerator;
+  NSString      *appName = nil;
+  NSDictionary	*apps = [self infoForScheme: scheme];
+  NSDictionary	*prefs;
+  NSDictionary	*info;
+
+  scheme = [scheme lowercaseString];
+
+  /*
+   *	Look for the name of the preferred app in this role.
+   *	A 'nil' roll is a wildcard - find the preferred Editor or Viewer.
+   */
+  prefs = [urlPreferences objectForKey: scheme];
+  if (role == nil || [role isEqualToString: @"Editor"])
+    {
+      appName = [prefs objectForKey: @"Editor"];
+      if (appName != nil)
+	{
+	  info = [apps objectForKey: appName];
+	  if (info != nil)
+	    {
+	      if (app != 0)
+		{
+		  *app = appName;
+		}
+	      return YES;
+	    }
+	  else if ([self locateApplicationBinary: appName] != nil)
+	    {
+	      /*
+	       * Return the preferred application even though it doesn't
+	       * say it opens this type of file ... preferences overrule.
+	       */
+	      if (app != 0)
+		{
+		  *app = appName;
+		}
+	      return YES;
+	    }
+	}
+    }
+  if (role == nil || [role isEqualToString: @"Viewer"])
+    {
+      appName = [prefs objectForKey: @"Viewer"];
+      if (appName != nil)
+	{
+	  info = [apps objectForKey: appName];
+	  if (info != nil)
+	    {
+	      if (app != 0)
+		{
+		  *app = appName;
+		}
+	      return YES;
+	    }
+	  else if ([self locateApplicationBinary: appName] != nil)
+	    {
+	      /*
+	       * Return the preferred application even though it doesn't
+	       * say it opens this type of file ... preferences overrule.
+	       */
+	      if (app != 0)
+		{
+		  *app = appName;
+		}
+	      return YES;
+	    }
+	}
+    }
+
+  /*
+   * Go through the dictionary of apps that know about this file type and
+   * determine the best application to open the file by examining the
+   * type information for each app.
+   * The 'NSRole' field specifies what the app can do with the file - if it
+   * is missing, we assume an 'Editor' role.
+   */
+  if (apps == nil || [apps count] == 0)
+    {
+      return NO;
+    }
+  enumerator = [apps keyEnumerator];
+
+  if (role == nil)
+    {
+      BOOL	found = NO;
+
+      /*
+       * If the requested role is 'nil', we can accept an app that is either
+       * an Editor (preferred) or a Viewer, or unknown.
+       */
+      while ((appName = [enumerator nextObject]) != nil)
+	{
+	  NSString	*str;
+
+	  info = [apps objectForKey: appName];
+	  str = [info objectForKey: @"NSRole"];
+	  /* NB. If str is nil or an empty string, there is no role set,
+	   * and we treat this as an Editor since the role is unrestricted.
+	   */
+	  if ([str length] == 0 || [str isEqualToString: @"Editor"])
+	    {
+	      if (app != 0)
+		{
+		  *app = appName;
+		}
+	      return YES;
+	    }
+	  if ([str isEqualToString: @"Viewer"])
+	    {
+	      if (app != 0)
+		{
+		  *app = appName;
+		}
+	      found = YES;
+	    }
+	}
+      return found;
+    }
+  else
+    {
+      while ((appName = [enumerator nextObject]) != nil)
+	{
+	  NSString	*str;
+
+	  info = [apps objectForKey: appName];
+	  str = [info objectForKey: @"NSRole"];
+	  if ((str == nil && [role isEqualToString: @"Editor"])
+	    || [str isEqualToString: role])
+	    {
+	      if (app != 0)
+		{
+		  *app = appName;
+		}
+	      return YES;
+	    }
+	}
+      return NO;
+    }
+}
+
+- (void) _workspacePreferencesChanged: (NSNotification *)aNotification
+{
+  /* FIXME reload only those preferences that really were changed
+   * TODO  add a user info to aNotification, which includes a bitmask
+   *       denoting the updated preference files.
+   */
+  NSFileManager		*mgr = [NSFileManager defaultManager];
+  NSData		*data;
+  NSDictionary		*dict;
+
+  if ([mgr isReadableFileAtPath: extPrefPath] == YES)
+    {
+      data = [NSData dataWithContentsOfFile: extPrefPath];
+      if (data)
+	{
+	  dict = [NSDeserializer deserializePropertyListFromData: data
+					       mutableContainers: NO];
+	  ASSIGN(extPreferences, dict);
+	}
+    }
+
+  if ([mgr isReadableFileAtPath: urlPrefPath] == YES)
+    {
+      data = [NSData dataWithContentsOfFile: urlPrefPath];
+      if (data)
+	{
+	  dict = [NSDeserializer deserializePropertyListFromData: data
+					       mutableContainers: NO];
+	  ASSIGN(urlPreferences, dict);
+	}
+    }
+
+  if ([mgr isReadableFileAtPath: appListPath] == YES)
+    {
+      data = [NSData dataWithContentsOfFile: appListPath];
+      if (data)
+	{
+	  dict = [NSDeserializer deserializePropertyListFromData: data
+					       mutableContainers: NO];
+	  ASSIGN(applications, dict);
+	}
+    }
+  /*
+   *	Invalidate the cache of icons for file extensions.
+   */
+  [_iconMap removeAllObjects];
+}
+
+
 /**
  * Launch an application locally (ie without reference to the workspace
  * manager application).  We should only call this method when we want
