@@ -27,9 +27,9 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "config.h"
-#include <Foundation/NSString.h>
-#include "AppKit/NSEvent.h"
+#import "config.h"
+#import <Foundation/NSString.h>
+#import "AppKit/NSEvent.h"
 
 // Global strings
 NSString *NSModalPanelRunLoopMode = @"ModalPanelMode";
@@ -149,11 +149,18 @@ NSString *NSSystemColorsDidChangeNotification =
             @"NSSystemColorsDidChangeNotification";
 
 // NSColorList notifications
-NSString *NSColorListChangedNotification = @"NSColorListChange";
+NSString *NSColorListDidChangeNotification = @"NSColorListDidChange";
 
 // NSColorPanel notifications
-NSString *NSColorPanelColorChangedNotification =
-@"NSColorPanelColorChangedNotification";
+NSString *NSColorPanelColorDidChangeNotification =
+  @"NSColorPanelColorDidChange";
+
+#if GNUSTEP_GUI_MAJOR_VERSION == 0 && GNUSTEP_GUI_MINOR_VERSION < 20
+/* The above notifications had been misspelled in GNUstep. We keep them around
+   here to preserve binary compatibility until the next release. */
+NSString *NSColorListChangedNotification = @"NSColorListDidChange";
+NSString *NSColorPanelColorChangedNotification = @"NSColorPanelColorDidChange";
+#endif
 
 // NSComboBox notifications
 NSString *NSComboBoxWillPopUpNotification = 
@@ -500,6 +507,7 @@ NSString *NSParagraphStyleAttributeName = @"NSParagraphStyleAttributeName";
 NSString *NSShadowAttributeName = @"NSShadowAttributeName";
 NSString *NSStrikethroughColorAttributeName
   = @"NSStrikethroughColorAttributeName";
+NSString *NSStrikethroughStyleAttributeName = @"NSStrikethroughStyleAttributeName";
 NSString *NSStrokeColorAttributeName = @"NSStrokeColorAttributeName";
 NSString *NSStrokeWidthAttributeName = @"NSStrokeWidthAttributeName";
 NSString *NSSuperscriptAttributeName = @"NSSuperscriptAttributeName";
@@ -539,6 +547,11 @@ NSString *NSModificationTimeDocumentAttribute = @"ModificationTime";
 
 const unsigned NSUnderlineByWordMask = 0x01;
 
+NSString *NSSpellingStateAttributeName = @"NSSpellingState";
+const unsigned NSSpellingStateSpellingFlag = 1;
+const unsigned NSSpellingStateGrammarFlag = 2;
+
+
 NSString *NSPlainTextDocumentType = @"PlainText";
 NSString *NSRTFTextDocumentType = @"RTF";
 NSString *NSRTFDTextDocumentType = @"RTFD";
@@ -646,8 +659,11 @@ NSString *NSContentObjectBinding = @"contentObject";
 NSString *NSEditableBinding = @"editable";
 NSString *NSEnabledBinding = @"enabled";
 NSString *NSFontBinding = @"font";
+NSString *NSFontNameBinding = @"fontName";
+NSString *NSFontSizeBinding = @"fontSize";
 NSString *NSHiddenBinding = @"hidden";
 NSString *NSSelectedIndexBinding = @"selectedIndex";
+NSString *NSSelectedTagBinding = @"selectedTag";
 NSString *NSTextColorBinding = @"textColor";
 NSString *NSTitleBinding = @"title";
 NSString *NSToolTipBinding = @"toolTip";
