@@ -3,7 +3,7 @@
 
    Functions for dealing with tiff images.
 
-   Copyright (C) 1996,1999 Free Software Foundation, Inc.
+   Copyright (C) 1996,1999-2010 Free Software Foundation, Inc.
    
    Author:  Adam Fedor <fedor@colorado.edu>
    Date: Feb 1996
@@ -59,12 +59,12 @@
 
 #include "config.h"
 #include "nsimage-tiff.h"
-#include <Foundation/NSArray.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSString.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSEnumerator.h>
-#include "GSGuiPrivate.h"
+#import <Foundation/NSArray.h>
+#import <Foundation/NSDebug.h>
+#import <Foundation/NSString.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSEnumerator.h>
+#import "GSGuiPrivate.h"
 
 #include <math.h>
 #include <stdlib.h>
@@ -125,7 +125,7 @@ TiffHandleWrite(thandle_t handle, tdata_t buf, tsize_t count)
   if (chand->position + count > chand->size)
     {
       chand->size = chand->position + count + 1;
-      chand->data = objc_realloc(chand->data, chand->size);
+      chand->data = realloc(chand->data, chand->size);
       *(chand->outdata) = chand->data;
       if (chand->data == NULL)
 	return 0;
@@ -161,7 +161,7 @@ TiffHandleClose(thandle_t handle)
   chandle_t* chand = (chandle_t *)handle;
 
   /* Presumably, we don't need the handle anymore */
-  OBJC_FREE(chand);
+  free(chand);
   return 0;
 }
 
@@ -202,7 +202,7 @@ NSTiffOpenDataRead(const char* data, long size)
       TIFFSetWarningHandler(NSTiffWarning);
     }
 
-  OBJC_MALLOC(handle, chandle_t, 1);
+  handle = malloc(sizeof(chandle_t));
   handle->data = (char*)data;
   handle->outdata = 0;
   handle->position = 0;
@@ -221,7 +221,7 @@ TIFF*
 NSTiffOpenDataWrite(char **data, long *size)
 {
   chandle_t* handle;
-  OBJC_MALLOC(handle, chandle_t, 1);
+  handle = malloc(sizeof(chandle_t));
   handle->data = *data;
   handle->outdata = data;
   handle->position = 0;
@@ -269,7 +269,7 @@ NSTiffGetInfo(int imageNumber, TIFF* image)
   if (image == NULL)
     return NULL;
 
-  OBJC_MALLOC(info, NSTiffInfo, 1);
+  info = malloc(sizeof(NSTiffInfo));
   memset(info, 0, sizeof(NSTiffInfo));
   if (imageNumber >= 0)
     {
@@ -341,7 +341,6 @@ NSTiffRead(TIFF *image, NSTiffInfo *info, unsigned char *data)
 {
   int     i;
   unsigned int row, col;
-  int     maxval;
   int	  error = 0;
   uint8* outP;
   uint8* buf;
@@ -360,7 +359,6 @@ NSTiffRead(TIFF *image, NSTiffInfo *info, unsigned char *data)
 	return -1;
     }
 
-  maxval = (1 << info->bitsPerSample) - 1;
   scan_line_size = TIFFScanlineSize(image);
   buf = _TIFFmalloc(scan_line_size);
   
@@ -453,6 +451,7 @@ NSTiffWrite(TIFF *image, NSTiffInfo *info, unsigned char *data)
   int		i;
   unsigned int 	row;
   int           error = 0;
+  int           scan_line_size;
 
   TIFFSetField(image, TIFFTAG_IMAGEWIDTH, info->width);
   TIFFSetField(image, TIFFTAG_IMAGELENGTH, info->height);
@@ -470,6 +469,7 @@ NSTiffWrite(TIFF *image, NSTiffInfo *info, unsigned char *data)
   else
     sample_info[0] = EXTRASAMPLE_UNASSALPHA;
   TIFFSetField(image, TIFFTAG_EXTRASAMPLES, info->extraSamples, sample_info);
+  scan_line_size = TIFFScanlineSize(image);
 
   switch (info->photoInterp) 
     {
@@ -477,24 +477,20 @@ NSTiffWrite(TIFF *image, NSTiffInfo *info, unsigned char *data)
       case PHOTOMETRIC_MINISWHITE:
 	if (info->planarConfig == PLANARCONFIG_CONTIG) 
 	  {
-	    int	line = ceil((float)info->width * info->bitsPerSample / 8.0);
-
 	    for (row = 0; row < info->height; ++row) 
 	      {
 		WRITE_SCANLINE(0)
-		buf += line;
+		buf += scan_line_size;
 	      }
 	  } 
 	else 
 	  {
-	    int	line = ceil((float)info->width / 8.0);
-
 	    for (i = 0; i < info->samplesPerPixel; i++)
 	      {
 		for (row = 0; row < info->height; ++row) 
 		  {
 		    WRITE_SCANLINE(i)
-		    buf += line;
+		    buf += scan_line_size;
 		  }
 	      }
 	  }
@@ -506,7 +502,7 @@ NSTiffWrite(TIFF *image, NSTiffInfo *info, unsigned char *data)
 	    for (row = 0; row < info->height; ++row) 
 	      {
 		WRITE_SCANLINE(0)
-		buf += info->width * info->samplesPerPixel;
+		buf += scan_line_size;
 	      }
 	  } 
 	else 
@@ -516,7 +512,7 @@ NSTiffWrite(TIFF *image, NSTiffInfo *info, unsigned char *data)
 		for (row = 0; row < info->height; ++row) 
 		  {
 		    WRITE_SCANLINE(i)
-		    buf += info->width;
+		    buf += scan_line_size;
 		  }
 	      }
 	  }
@@ -571,14 +567,14 @@ NSTiffGetColormap(TIFF* image)
   if (info->photoInterp != PHOTOMETRIC_PALETTE)
     return NULL;
 
-  OBJC_MALLOC(map, NSTiffColormap, 1);
+  map = malloc(sizeof(NSTiffColormap));
   map->size = 1 << info->bitsPerSample;
 
   if (!TIFFGetField(image, TIFFTAG_COLORMAP,
 		    &map->red, &map->green, &map->blue)) 
     {
       TIFFError(TIFFFileName(image), "Missing required \"Colormap\" tag");
-      OBJC_FREE(map);
+      free(map);
       return NULL;
     }
   if (CheckAndCorrectColormap(map) == 8)
